@@ -1,22 +1,21 @@
-//! A module which contains configuration options for a [`Grid`].
-//!
-//! [`Grid`]: crate::grid::iterable::Grid
+//! A module which contains configuration options for a grid.
 
 mod borders_config;
 mod entity_map;
-mod offset;
 
 use std::collections::HashMap;
 
 use crate::ansi::{ANSIBuf, ANSIStr};
 use crate::config::compact::CompactConfig;
-use crate::config::Formatting;
 use crate::config::{
-    AlignmentHorizontal, AlignmentVertical, Border, Borders, Entity, Indent, Position, Sides,
+    AlignmentHorizontal, AlignmentVertical, Border, Borders, Entity, Indent, Offset, Position,
+    Sides,
 };
 use borders_config::BordersConfig;
 
-pub use self::{entity_map::EntityMap, offset::Offset};
+pub use self::entity_map::EntityMap;
+
+use super::Formatting;
 
 /// HorizontalLine represents a horizontal border line.
 type HorizontalLine = super::HorizontalLine<char>;
@@ -29,11 +28,14 @@ type VerticalLine = super::VerticalLine<char>;
 /// grid: crate::Grid.
 #[derive(Debug, PartialEq, Eq, Clone)]
 pub struct SpannedConfig {
-    margin: Sides<ColoredMarginIndent>,
-    padding: EntityMap<Sides<ColoredIndent>>,
+    margin: Sides<MarginIndent>,
+    padding: EntityMap<Sides<Indent>>,
+    padding_color: EntityMap<Sides<Option<ANSIBuf>>>,
     alignment_h: EntityMap<AlignmentHorizontal>,
     alignment_v: EntityMap<AlignmentVertical>,
-    formatting: EntityMap<Formatting>,
+    formatting_trim_h: EntityMap<bool>,
+    formatting_trim_v: EntityMap<bool>,
+    formatting_line_alignment: EntityMap<bool>,
     span_columns: HashMap<Position, usize>,
     span_rows: HashMap<Position, usize>,
     borders: BordersConfig<char>,
@@ -52,7 +54,10 @@ impl Default for SpannedConfig {
         Self {
             margin: Sides::default(),
             padding: EntityMap::default(),
-            formatting: EntityMap::default(),
+            padding_color: EntityMap::default(),
+            formatting_trim_h: EntityMap::default(),
+            formatting_trim_v: EntityMap::default(),
+            formatting_line_alignment: EntityMap::default(),
             alignment_h: EntityMap::new(AlignmentHorizontal::Left),
             alignment_v: EntityMap::new(AlignmentVertical::Top),
             span_columns: HashMap::default(),
@@ -71,6 +76,11 @@ impl Default for SpannedConfig {
 }
 
 impl SpannedConfig {
+    /// Creates a new spanned config.
+    pub fn new() -> Self {
+        Self::default()
+    }
+
     /// Set a margin of a grid.
     pub fn set_margin(&mut self, margin: Sides<Indent>) {
         self.margin.left.indent = margin.left;
@@ -106,12 +116,12 @@ impl SpannedConfig {
     }
 
     /// Returns a margin color value currently set.
-    pub fn get_margin_color(&self) -> Sides<Option<ANSIBuf>> {
+    pub fn get_margin_color(&self) -> Sides<Option<&ANSIBuf>> {
         Sides::new(
-            self.margin.left.color.clone(),
-            self.margin.right.color.clone(),
-            self.margin.top.color.clone(),
-            self.margin.bottom.color.clone(),
+            self.margin.left.color.as_ref(),
+            self.margin.right.color.as_ref(),
+            self.margin.top.color.as_ref(),
+            self.margin.bottom.color.as_ref(),
         )
     }
 
@@ -245,7 +255,7 @@ impl SpannedConfig {
     ///
     /// It takes not cell position but line as row and column of a cell;
     /// So its range is line <= count_rows && col < count_columns.
-    pub fn set_horizontal_char(&mut self, pos: Position, c: char, offset: Offset) {
+    pub fn set_horizontal_char(&mut self, pos: Position, offset: Offset, c: char) {
         let chars = self
             .horizontal_chars
             .entry(pos)
@@ -262,7 +272,7 @@ impl SpannedConfig {
         self.horizontal_chars
             .get(&pos)
             .and_then(|chars| {
-                chars.get(&Offset::Begin(offset)).or_else(|| {
+                chars.get(&Offset::Start(offset)).or_else(|| {
                     if end > offset {
                         if end == 0 {
                             chars.get(&Offset::End(0))
@@ -299,7 +309,7 @@ impl SpannedConfig {
     ///
     /// It takes not cell position but cell row and column of a line;
     /// So its range is row < count_rows && col <= count_columns.
-    pub fn set_vertical_char(&mut self, pos: Position, c: char, offset: Offset) {
+    pub fn set_vertical_char(&mut self, pos: Position, offset: Offset, c: char) {
         let chars = self
             .vertical_chars
             .entry(pos)
@@ -316,7 +326,7 @@ impl SpannedConfig {
         self.vertical_chars
             .get(&pos)
             .and_then(|chars| {
-                chars.get(&Offset::Begin(offset)).or_else(|| {
+                chars.get(&Offset::Start(offset)).or_else(|| {
                     if end > offset {
                         if end == 0 {
                             chars.get(&Offset::End(0))
@@ -348,7 +358,7 @@ impl SpannedConfig {
     }
 
     /// Override a character color on a horizontal line.
-    pub fn set_horizontal_color(&mut self, pos: Position, c: ANSIBuf, offset: Offset) {
+    pub fn set_horizontal_char_color(&mut self, pos: Position, offset: Offset, c: ANSIBuf) {
         let chars = self
             .horizontal_colors
             .entry(pos)
@@ -365,7 +375,7 @@ impl SpannedConfig {
         end: usize,
     ) -> Option<&ANSIBuf> {
         self.horizontal_colors.get(&pos).and_then(|chars| {
-            chars.get(&Offset::Begin(offset)).or_else(|| {
+            chars.get(&Offset::Start(offset)).or_else(|| {
                 if end > offset {
                     if end == 0 {
                         chars.get(&Offset::End(0))
@@ -380,7 +390,7 @@ impl SpannedConfig {
     }
 
     /// Override a character color on a vertical line.
-    pub fn set_vertical_color(&mut self, pos: Position, c: ANSIBuf, offset: Offset) {
+    pub fn set_vertical_char_color(&mut self, pos: Position, offset: Offset, c: ANSIBuf) {
         let chars = self
             .vertical_colors
             .entry(pos)
@@ -397,7 +407,7 @@ impl SpannedConfig {
         end: usize,
     ) -> Option<&ANSIBuf> {
         self.vertical_colors.get(&pos).and_then(|chars| {
-            chars.get(&Offset::Begin(offset)).or_else(|| {
+            chars.get(&Offset::Start(offset)).or_else(|| {
                 if end > offset {
                     if end == 0 {
                         chars.get(&Offset::End(0))
@@ -413,56 +423,61 @@ impl SpannedConfig {
 
     /// Set a padding to a given cells.
     pub fn set_padding(&mut self, entity: Entity, padding: Sides<Indent>) {
-        let mut pad = self.padding.get(entity).clone();
-        pad.left.indent = padding.left;
-        pad.right.indent = padding.right;
-        pad.top.indent = padding.top;
-        pad.bottom.indent = padding.bottom;
-
-        self.padding.insert(entity, pad);
+        self.padding.insert(entity, padding);
     }
 
     /// Set a padding to a given cells.
     pub fn set_padding_color(&mut self, entity: Entity, padding: Sides<Option<ANSIBuf>>) {
-        let mut pad = self.padding.get(entity).clone();
-        pad.left.color = padding.left;
-        pad.right.color = padding.right;
-        pad.top.color = padding.top;
-        pad.bottom.color = padding.bottom;
-
-        self.padding.insert(entity, pad);
+        self.padding_color.insert(entity, padding);
     }
 
-    /// Get a padding for a given [Entity].
-    pub fn get_padding(&self, entity: Entity) -> Sides<Indent> {
-        let pad = self.padding.get(entity);
-        Sides::new(
-            pad.left.indent,
-            pad.right.indent,
-            pad.top.indent,
-            pad.bottom.indent,
-        )
+    /// Get a padding for a given cell by [Position].
+    pub fn get_padding(&self, pos: Position) -> &Sides<Indent> {
+        self.padding.get(pos)
     }
 
-    /// Get a padding color for a given [Entity].
-    pub fn get_padding_color(&self, entity: Entity) -> Sides<Option<ANSIBuf>> {
-        let pad = self.padding.get(entity);
-        Sides::new(
-            pad.left.color.clone(),
-            pad.right.color.clone(),
-            pad.top.color.clone(),
-            pad.bottom.color.clone(),
-        )
+    /// Get a padding color for a given cell by [Position].
+    pub fn get_padding_color(&self, pos: Position) -> &Sides<Option<ANSIBuf>> {
+        self.padding_color.get(pos)
     }
 
     /// Set a formatting to a given cells.
-    pub fn set_formatting(&mut self, entity: Entity, formatting: Formatting) {
-        self.formatting.insert(entity, formatting);
+    pub fn set_trim_horizontal(&mut self, entity: Entity, on: bool) {
+        self.formatting_trim_h.insert(entity, on);
     }
 
-    /// Get a formatting settings for a given [Entity].
-    pub fn get_formatting(&self, entity: Entity) -> &Formatting {
-        self.formatting.get(entity)
+    /// Get a formatting settings for a given cell by [Position].
+    pub fn get_trim_horizonal(&self, pos: Position) -> bool {
+        *self.formatting_trim_h.get(pos)
+    }
+
+    /// Set a formatting to a given cells.
+    pub fn set_trim_vertical(&mut self, entity: Entity, on: bool) {
+        self.formatting_trim_v.insert(entity, on);
+    }
+
+    /// Get a formatting settings for a given cell by [Position].
+    pub fn get_trim_vertical(&self, pos: Position) -> bool {
+        *self.formatting_trim_v.get(pos)
+    }
+
+    /// Set a formatting to a given cells.
+    pub fn set_line_alignment(&mut self, entity: Entity, on: bool) {
+        self.formatting_line_alignment.insert(entity, on);
+    }
+
+    /// Get a formatting settings for a given cell by [Position].
+    pub fn get_line_alignment(&self, pos: Position) -> bool {
+        *self.formatting_line_alignment.get(pos)
+    }
+
+    /// Get a formatting settings for a given cell by [Position].
+    pub fn get_formatting(&self, pos: Position) -> Formatting {
+        Formatting::new(
+            *self.formatting_trim_h.get(pos),
+            *self.formatting_trim_v.get(pos),
+            *self.formatting_line_alignment.get(pos),
+        )
     }
 
     /// Set a vertical alignment to a given cells.
@@ -470,9 +485,9 @@ impl SpannedConfig {
         self.alignment_v.insert(entity, alignment);
     }
 
-    /// Get a vertical alignment for a given [Entity].
-    pub fn get_alignment_vertical(&self, entity: Entity) -> &AlignmentVertical {
-        self.alignment_v.get(entity)
+    /// Get a vertical alignment for a given cell by [Position].
+    pub fn get_alignment_vertical(&self, pos: Position) -> &AlignmentVertical {
+        self.alignment_v.get(pos)
     }
 
     /// Set a horizontal alignment to a given cells.
@@ -480,9 +495,9 @@ impl SpannedConfig {
         self.alignment_h.insert(entity, alignment);
     }
 
-    /// Get a horizontal alignment for a given [Entity].
-    pub fn get_alignment_horizontal(&self, entity: Entity) -> &AlignmentHorizontal {
-        self.alignment_h.get(entity)
+    /// Get a horizontal alignment for a given cell by [Position].
+    pub fn get_alignment_horizontal(&self, pos: Position) -> &AlignmentHorizontal {
+        self.alignment_h.get(pos)
     }
 
     /// Set border set a border value to all cells in [`Entity`].
@@ -555,15 +570,15 @@ impl SpannedConfig {
     }
 
     /// Get a justification which will be used while expanding cells width/height.
-    pub fn get_justification(&self, entity: Entity) -> char {
-        *self.justification.get(entity)
+    pub fn get_justification(&self, pos: Position) -> char {
+        *self.justification.get(pos)
     }
 
     /// Get a justification color which will be used while expanding cells width/height.
     ///
     /// `None` means no color.
-    pub fn get_justification_color(&self, entity: Entity) -> Option<&ANSIBuf> {
-        self.justification_color.get(entity).as_ref()
+    pub fn get_justification_color(&self, pos: Position) -> Option<&ANSIBuf> {
+        self.justification_color.get(pos).as_ref()
     }
 
     /// Set a justification which will be used while expanding cells width/height.
@@ -650,7 +665,9 @@ impl SpannedConfig {
 
     /// Verifies if there's any colors set for a borders.
     pub fn has_justification(&self) -> bool {
-        !self.justification.is_empty() || !self.justification_color.is_empty()
+        !self.justification.is_empty()
+            || !self.justification_color.is_empty()
+            || self.justification_color.as_ref().is_some()
     }
 
     /// Verifies if there's any custom padding set.
@@ -660,29 +677,27 @@ impl SpannedConfig {
 
     /// Verifies if there's any custom padding set.
     pub fn has_padding_color(&self) -> bool {
-        let map = HashMap::from(self.padding.clone());
-        let mut has_color = false;
-        for (entity, value) in map {
-            if matches!(entity, Entity::Global) {
-                continue;
-            }
+        if !self.padding_color.is_empty() {
+            let map = HashMap::from(self.padding_color.clone());
+            for (entity, value) in map {
+                if matches!(entity, Entity::Global) {
+                    continue;
+                }
 
-            has_color = value.bottom.color.is_some()
-                || value.top.color.is_some()
-                || value.left.color.is_some()
-                || value.right.color.is_some();
-
-            if has_color {
-                break;
+                if !value.is_empty() {
+                    return true;
+                }
             }
         }
 
-        has_color
+        !self.padding_color.as_ref().is_empty()
     }
 
     /// Verifies if there's any custom formatting set.
     pub fn has_formatting(&self) -> bool {
-        !self.formatting.is_empty()
+        !self.formatting_trim_h.is_empty()
+            || !self.formatting_trim_v.is_empty()
+            || !self.formatting_line_alignment.is_empty()
     }
 
     /// Verifies if there's any custom alignment vertical set.
@@ -704,7 +719,7 @@ impl SpannedConfig {
             return Some(*c);
         }
 
-        if self.has_horizontal(pos.0, shape.0) && self.has_vertical(pos.1, shape.1) {
+        if self.has_horizontal(pos.row, shape.0) && self.has_vertical(pos.col, shape.1) {
             return Some(self.get_borders_missing());
         }
 
@@ -720,7 +735,7 @@ impl SpannedConfig {
             return Some(*c);
         }
 
-        if self.has_horizontal(pos.0, count_rows) {
+        if self.has_horizontal(pos.row, count_rows) {
             return Some(self.get_borders_missing());
         }
 
@@ -735,7 +750,7 @@ impl SpannedConfig {
             return Some(*c);
         }
 
-        if self.has_vertical(pos.1, count_columns) {
+        if self.has_vertical(pos.col, count_columns) {
             return Some(self.get_borders_missing());
         }
 
@@ -874,13 +889,13 @@ fn set_cell_column_span(cfg: &mut SpannedConfig, pos: Position, span: usize) {
 fn is_cell_covered_by_column_span(cfg: &SpannedConfig, pos: Position) -> bool {
     cfg.span_columns
         .iter()
-        .any(|(&(row, col), span)| pos.1 > col && pos.1 < col + span && row == pos.0)
+        .any(|(p, span)| p.row == pos.row && pos.col > p.col && pos.col < p.col + span)
 }
 
 fn is_cell_covered_by_row_span(cfg: &SpannedConfig, pos: Position) -> bool {
     cfg.span_rows
         .iter()
-        .any(|(&(row, col), span)| pos.0 > row && pos.0 < row + span && col == pos.1)
+        .any(|(p, span)| p.col == pos.col && pos.row > p.row && pos.row < p.row + span)
 }
 
 fn is_cell_covered_by_both_spans(cfg: &SpannedConfig, pos: Position) -> bool {
@@ -893,20 +908,17 @@ fn is_cell_covered_by_both_spans(cfg: &SpannedConfig, pos: Position) -> bool {
             .iter()
             .filter(|(p2, _)| &p1 == p2)
             .any(|(_, col_span)| {
-                pos.0 > p1.0 && pos.0 < p1.0 + row_span && pos.1 > p1.1 && pos.1 < p1.1 + col_span
+                pos.row > p1.row
+                    && pos.row < p1.row + row_span
+                    && pos.col > p1.col
+                    && pos.col < p1.col + col_span
             })
     })
 }
 
-#[derive(Default, Debug, Clone, PartialEq, Eq)]
-struct ColoredIndent {
-    indent: Indent,
-    color: Option<ANSIBuf>,
-}
-
 /// A colorefull margin indent.
-#[derive(Debug, Clone, PartialEq, Eq)]
-struct ColoredMarginIndent {
+#[derive(Debug, Clone, PartialEq, Eq, Hash, PartialOrd, Ord)]
+struct MarginIndent {
     /// An indent value.
     indent: Indent,
     /// An offset value.
@@ -915,11 +927,11 @@ struct ColoredMarginIndent {
     color: Option<ANSIBuf>,
 }
 
-impl Default for ColoredMarginIndent {
+impl Default for MarginIndent {
     fn default() -> Self {
         Self {
             indent: Indent::default(),
-            offset: Offset::Begin(0),
+            offset: Offset::Start(0),
             color: None,
         }
     }
