@@ -58,9 +58,9 @@ fn includes_null_predicate_3038() -> PolarsResult<()> {
                     s.str()?
                         .to_lowercase()
                         .contains("not_exist", true)
-                        .map(|ca| Some(ca.into_column()))
+                        .map(|ca| ca.into_column())
                 },
-                GetOutput::from_type(DataType::Boolean),
+                |_, f| Ok(Field::new(f.name().clone(), DataType::Boolean)),
             ))
             .then(lit("unexpected"))
             .when(col("a").eq(lit("a1".to_string())))
@@ -88,9 +88,9 @@ fn includes_null_predicate_3038() -> PolarsResult<()> {
                     s.str()?
                         .to_lowercase()
                         .contains_literal("non-existent")
-                        .map(|ca| Some(ca.into_column()))
+                        .map(|ca| ca.into_column())
                 },
-                GetOutput::from_type(DataType::Boolean),
+                |_, f| Ok(Field::new(f.name().clone(), DataType::Boolean)),
             ))
             .then(lit("weird-1"))
             .when(col("a").eq(lit("a1".to_string())))
@@ -116,8 +116,6 @@ fn includes_null_predicate_3038() -> PolarsResult<()> {
 #[test]
 #[cfg(feature = "dtype-categorical")]
 fn test_when_then_otherwise_cats() -> PolarsResult<()> {
-    polars::enable_string_cache();
-
     let lf = df!["book" => [Some("bookA"),
         None,
         Some("bookB"),
@@ -130,8 +128,8 @@ fn test_when_then_otherwise_cats() -> PolarsResult<()> {
     ]?.lazy();
 
     let out = lf
-        .with_column(col("book").cast(DataType::Categorical(None, Default::default())))
-        .with_column(col("user").cast(DataType::Categorical(None, Default::default())))
+        .with_column(col("book").cast(DataType::from_categories(Categories::global())))
+        .with_column(col("user").cast(DataType::from_categories(Categories::global())))
         .with_column(
             when(col("book").is_null())
                 .then(col("user"))
@@ -142,7 +140,7 @@ fn test_when_then_otherwise_cats() -> PolarsResult<()> {
 
     assert_eq!(
         out.column("a")?
-            .categorical()?
+            .cat32()?
             .iter_str()
             .flatten()
             .collect::<Vec<_>>(),
@@ -217,9 +215,11 @@ fn test_when_then_otherwise_sum_in_agg() -> PolarsResult<()> {
     let q = df
         .lazy()
         .group_by([col("groups")])
-        .agg([when(all().exclude(["groups"]).sum().eq(lit(1)))
-            .then(all().exclude(["groups"]).sum())
-            .otherwise(lit(LiteralValue::untyped_null()))])
+        .agg([
+            when(all().exclude_cols(["groups"]).as_expr().sum().eq(lit(1)))
+                .then(all().exclude_cols(["groups"]).as_expr().sum())
+                .otherwise(lit(LiteralValue::untyped_null())),
+        ])
         .sort(["groups"], Default::default());
 
     let expected = df![
