@@ -1,10 +1,10 @@
 #![allow(unsafe_op_in_unsafe_fn)]
 use std::ops::Deref;
 use std::sync::LazyLock;
-use std::sync::atomic::{AtomicU64, Ordering};
 
 use either::Either;
 use polars_error::{PolarsResult, polars_bail};
+use polars_utils::relaxed_cell::RelaxedCell;
 
 use super::utils::{self, BitChunk, BitChunks, BitmapIter, count_zeros, fmt, get_bit_unchecked};
 use super::{IntoIter, MutableBitmap, chunk_iter_to_vec, num_intersections_with};
@@ -52,7 +52,7 @@ const UNKNOWN_BIT_COUNT: u64 = u64::MAX;
 /// // when sliced (or cloned), it is no longer possible to `into_mut`.
 /// let same: Bitmap = sliced.into_mut().left().unwrap();
 /// ```
-#[derive(Default)]
+#[derive(Default, Clone)]
 pub struct Bitmap {
     storage: SharedStorage<u8>,
     // Both offset and length are measured in bits. They are used to bound the
@@ -64,7 +64,7 @@ pub struct Bitmap {
     // If it is u64::MAX, we have no known value at all.
     // Other bit patterns where the top bit is set is reserved for future use.
     // If the top bit is not set we have an exact count.
-    unset_bit_count_cache: AtomicU64,
+    unset_bit_count_cache: RelaxedCell<u64>,
 }
 
 #[inline(always)]
@@ -72,19 +72,6 @@ fn has_cached_unset_bit_count(ubcc: u64) -> bool {
     ubcc >> 63 == 0
 }
 
-impl Clone for Bitmap {
-    fn clone(&self) -> Self {
-        Self {
-            storage: self.storage.clone(),
-            offset: self.offset,
-            length: self.length,
-            unset_bit_count_cache: AtomicU64::new(
-                self.unset_bit_count_cache.load(Ordering::Relaxed),
-            ),
-        }
-    }
-}
-
 impl std::fmt::Debug for Bitmap {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         let (bytes, offset, len) = self.as_slice();
@@ -120,7 +107,11 @@ impl Bitmap {
             storage: SharedStorage::from_vec(bytes),
             length,
             offset: 0,
-            unset_bit_count_cache: AtomicU64::new(if length == 0 { 0 } else { UNKNOWN_BIT_COUNT }),
+            unset_bit_count_cache: RelaxedCell::from(if length == 0 {
+                0
+            } else {
+                UNKNOWN_BIT_COUNT
+            }),
         })
     }
 
@@ -137,14 +128,14 @@ impl Bitmap {
     }
 
     /// Returns a new iterator of `bool` over this bitmap
-    pub fn iter(&self) -> BitmapIter {
+    pub fn iter(&self) -> BitmapIter<'_> {
         BitmapIter::new(&self.storage, self.offset, self.length)
     }
 
     /// Returns an iterator over bits in bit chunks [`BitChunk`].
     ///
     /// This iterator is useful to operate over multiple bits via e.g. bitwise.
-    pub fn chunks<T: BitChunk>(&self) -> BitChunks<T> {
+    pub fn chunks<T: BitChunk>(&self) -> BitChunks<'_, T> {
         BitChunks::new(&self.storage, self.offset, self.length)
     }
 
@@ -221,8 +212,7 @@ impl Bitmap {
     pub fn unset_bits(&self) -> usize {
         self.lazy_unset_bits().unwrap_or_else(|| {
             let zeros = count_zeros(&self.storage, self.offset, self.length);
-            self.unset_bit_count_cache
-                .store(zeros as u64, Ordering::Relaxed);
+            self.unset_bit_count_cache.store(zeros as u64);
             zeros
         })
     }
@@ -231,7 +221,7 @@ impl Bitmap {
     ///
     /// Guaranteed to be `<= self.len()`.
     pub fn lazy_unset_bits(&self) -> Option<usize> {
-        let cache = self.unset_bit_count_cache.load(Ordering::Relaxed);
+        let cache = self.unset_bit_count_cache.load();
         has_cached_unset_bit_count(cache).then_some(cache as usize)
     }
 
@@ -243,8 +233,7 @@ impl Bitmap {
     pub unsafe fn update_bit_count(&mut self, bits_set: usize) {
         assert!(bits_set <= self.length);
         let zeros = self.length - bits_set;
-        self.unset_bit_count_cache
-            .store(zeros as u64, Ordering::Relaxed);
+        self.unset_bit_count_cache.store(zeros as u64);
     }
 
     /// Slices `self`, offsetting by `offset` and truncating up to `length` bits.
@@ -416,7 +405,7 @@ impl Bitmap {
             storage,
             offset: 0,
             length,
-            unset_bit_count_cache: AtomicU64::new(length as u64),
+            unset_bit_count_cache: RelaxedCell::from(length as u64),
         }
     }
 
@@ -484,9 +473,9 @@ impl Bitmap {
         debug_assert!(check(&storage[..], offset, length).is_ok());
 
         let unset_bit_count_cache = if let Some(n) = unset_bits {
-            AtomicU64::new(n as u64)
+            RelaxedCell::from(n as u64)
         } else {
-            AtomicU64::new(UNKNOWN_BIT_COUNT)
+            RelaxedCell::from(UNKNOWN_BIT_COUNT)
         };
         Self {
             storage,
@@ -725,7 +714,7 @@ impl Splitable for Bitmap {
             return (self.clone(), Self::new());
         }
 
-        let ubcc = self.unset_bit_count_cache.load(Ordering::Relaxed);
+        let ubcc = self.unset_bit_count_cache.load();
 
         let lhs_length = offset;
         let rhs_length = self.length - offset;
@@ -768,13 +757,13 @@ impl Splitable for Bitmap {
                 storage: self.storage.clone(),
                 offset: self.offset,
                 length: lhs_length,
-                unset_bit_count_cache: AtomicU64::new(lhs_ubcc),
+                unset_bit_count_cache: RelaxedCell::from(lhs_ubcc),
             },
             Self {
                 storage: self.storage.clone(),
                 offset: self.offset + offset,
                 length: rhs_length,
-                unset_bit_count_cache: AtomicU64::new(rhs_ubcc),
+                unset_bit_count_cache: RelaxedCell::from(rhs_ubcc),
             },
         )
     }
