@@ -1,8 +1,4 @@
 #![cfg_attr(feature = "simd", feature(portable_simd))]
-#![cfg_attr(
-    all(feature = "simd", target_arch = "x86_64"),
-    feature(stdarch_x86_avx512)
-)]
 
 use arrow::types::NativeType;
 
