@@ -20,7 +20,7 @@ where
 {
     /// If data is aligned in a single chunk and has no Null values a zero copy view is returned
     /// as an [ndarray]
-    pub fn to_ndarray(&self) -> PolarsResult<ArrayView1<T::Native>> {
+    pub fn to_ndarray(&self) -> PolarsResult<ArrayView1<'_, T::Native>> {
         let slice = self.cont_slice()?;
         Ok(aview1(slice))
     }
