@@ -19,13 +19,13 @@ use compare_inner::NonNull;
 use rayon::prelude::*;
 pub use slice::*;
 
-use crate::chunked_array::ops::row_encode::_get_rows_encoded_ca;
+use super::row_encode::_get_rows_encoded_ca;
+use crate::POOL;
 use crate::prelude::compare_inner::TotalOrdInner;
 use crate::prelude::sort::arg_sort_multiple::*;
 use crate::prelude::*;
 use crate::series::IsSorted;
 use crate::utils::NoNull;
-use crate::POOL;
 
 fn partition_nulls<T: Copy>(
     values: &mut [T],
@@ -209,7 +209,7 @@ where
         let mut vals = Vec::with_capacity(ca.len());
 
         if !options.nulls_last {
-            let iter = std::iter::repeat(T::Native::default()).take(null_count);
+            let iter = std::iter::repeat_n(T::Native::default(), null_count);
             vals.extend(iter);
         }
 
@@ -226,11 +226,11 @@ where
         sort_impl_unstable(mut_slice, options);
 
         if options.nulls_last {
-            vals.extend(std::iter::repeat(T::Native::default()).take(ca.null_count()));
+            vals.extend(std::iter::repeat_n(T::Native::default(), ca.null_count()));
         }
 
         let arr = PrimitiveArray::new(
-            T::get_dtype().to_arrow(CompatLevel::newest()),
+            T::get_static_dtype().to_arrow(CompatLevel::newest()),
             vals.into(),
             Some(create_validity(len, null_count, options.nulls_last)),
         );
@@ -408,7 +408,7 @@ impl ChunkSort<BinaryType> for BinaryChunked {
         // We will sort by the views and reconstruct with sorted views. We leave the buffers as is.
         // We must rechunk to ensure that all views point into the proper buffers.
         let ca = self.rechunk();
-        let arr = ca.downcast_into_array();
+        let arr = ca.downcast_as_array().clone();
 
         let (views, buffers, validity, total_bytes_len, total_buffer_len) = arr.into_inner();
         let mut views = views.make_mut();
@@ -535,7 +535,7 @@ impl ChunkSort<BinaryOffsetType> for BinaryOffsetChunked {
                     length_so_far = values.len() as i64;
                     offsets.push(length_so_far);
                 }
-                offsets.extend(std::iter::repeat(length_so_far).take(null_count));
+                offsets.extend(std::iter::repeat_n(length_so_far, null_count));
 
                 // SAFETY: offsets are correctly created.
                 let arr = unsafe {
@@ -548,7 +548,7 @@ impl ChunkSort<BinaryOffsetType> for BinaryOffsetChunked {
                 ChunkedArray::with_chunk(self.name().clone(), arr)
             },
             (_, false) => {
-                offsets.extend(std::iter::repeat(length_so_far).take(null_count));
+                offsets.extend(std::iter::repeat_n(length_so_far, null_count));
 
                 for val in v {
                     values.extend_from_slice(val);
@@ -590,15 +590,28 @@ impl ChunkSort<BinaryOffsetType> for BinaryOffsetChunked {
     fn arg_sort(&self, mut options: SortOptions) -> IdxCa {
         options.multithreaded &= POOL.current_num_threads() > 1;
         let ca = self.rechunk();
-        let arr = ca.downcast_into_array();
+        let arr = ca.downcast_as_array();
         let mut idx = (0..(arr.len() as IdxSize)).collect::<Vec<_>>();
 
         let argsort = |args| {
-            sort_unstable_by_branch(args, options, |a, b| unsafe {
-                let a = arr.value_unchecked(*a as usize);
-                let b = arr.value_unchecked(*b as usize);
-                a.tot_cmp(&b)
-            });
+            if options.maintain_order {
+                sort_by_branch(
+                    args,
+                    options.descending,
+                    |a, b| unsafe {
+                        let a = arr.value_unchecked(*a as usize);
+                        let b = arr.value_unchecked(*b as usize);
+                        a.tot_cmp(&b)
+                    },
+                    options.multithreaded,
+                );
+            } else {
+                sort_unstable_by_branch(args, options, |a, b| unsafe {
+                    let a = arr.value_unchecked(*a as usize);
+                    let b = arr.value_unchecked(*b as usize);
+                    a.tot_cmp(&b)
+                });
+            }
         };
 
         if self.null_count() == 0 {
@@ -645,26 +658,20 @@ impl ChunkSort<BinaryOffsetType> for BinaryOffsetChunked {
     }
 }
 
-#[cfg(feature = "dtype-struct")]
-impl StructChunked {
-    pub(crate) fn arg_sort(&self, options: SortOptions) -> IdxCa {
-        let bin = _get_rows_encoded_ca(
-            self.name().clone(),
-            &[self.clone().into_column()],
-            &[options.descending],
-            &[options.nulls_last],
-        )
-        .unwrap();
-        bin.arg_sort(Default::default())
-    }
-}
-
 #[cfg(feature = "dtype-struct")]
 impl ChunkSort<StructType> for StructChunked {
     fn sort_with(&self, mut options: SortOptions) -> ChunkedArray<StructType> {
         options.multithreaded &= POOL.current_num_threads() > 1;
         let idx = self.arg_sort(options);
-        unsafe { self.take_unchecked(&idx) }
+        let mut out = unsafe { self.take_unchecked(&idx) };
+
+        let s = if options.descending {
+            IsSorted::Descending
+        } else {
+            IsSorted::Ascending
+        };
+        out.set_sorted_flag(s);
+        out
     }
 
     fn sort(&self, descending: bool) -> ChunkedArray<StructType> {
@@ -677,41 +684,79 @@ impl ChunkSort<StructType> for StructChunked {
     }
 }
 
+impl ChunkSort<ListType> for ListChunked {
+    fn sort_with(&self, mut options: SortOptions) -> ListChunked {
+        options.multithreaded &= POOL.current_num_threads() > 1;
+        let idx = self.arg_sort(options);
+        let mut out = unsafe { self.take_unchecked(&idx) };
+
+        let s = if options.descending {
+            IsSorted::Descending
+        } else {
+            IsSorted::Ascending
+        };
+        out.set_sorted_flag(s);
+        out
+    }
+
+    fn sort(&self, descending: bool) -> ListChunked {
+        self.sort_with(SortOptions::new().with_order_descending(descending))
+    }
+
+    fn arg_sort(&self, options: SortOptions) -> IdxCa {
+        let bin = _get_rows_encoded_ca(
+            self.name().clone(),
+            &[self.clone().into_column()],
+            &[options.descending],
+            &[options.nulls_last],
+        )
+        .unwrap();
+        bin.arg_sort(Default::default())
+    }
+}
+
 impl ChunkSort<BooleanType> for BooleanChunked {
     fn sort_with(&self, mut options: SortOptions) -> ChunkedArray<BooleanType> {
         options.multithreaded &= POOL.current_num_threads() > 1;
         sort_with_fast_path!(self, options);
-        assert!(
-            !options.nulls_last,
-            "null last not yet supported for bool dtype"
-        );
-        if self.null_count() == 0 {
-            let len = self.len();
-            let n_set = self.sum().unwrap() as usize;
-            let mut bitmap = BitmapBuilder::with_capacity(len);
-            let (first, second, n_set) = if options.descending {
-                (true, false, len - n_set)
-            } else {
-                (false, true, n_set)
-            };
-            bitmap.extend_constant(len - n_set, first);
-            bitmap.extend_constant(n_set, second);
-            let arr = BooleanArray::from_data_default(bitmap.freeze(), None);
+        let mut bitmap = BitmapBuilder::with_capacity(self.len());
+        let mut validity =
+            (self.null_count() > 0).then(|| BitmapBuilder::with_capacity(self.len()));
 
-            return unsafe { self.with_chunks(vec![Box::new(arr) as ArrayRef]) };
+        if self.null_count() > 0 && !options.nulls_last {
+            bitmap.extend_constant(self.null_count(), false);
+            if let Some(validity) = &mut validity {
+                validity.extend_constant(self.null_count(), false);
+            }
         }
 
-        let mut vals = self.into_iter().collect::<Vec<_>>();
-
+        let n_valid = self.len() - self.null_count();
+        let n_set = self.sum().unwrap() as usize;
         if options.descending {
-            vals.sort_by(|a, b| b.cmp(a))
+            bitmap.extend_constant(n_set, true);
+            bitmap.extend_constant(n_valid - n_set, false);
         } else {
-            vals.sort()
+            bitmap.extend_constant(n_valid - n_set, false);
+            bitmap.extend_constant(n_set, true);
+        }
+        if let Some(validity) = &mut validity {
+            validity.extend_constant(n_valid, true);
         }
 
-        let mut ca: BooleanChunked = vals.into_iter().collect_trusted();
-        ca.rename(self.name().clone());
-        ca
+        if self.null_count() > 0 && options.nulls_last {
+            bitmap.extend_constant(self.null_count(), false);
+            if let Some(validity) = &mut validity {
+                validity.extend_constant(self.null_count(), false);
+            }
+        }
+
+        Self::from_chunk_iter(
+            self.name().clone(),
+            Some(BooleanArray::from_data_default(
+                bitmap.freeze(),
+                validity.map(|v| v.freeze()),
+            )),
+        )
     }
 
     fn sort(&self, descending: bool) -> BooleanChunked {
