@@ -130,8 +130,10 @@ impl GroupsIdx {
 
     pub fn iter(
         &self,
-    ) -> std::iter::Zip<std::iter::Copied<std::slice::Iter<IdxSize>>, std::slice::Iter<IdxVec>>
-    {
+    ) -> std::iter::Zip<
+        std::iter::Copied<std::slice::Iter<'_, IdxSize>>,
+        std::slice::Iter<'_, IdxVec>,
+    > {
         self.into_iter()
     }
 
@@ -151,11 +153,20 @@ impl GroupsIdx {
         self.first.len()
     }
 
-    pub(crate) unsafe fn get_unchecked(&self, index: usize) -> BorrowIdxItem {
+    pub(crate) unsafe fn get_unchecked(&self, index: usize) -> BorrowIdxItem<'_> {
         let first = *self.first.get_unchecked(index);
         let all = self.all.get_unchecked(index);
         (first, all)
     }
+
+    // Create an 'empty group', containing 1 group of length 0
+    pub fn new_empty() -> Self {
+        Self {
+            sorted: false,
+            first: vec![0],
+            all: vec![vec![].into()],
+        }
+    }
 }
 
 impl FromIterator<IdxItem> for GroupsIdx {
@@ -327,7 +338,7 @@ impl GroupsType {
         }
     }
 
-    pub fn iter(&self) -> GroupsTypeIter {
+    pub fn iter(&self) -> GroupsTypeIter<'_> {
         GroupsTypeIter::new(self)
     }
 
@@ -377,7 +388,7 @@ impl GroupsType {
         }
     }
 
-    pub fn par_iter(&self) -> GroupsTypeParIter {
+    pub fn par_iter(&self) -> GroupsTypeParIter<'_> {
         GroupsTypeParIter::new(self)
     }
 
@@ -405,7 +416,7 @@ impl GroupsType {
         }
     }
 
-    pub fn get(&self, index: usize) -> GroupsIndicator {
+    pub fn get(&self, index: usize) -> GroupsIndicator<'_> {
         match self {
             GroupsType::Idx(groups) => {
                 let first = groups.first[index];
