@@ -1,7 +1,7 @@
 use crate::datatypes::AnyValue;
 use crate::datatypes::time_unit::TimeUnit;
 #[cfg(feature = "dtype-date")]
-use crate::prelude::MS_IN_DAY;
+use crate::prelude::US_IN_DAY;
 use crate::prelude::{DataType, Scalar};
 
 pub fn mean_reduce(value: Option<f64>, dtype: DataType) -> Scalar {
@@ -15,8 +15,8 @@ pub fn mean_reduce(value: Option<f64>, dtype: DataType) -> Scalar {
         },
         #[cfg(feature = "dtype-date")]
         DataType::Date => {
-            let val = value.map(|v| (v * MS_IN_DAY as f64) as i64);
-            Scalar::new(DataType::Datetime(TimeUnit::Milliseconds, None), val.into())
+            let val = value.map(|v| (v * US_IN_DAY as f64) as i64);
+            Scalar::new(DataType::Datetime(TimeUnit::Microseconds, None), val.into())
         },
         #[cfg(feature = "dtype-datetime")]
         dt @ DataType::Datetime(_, _) => {
