@@ -25,26 +25,27 @@ macro_rules! impl_eq_compare {
 
         #[cfg(feature = "dtype-categorical")]
         match (lhs.dtype(), rhs.dtype()) {
-            (Categorical(_, _) | Enum(_, _), Categorical(_, _) | Enum(_, _)) => {
-                return Ok(lhs
-                    .categorical()
-                    .unwrap()
-                    .$method(rhs.categorical().unwrap())?
-                    .with_name(lhs.name().clone()));
+            (Categorical(lcats, _), Categorical(rcats, _)) => {
+                ensure_same_categories(lcats, rcats)?;
+                return with_match_categorical_physical_type!(lcats.physical(), |$C| {
+                    lhs.cat::<$C>().unwrap().$method(rhs.cat::<$C>().unwrap())
+                })
+            },
+            (Enum(lfcats, _), Enum(rfcats, _)) => {
+                ensure_same_frozen_categories(lfcats, rfcats)?;
+                return with_match_categorical_physical_type!(lfcats.physical(), |$C| {
+                    lhs.cat::<$C>().unwrap().$method(rhs.cat::<$C>().unwrap())
+                })
             },
             (Categorical(_, _) | Enum(_, _), String) => {
-                return Ok(lhs
-                    .categorical()
-                    .unwrap()
-                    .$method(rhs.str().unwrap())?
-                    .with_name(lhs.name().clone()));
+                return with_match_categorical_physical_type!(lhs.dtype().cat_physical().unwrap(), |$C| {
+                    Ok(lhs.cat::<$C>().unwrap().$method(rhs.str().unwrap()))
+                })
             },
             (String, Categorical(_, _) | Enum(_, _)) => {
-                return Ok(rhs
-                    .categorical()
-                    .unwrap()
-                    .$method(lhs.str().unwrap())?
-                    .with_name(lhs.name().clone()));
+                return with_match_categorical_physical_type!(rhs.dtype().cat_physical().unwrap(), |$C| {
+                    Ok(rhs.cat::<$C>().unwrap().$method(lhs.str().unwrap()))
+                })
             },
             _ => (),
         };
@@ -61,6 +62,7 @@ macro_rules! impl_eq_compare {
             Boolean => lhs.bool().unwrap().$method(rhs.bool().unwrap()),
             String => lhs.str().unwrap().$method(rhs.str().unwrap()),
             Binary => lhs.binary().unwrap().$method(rhs.binary().unwrap()),
+            BinaryOffset => lhs.binary_offset().unwrap().$method(rhs.binary_offset().unwrap()),
             UInt8 => feature_gated!("dtype-u8", lhs.u8().unwrap().$method(rhs.u8().unwrap())),
             UInt16 => feature_gated!("dtype-u16", lhs.u16().unwrap().$method(rhs.u16().unwrap())),
             UInt32 => lhs.u32().unwrap().$method(rhs.u32().unwrap()),
@@ -97,7 +99,7 @@ macro_rules! bail_invalid_ineq {
 }
 
 macro_rules! impl_ineq_compare {
-    ($self:expr, $rhs:expr, $method:ident, $op:literal) => {{
+    ($self:expr, $rhs:expr, $method:ident, $op:literal, $rev_method:ident) => {{
         use DataType::*;
         let (lhs, rhs) = ($self, $rhs);
         validate_types(lhs.dtype(), rhs.dtype())?;
@@ -117,26 +119,28 @@ macro_rules! impl_ineq_compare {
 
         #[cfg(feature = "dtype-categorical")]
         match (lhs.dtype(), rhs.dtype()) {
-            (Categorical(_, _) | Enum(_, _), Categorical(_, _) | Enum(_, _)) => {
-                return Ok(lhs
-                    .categorical()
-                    .unwrap()
-                    .$method(rhs.categorical().unwrap())?
-                    .with_name(lhs.name().clone()));
+            (Categorical(lcats, _), Categorical(rcats, _)) => {
+                ensure_same_categories(lcats, rcats)?;
+                return with_match_categorical_physical_type!(lcats.physical(), |$C| {
+                    lhs.cat::<$C>().unwrap().$method(rhs.cat::<$C>().unwrap())
+                })
+            },
+            (Enum(lfcats, _), Enum(rfcats, _)) => {
+                ensure_same_frozen_categories(lfcats, rfcats)?;
+                return with_match_categorical_physical_type!(lfcats.physical(), |$C| {
+                    lhs.cat::<$C>().unwrap().$method(rhs.cat::<$C>().unwrap())
+                })
             },
             (Categorical(_, _) | Enum(_, _), String) => {
-                return Ok(lhs
-                    .categorical()
-                    .unwrap()
-                    .$method(rhs.str().unwrap())?
-                    .with_name(lhs.name().clone()));
+                return with_match_categorical_physical_type!(lhs.dtype().cat_physical().unwrap(), |$C| {
+                    lhs.cat::<$C>().unwrap().$method(rhs.str().unwrap())
+                })
             },
             (String, Categorical(_, _) | Enum(_, _)) => {
-                return Ok(rhs
-                    .categorical()
-                    .unwrap()
-                    .$method(lhs.str().unwrap())?
-                    .with_name(lhs.name().clone()));
+                return with_match_categorical_physical_type!(rhs.dtype().cat_physical().unwrap(), |$C| {
+                    // We use the reverse method as string <-> enum comparisons are only implemented one-way.
+                    rhs.cat::<$C>().unwrap().$rev_method(lhs.str().unwrap())
+                })
             },
             _ => (),
         };
@@ -156,6 +160,7 @@ macro_rules! impl_ineq_compare {
             Boolean => lhs.bool().unwrap().$method(rhs.bool().unwrap()),
             String => lhs.str().unwrap().$method(rhs.str().unwrap()),
             Binary => lhs.binary().unwrap().$method(rhs.binary().unwrap()),
+            BinaryOffset => lhs.binary_offset().unwrap().$method(rhs.binary_offset().unwrap()),
             UInt8 => feature_gated!("dtype-u8", lhs.u8().unwrap().$method(rhs.u8().unwrap())),
             UInt16 => feature_gated!("dtype-u16", lhs.u16().unwrap().$method(rhs.u16().unwrap())),
             UInt32 => lhs.u32().unwrap().$method(rhs.u32().unwrap()),
@@ -227,22 +232,22 @@ impl ChunkCompareIneq<&Series> for Series {
 
     /// Create a boolean mask by checking if self > rhs.
     fn gt(&self, rhs: &Series) -> Self::Item {
-        impl_ineq_compare!(self, rhs, gt, ">")
+        impl_ineq_compare!(self, rhs, gt, ">", lt)
     }
 
     /// Create a boolean mask by checking if self >= rhs.
     fn gt_eq(&self, rhs: &Series) -> Self::Item {
-        impl_ineq_compare!(self, rhs, gt_eq, ">=")
+        impl_ineq_compare!(self, rhs, gt_eq, ">=", lt_eq)
     }
 
     /// Create a boolean mask by checking if self < rhs.
     fn lt(&self, rhs: &Series) -> Self::Item {
-        impl_ineq_compare!(self, rhs, lt, "<")
+        impl_ineq_compare!(self, rhs, lt, "<", gt)
     }
 
     /// Create a boolean mask by checking if self <= rhs.
     fn lt_eq(&self, rhs: &Series) -> Self::Item {
-        impl_ineq_compare!(self, rhs, lt_eq, "<=")
+        impl_ineq_compare!(self, rhs, lt_eq, "<=", gt_eq)
     }
 }
 
@@ -316,9 +321,11 @@ impl ChunkCompareEq<&str> for Series {
         match self.dtype() {
             DataType::String => Ok(self.str().unwrap().equal(rhs)),
             #[cfg(feature = "dtype-categorical")]
-            DataType::Categorical(_, _) | DataType::Enum(_, _) => {
-                self.categorical().unwrap().equal(rhs)
-            },
+            DataType::Categorical(_, _) | DataType::Enum(_, _) => Ok(
+                with_match_categorical_physical_type!(self.dtype().cat_physical().unwrap(), |$C| {
+                    self.cat::<$C>().unwrap().equal(rhs)
+                }),
+            ),
             _ => Ok(BooleanChunked::full(self.name().clone(), false, self.len())),
         }
     }
@@ -328,9 +335,11 @@ impl ChunkCompareEq<&str> for Series {
         match self.dtype() {
             DataType::String => Ok(self.str().unwrap().equal_missing(rhs)),
             #[cfg(feature = "dtype-categorical")]
-            DataType::Categorical(_, _) | DataType::Enum(_, _) => {
-                self.categorical().unwrap().equal_missing(rhs)
-            },
+            DataType::Categorical(_, _) | DataType::Enum(_, _) => Ok(
+                with_match_categorical_physical_type!(self.dtype().cat_physical().unwrap(), |$C| {
+                    self.cat::<$C>().unwrap().equal_missing(rhs)
+                }),
+            ),
             _ => Ok(replace_non_null(
                 self.name().clone(),
                 self.0.chunks(),
@@ -344,9 +353,11 @@ impl ChunkCompareEq<&str> for Series {
         match self.dtype() {
             DataType::String => Ok(self.str().unwrap().not_equal(rhs)),
             #[cfg(feature = "dtype-categorical")]
-            DataType::Categorical(_, _) | DataType::Enum(_, _) => {
-                self.categorical().unwrap().not_equal(rhs)
-            },
+            DataType::Categorical(_, _) | DataType::Enum(_, _) => Ok(
+                with_match_categorical_physical_type!(self.dtype().cat_physical().unwrap(), |$C| {
+                    self.cat::<$C>().unwrap().not_equal(rhs)
+                }),
+            ),
             _ => Ok(BooleanChunked::full(self.name().clone(), true, self.len())),
         }
     }
@@ -356,9 +367,11 @@ impl ChunkCompareEq<&str> for Series {
         match self.dtype() {
             DataType::String => Ok(self.str().unwrap().not_equal_missing(rhs)),
             #[cfg(feature = "dtype-categorical")]
-            DataType::Categorical(_, _) | DataType::Enum(_, _) => {
-                self.categorical().unwrap().not_equal_missing(rhs)
-            },
+            DataType::Categorical(_, _) | DataType::Enum(_, _) => Ok(
+                with_match_categorical_physical_type!(self.dtype().cat_physical().unwrap(), |$C| {
+                    self.cat::<$C>().unwrap().not_equal_missing(rhs)
+                }),
+            ),
             _ => Ok(replace_non_null(self.name().clone(), self.0.chunks(), true)),
         }
     }
@@ -372,9 +385,11 @@ impl ChunkCompareIneq<&str> for Series {
         match self.dtype() {
             DataType::String => Ok(self.str().unwrap().gt(rhs)),
             #[cfg(feature = "dtype-categorical")]
-            DataType::Categorical(_, _) | DataType::Enum(_, _) => {
-                self.categorical().unwrap().gt(rhs)
-            },
+            DataType::Categorical(_, _) | DataType::Enum(_, _) => Ok(
+                with_match_categorical_physical_type!(self.dtype().cat_physical().unwrap(), |$C| {
+                    self.cat::<$C>().unwrap().gt(rhs)
+                }),
+            ),
             _ => polars_bail!(
                 ComputeError: "cannot compare str value to series of type {}", self.dtype(),
             ),
@@ -386,9 +401,11 @@ impl ChunkCompareIneq<&str> for Series {
         match self.dtype() {
             DataType::String => Ok(self.str().unwrap().gt_eq(rhs)),
             #[cfg(feature = "dtype-categorical")]
-            DataType::Categorical(_, _) | DataType::Enum(_, _) => {
-                self.categorical().unwrap().gt_eq(rhs)
-            },
+            DataType::Categorical(_, _) | DataType::Enum(_, _) => Ok(
+                with_match_categorical_physical_type!(self.dtype().cat_physical().unwrap(), |$C| {
+                    self.cat::<$C>().unwrap().gt_eq(rhs)
+                }),
+            ),
             _ => polars_bail!(
                 ComputeError: "cannot compare str value to series of type {}", self.dtype(),
             ),
@@ -400,9 +417,11 @@ impl ChunkCompareIneq<&str> for Series {
         match self.dtype() {
             DataType::String => Ok(self.str().unwrap().lt(rhs)),
             #[cfg(feature = "dtype-categorical")]
-            DataType::Categorical(_, _) | DataType::Enum(_, _) => {
-                self.categorical().unwrap().lt(rhs)
-            },
+            DataType::Categorical(_, _) | DataType::Enum(_, _) => Ok(
+                with_match_categorical_physical_type!(self.dtype().cat_physical().unwrap(), |$C| {
+                    self.cat::<$C>().unwrap().lt(rhs)
+                }),
+            ),
             _ => polars_bail!(
                 ComputeError: "cannot compare str value to series of type {}", self.dtype(),
             ),
@@ -414,9 +433,11 @@ impl ChunkCompareIneq<&str> for Series {
         match self.dtype() {
             DataType::String => Ok(self.str().unwrap().lt_eq(rhs)),
             #[cfg(feature = "dtype-categorical")]
-            DataType::Categorical(_, _) | DataType::Enum(_, _) => {
-                self.categorical().unwrap().lt_eq(rhs)
-            },
+            DataType::Categorical(_, _) | DataType::Enum(_, _) => Ok(
+                with_match_categorical_physical_type!(self.dtype().cat_physical().unwrap(), |$C| {
+                    self.cat::<$C>().unwrap().lt_eq(rhs)
+                }),
+            ),
             _ => polars_bail!(
                 ComputeError: "cannot compare str value to series of type {}", self.dtype(),
             ),
