@@ -18,7 +18,7 @@ impl Series {
 
     /// Convert a chunk in the Series to the correct Arrow type.
     /// This conversion is needed because polars doesn't use a
-    /// 1 on 1 mapping for logical/ categoricals, etc.
+    /// 1 on 1 mapping for logical/categoricals, etc.
     pub fn to_arrow(&self, chunk_idx: usize, compat_level: CompatLevel) -> ArrayRef {
         match self.dtype() {
             // make sure that we recursively apply all logical types.
@@ -76,7 +76,7 @@ impl Series {
                     s.to_arrow(0, compat_level)
                 };
 
-                let dtype = ListArray::<i64>::default_datatype(inner.to_arrow(compat_level));
+                let dtype = self.dtype().to_arrow(compat_level);
                 let arr = ListArray::<i64>::new(
                     dtype,
                     arr.offsets().clone(),
@@ -114,23 +114,16 @@ impl Series {
                 Box::new(arr)
             },
             #[cfg(feature = "dtype-categorical")]
-            dt @ (DataType::Categorical(_, ordering) | DataType::Enum(_, ordering)) => {
-                let ca = self.categorical().unwrap();
-                let arr = ca.physical().chunks()[chunk_idx].clone();
-                // SAFETY: categoricals are always u32's.
-                let cats = unsafe { UInt32Chunked::from_chunks(PlSmallStr::EMPTY, vec![arr]) };
-
-                // SAFETY: we only take a single chunk and change nothing about the index/rev_map mapping.
-                let new = unsafe {
-                    CategoricalChunked::from_cats_and_rev_map_unchecked(
-                        cats,
-                        ca.get_rev_map().clone(),
-                        matches!(dt, DataType::Enum(_, _)),
-                        *ordering,
-                    )
-                };
-
-                new.to_arrow(compat_level, false)
+            dt @ (DataType::Categorical(_, _) | DataType::Enum(_, _)) => {
+                with_match_categorical_physical_type!(dt.cat_physical().unwrap(), |$C| {
+                    let ca = self.cat::<$C>().unwrap();
+                    let arr = ca.physical().chunks()[chunk_idx].clone();
+                    unsafe {
+                        let new_phys = ChunkedArray::from_chunks(PlSmallStr::EMPTY, vec![arr]);
+                        let new = CategoricalChunked::<$C>::from_cats_and_dtype_unchecked(new_phys, dt.clone());
+                        new.to_arrow(compat_level).boxed()
+                    }
+                })
             },
             #[cfg(feature = "dtype-date")]
             DataType::Date => cast(
@@ -157,7 +150,7 @@ impl Series {
             )
             .unwrap(),
             #[cfg(feature = "dtype-decimal")]
-            DataType::Decimal(_, _) => self.decimal().unwrap().chunks()[chunk_idx]
+            DataType::Decimal(_, _) => self.decimal().unwrap().physical().chunks()[chunk_idx]
                 .as_any()
                 .downcast_ref::<PrimitiveArray<i128>>()
                 .unwrap()
