@@ -383,7 +383,7 @@ impl Series {
 
         use DataType as D;
         let do_clone = match dtype {
-            D::Unknown(UnknownKind::Any) => true,
+            D::Unknown(UnknownKind::Any | UnknownKind::Ufunc) => true,
             D::Unknown(UnknownKind::Int(_)) if slf.dtype().is_integer() => true,
             D::Unknown(UnknownKind::Float) if slf.dtype().is_float() => true,
             D::Unknown(UnknownKind::Str)
@@ -392,10 +392,6 @@ impl Series {
                 true
             },
             dt if dt.is_primitive() && dt == slf.dtype() => true,
-            #[cfg(feature = "dtype-categorical")]
-            D::Enum(None, _) => {
-                polars_bail!(InvalidOperation: "cannot cast / initialize Enum without categories present");
-            },
             _ => false,
         };
 
@@ -432,7 +428,7 @@ impl Series {
                     new_fields.extend(fields.iter().skip(new_fields.len()).cloned().map(|field| {
                         let dtype = cast_dtype(&field.dtype).unwrap_or(field.dtype);
                         Field {
-                            name: field.name.clone(),
+                            name: field.name,
                             dtype,
                         }
                     }));
@@ -500,7 +496,7 @@ impl Series {
     ///
     /// This can lead to invalid memory access in downstream code.
     pub unsafe fn from_physical_unchecked(&self, dtype: &DataType) -> PolarsResult<Self> {
-        debug_assert!(!self.dtype().is_logical());
+        debug_assert!(!self.dtype().is_logical(), "{:?}", self.dtype());
 
         if self.dtype() == dtype {
             return Ok(self.clone());
@@ -514,36 +510,29 @@ impl Series {
             },
 
             #[cfg(feature = "dtype-categorical")]
-            (D::UInt32, D::Categorical(revmap, ordering)) => match revmap {
-                Some(revmap) => Ok(unsafe {
-                    CategoricalChunked::from_cats_and_rev_map_unchecked(
-                        self.u32().unwrap().clone(),
-                        revmap.clone(),
-                        false,
-                        *ordering,
+            (phys, D::Categorical(cats, _)) if &cats.physical().dtype() == phys => {
+                with_match_categorical_physical_type!(cats.physical(), |$C| {
+                    type CA = ChunkedArray<<$C as PolarsCategoricalType>::PolarsPhysical>;
+                    let ca = self.as_ref().as_any().downcast_ref::<CA>().unwrap();
+                    Ok(CategoricalChunked::<$C>::from_cats_and_dtype_unchecked(
+                        ca.clone(),
+                        dtype.clone(),
                     )
-                }
-                .into_series()),
-                // In the streaming engine this is `None` and the global string cache is turned on
-                // for the duration of the query.
-                None => Ok(unsafe {
-                    CategoricalChunked::from_global_indices_unchecked(
-                        self.u32().unwrap().clone(),
-                        *ordering,
-                    )
-                    .into_series()
-                }),
+                    .into_series())
+                })
             },
             #[cfg(feature = "dtype-categorical")]
-            (D::UInt32, D::Enum(revmap, ordering)) => Ok(unsafe {
-                CategoricalChunked::from_cats_and_rev_map_unchecked(
-                    self.u32().unwrap().clone(),
-                    revmap.as_ref().unwrap().clone(),
-                    true,
-                    *ordering,
-                )
-            }
-            .into_series()),
+            (phys, D::Enum(fcats, _)) if &fcats.physical().dtype() == phys => {
+                with_match_categorical_physical_type!(fcats.physical(), |$C| {
+                    type CA = ChunkedArray<<$C as PolarsCategoricalType>::PolarsPhysical>;
+                    let ca = self.as_ref().as_any().downcast_ref::<CA>().unwrap();
+                    Ok(CategoricalChunked::<$C>::from_cats_and_dtype_unchecked(
+                        ca.clone(),
+                        dtype.clone(),
+                    )
+                    .into_series())
+                })
+            },
 
             (D::Int32, D::Date) => feature_gated!("dtype-time", Ok(self.clone().into_date())),
             (D::Int64, D::Datetime(tu, tz)) => feature_gated!(
@@ -711,11 +700,11 @@ impl Series {
     /// * Duration -> Int64
     /// * Decimal -> Int128
     /// * Time -> Int64
-    /// * Categorical -> UInt32
+    /// * Categorical -> U8/U16/U32
     /// * List(inner) -> List(physical of inner)
     /// * Array(inner) -> Array(physical of inner)
     /// * Struct -> Struct with physical repr of each struct column
-    pub fn to_physical_repr(&self) -> Cow<Series> {
+    pub fn to_physical_repr(&self) -> Cow<'_, Series> {
         use DataType::*;
         match self.dtype() {
             // NOTE: Don't use cast here, as it might rechunk (if all nulls)
@@ -729,9 +718,11 @@ impl Series {
             #[cfg(feature = "dtype-time")]
             Time => Cow::Owned(self.time().unwrap().phys.clone().into_series()),
             #[cfg(feature = "dtype-categorical")]
-            Categorical(_, _) | Enum(_, _) => {
-                let ca = self.categorical().unwrap();
-                Cow::Owned(ca.physical().clone().into_series())
+            dt @ (Categorical(_, _) | Enum(_, _)) => {
+                with_match_categorical_physical_type!(dt.cat_physical().unwrap(), |$C| {
+                    let ca = self.cat::<$C>().unwrap();
+                    Cow::Owned(ca.physical().clone().into_series())
+                })
             },
             #[cfg(feature = "dtype-decimal")]
             Decimal(_, _) => Cow::Owned(self.decimal().unwrap().phys.clone().into_series()),
@@ -847,7 +838,7 @@ impl Series {
             DataType::Time => self
                 .time()
                 .unwrap()
-                .as_ref()
+                .physical()
                 .clone()
                 .into_time()
                 .into_series(),
@@ -866,7 +857,7 @@ impl Series {
             DataType::Date => self
                 .date()
                 .unwrap()
-                .as_ref()
+                .physical()
                 .clone()
                 .into_date()
                 .into_series(),
@@ -892,7 +883,7 @@ impl Series {
             DataType::Datetime(_, _) => self
                 .datetime()
                 .unwrap()
-                .as_ref()
+                .physical()
                 .clone()
                 .into_datetime(timeunit, tz)
                 .into_series(),
@@ -917,7 +908,7 @@ impl Series {
             DataType::Duration(_) => self
                 .duration()
                 .unwrap()
-                .as_ref()
+                .physical()
                 .clone()
                 .into_duration(timeunit)
                 .into_series(),
@@ -926,7 +917,7 @@ impl Series {
     }
 
     // used for formatting
-    pub fn str_value(&self, index: usize) -> PolarsResult<Cow<str>> {
+    pub fn str_value(&self, index: usize) -> PolarsResult<Cow<'_, str>> {
         Ok(self.0.get(index)?.str_value())
     }
     /// Get the head of the Series.
@@ -991,13 +982,7 @@ impl Series {
     pub fn estimated_size(&self) -> usize {
         let mut size = 0;
         match self.dtype() {
-            #[cfg(feature = "dtype-categorical")]
-            DataType::Categorical(Some(rv), _) | DataType::Enum(Some(rv), _) => match &**rv {
-                RevMapping::Local(arr, _) => size += estimated_bytes_size(arr),
-                RevMapping::Global(map, arr, _) => {
-                    size += map.capacity() * size_of::<u32>() * 2 + estimated_bytes_size(arr);
-                },
-            },
+            // TODO @ cat-rework: include mapping size here?
             #[cfg(feature = "object")]
             DataType::Object(_) => {
                 let ArrowDataType::FixedSizeBinary(size) = self.chunks()[0].dtype() else {
@@ -1034,6 +1019,26 @@ impl Series {
         out.set_inner_dtype(s.dtype().clone());
         out
     }
+
+    pub fn row_encode_unordered(&self) -> PolarsResult<BinaryOffsetChunked> {
+        row_encode::_get_rows_encoded_ca_unordered(
+            self.name().clone(),
+            &[self.clone().into_column()],
+        )
+    }
+
+    pub fn row_encode_ordered(
+        &self,
+        descending: bool,
+        nulls_last: bool,
+    ) -> PolarsResult<BinaryOffsetChunked> {
+        row_encode::_get_rows_encoded_ca(
+            self.name().clone(),
+            &[self.clone().into_column()],
+            &[descending],
+            &[nulls_last],
+        )
+    }
 }
 
 impl Deref for Series {
@@ -1044,7 +1049,7 @@ impl Deref for Series {
     }
 }
 
-impl<'a> AsRef<(dyn SeriesTrait + 'a)> for Series {
+impl<'a> AsRef<dyn SeriesTrait + 'a> for Series {
     fn as_ref(&self) -> &(dyn SeriesTrait + 'a) {
         self.0.as_ref()
     }
@@ -1119,7 +1124,7 @@ mod test {
             PlSmallStr::from_static("a"),
             [ListArray::new(
                 ArrowDataType::LargeList(Box::new(ArrowField::new(
-                    PlSmallStr::from_static("item"),
+                    LIST_VALUES_NAME,
                     ArrowDataType::Int32,
                     true,
                 ))),
@@ -1178,7 +1183,7 @@ mod test {
         }
 
         {
-            let mut s2 = s2.clone();
+            let mut s2 = s2;
             s2.extend(&s1).unwrap();
             assert_eq!(s2.get(2).unwrap(), AnyValue::Decimal(2, 0));
         }
