@@ -32,8 +32,12 @@ impl IsSorted {
 }
 
 pub enum BitRepr {
-    Small(UInt32Chunked),
-    Large(UInt64Chunked),
+    U8(UInt8Chunked),
+    U16(UInt16Chunked),
+    U32(UInt32Chunked),
+    U64(UInt64Chunked),
+    #[cfg(feature = "dtype-i128")]
+    I128(Int128Chunked),
 }
 
 pub(crate) mod private {
@@ -62,7 +66,7 @@ pub(crate) mod private {
         }
 
         /// Get field (used in schema)
-        fn _field(&self) -> Cow<Field>;
+        fn _field(&self) -> Cow<'_, Field>;
 
         fn _dtype(&self) -> &DataType;
 
@@ -89,16 +93,12 @@ pub(crate) mod private {
             &self,
             _build_hasher: PlSeedableRandomStateQuality,
             _buf: &mut Vec<u64>,
-        ) -> PolarsResult<()> {
-            polars_bail!(opq = vec_hash, self._dtype());
-        }
+        ) -> PolarsResult<()>;
         fn vec_hash_combine(
             &self,
             _build_hasher: PlSeedableRandomStateQuality,
             _hashes: &mut [u64],
-        ) -> PolarsResult<()> {
-            polars_bail!(opq = vec_hash_combine, self._dtype());
-        }
+        ) -> PolarsResult<()>;
 
         /// # Safety
         ///
@@ -212,13 +212,13 @@ pub trait SeriesTrait:
     fn rename(&mut self, name: PlSmallStr);
 
     /// Get the lengths of the underlying chunks
-    fn chunk_lengths(&self) -> ChunkLenIter;
+    fn chunk_lengths(&self) -> ChunkLenIter<'_>;
 
     /// Name of series.
     fn name(&self) -> &PlSmallStr;
 
     /// Get field (used in schema)
-    fn field(&self) -> Cow<Field> {
+    fn field(&self) -> Cow<'_, Field> {
         self._field()
     }
 
@@ -402,7 +402,7 @@ pub trait SeriesTrait:
 
     /// Get a single value by index. Don't use this operation for loops as a runtime cast is
     /// needed for every iteration.
-    fn get(&self, index: usize) -> PolarsResult<AnyValue> {
+    fn get(&self, index: usize) -> PolarsResult<AnyValue<'_>> {
         polars_ensure!(index < self.len(), oob = index, self.len());
         // SAFETY: Just did bounds check
         let value = unsafe { self.get_unchecked(index) };
@@ -416,7 +416,7 @@ pub trait SeriesTrait:
     ///
     /// # Safety
     /// Does not do any bounds checking
-    unsafe fn get_unchecked(&self, _index: usize) -> AnyValue;
+    unsafe fn get_unchecked(&self, _index: usize) -> AnyValue<'_>;
 
     fn sort_with(&self, _options: SortOptions) -> PolarsResult<Series> {
         polars_bail!(opq = sort_with, self._dtype());
@@ -614,14 +614,14 @@ pub trait SeriesTrait:
     /// This has quite some dynamic dispatch, so prefer rolling_min, max, mean, sum over this.
     fn rolling_map(
         &self,
-        _f: &dyn Fn(&Series) -> Series,
+        _f: &dyn Fn(&Series) -> PolarsResult<Series>,
         _options: RollingOptionsFixedWindow,
     ) -> PolarsResult<Series> {
         polars_bail!(opq = rolling_map, self._dtype());
     }
 }
 
-impl (dyn SeriesTrait + '_) {
+impl dyn SeriesTrait + '_ {
     pub fn unpack<T: PolarsPhysicalType>(&self) -> PolarsResult<&ChunkedArray<T>> {
         polars_ensure!(&T::get_static_dtype() == self.dtype(), unpack);
         Ok(self.as_ref())
