@@ -23,7 +23,7 @@ impl PhysicalExpr for RollingExpr {
 
         let groups = {
             // Groups must be set by expression runner.
-            state.window_cache.get_groups(&groups_key).clone()
+            state.window_cache.get_groups(&groups_key)
         };
 
         // There can be multiple rolling expressions in a single expr.
@@ -56,7 +56,7 @@ impl PhysicalExpr for RollingExpr {
     }
 
     fn to_field(&self, input_schema: &Schema) -> PolarsResult<Field> {
-        self.function.to_field(input_schema, Context::Default)
+        self.function.to_field(input_schema)
     }
 
     fn as_expression(&self) -> Option<&Expr> {
