@@ -328,7 +328,7 @@ impl WindowExpr {
             },
             (WindowMapping::Join, AggState::NotAggregated(_)) => Ok(MapStrategy::Join),
             // literals, do nothing and let broadcast
-            (_, AggState::Literal(_)) => Ok(MapStrategy::Nothing),
+            (_, AggState::LiteralScalar(_)) => Ok(MapStrategy::Nothing),
         }
     }
 }
@@ -476,12 +476,6 @@ impl PhysicalExpr for WindowExpr {
         }
         let gb = GroupBy::new(df, group_by_columns.clone(), groups, Some(apply_columns));
 
-        // If the aggregation creates categoricals and `MapStrategy` is `Join`,
-        // the string cache was needed. So we hold it for that case.
-        // Worst case is that a categorical is created with indexes from the string
-        // cache which is fine, as the physical representation is undefined.
-        #[cfg(feature = "dtype-categorical")]
-        let _sc = polars_core::StringCacheHolder::hold();
         let mut ac = self.run_aggregation(df, state, &gb)?;
 
         use MapStrategy::*;
@@ -590,9 +584,7 @@ impl PhysicalExpr for WindowExpr {
                                 jt
                             } else {
                                 let jt = get_join_tuples()?;
-                                state
-                                    .window_cache
-                                    .insert_join(cache_key.clone(), jt.clone());
+                                state.window_cache.insert_join(cache_key, jt.clone());
                                 jt
                             }
                         } else {
@@ -608,7 +600,7 @@ impl PhysicalExpr for WindowExpr {
     }
 
     fn to_field(&self, input_schema: &Schema) -> PolarsResult<Field> {
-        self.function.to_field(input_schema, Context::Default)
+        self.function.to_field(input_schema)
     }
 
     fn is_scalar(&self) -> bool {
