@@ -50,13 +50,15 @@ impl LazyFrame {
                 cache: false,
                 glob: false,
                 projection: None,
+                column_mapping: None,
+                default_values: None,
                 row_index: None,
                 pre_slice: args.n_rows.map(|len| Slice::Positive { offset: 0, len }),
                 cast_columns_policy: CastColumnsPolicy::ERROR_ON_MISMATCH,
                 missing_columns_policy: MissingColumnsPolicy::Raise,
                 extra_columns_policy: ExtraColumnsPolicy::Raise,
                 include_file_paths: None,
-                deletion_files: Default::default(),
+                deletion_files: None,
             },
         )?
         .build()
