@@ -208,8 +208,8 @@ fn can_run_partitioned(
 
         let (unique_estimate, sampled_method) = match (keys.len(), keys[0].dtype()) {
             #[cfg(feature = "dtype-categorical")]
-            (1, DataType::Categorical(Some(rev_map), _) | DataType::Enum(Some(rev_map), _)) => {
-                (rev_map.len(), "known")
+            (1, DataType::Categorical(_, mapping) | DataType::Enum(_, mapping)) => {
+                (mapping.num_cats_upper_bound(), "known")
             },
             _ => {
                 // sqrt(N) is a good sample size as it remains low on large numbers
