@@ -1,7 +1,6 @@
 use polars_core::POOL;
 use polars_core::prelude::*;
 use polars_expr::state::ExecutionState;
-use polars_plan::global::_set_n_rows_for_scan;
 use polars_plan::plans::expr_ir::ExprIR;
 use polars_utils::format_pl_smallstr;
 use polars_utils::unique_id::UniqueId;
@@ -11,10 +10,11 @@ use self::expr_ir::OutputName;
 use self::predicates::{aexpr_to_column_predicates, aexpr_to_skip_batch_predicate};
 #[cfg(feature = "python")]
 use self::python_dsl::PythonScanSource;
-use super::super::executors::{self, Executor};
 use super::*;
 use crate::ScanPredicate;
-use crate::executors::{CachePrefiller, SinkExecutor};
+use crate::executors::{
+    self, CachePrefiller, Executor, PartitionedSinkExecutor, SinkExecutor, sink_name,
+};
 use crate::predicate::PhysicalColumnPredicates;
 
 pub type StreamingExecutorBuilder =
@@ -25,7 +25,7 @@ fn partitionable_gb(
     aggs: &[ExprIR],
     input_schema: &Schema,
     expr_arena: &Arena<AExpr>,
-    apply: &Option<Arc<dyn DataFrameUdf>>,
+    apply: &Option<PlanCallback<DataFrame, DataFrame>>,
 ) -> bool {
     // checks:
     //      1. complex expressions in the group_by itself are also not partitionable
@@ -164,11 +164,25 @@ pub fn python_scan_predicate(
     let predicate = if let PythonPredicate::Polars(e) = &options.predicate {
         // Convert to a pyarrow eval string.
         if matches!(options.python_source, PythonScanSource::Pyarrow) {
-            if let Some(eval_str) = polars_plan::plans::python::pyarrow::predicate_to_pa(
+            use polars_core::config::verbose_print_sensitive;
+
+            let predicate_pa = polars_plan::plans::python::pyarrow::predicate_to_pa(
                 e.node(),
                 expr_arena,
                 Default::default(),
-            ) {
+            );
+
+            verbose_print_sensitive(|| {
+                format!(
+                    "python_scan_predicate: \
+                    predicate node: {}, \
+                    converted pyarrow predicate: {}",
+                    ExprIRDisplay::display_node(e.node(), expr_arena),
+                    &predicate_pa.as_deref().unwrap_or("<conversion failed>")
+                )
+            });
+
+            if let Some(eval_str) = predicate_pa {
                 options.predicate = PythonPredicate::PyArrow(eval_str);
                 // We don't have to use a physical expression as pyarrow deals with the filter.
                 None
@@ -209,7 +223,7 @@ fn create_physical_plan_impl(
     expr_arena: &mut Arena<AExpr>,
     state: &mut ConversionState,
     // Cache nodes in order of discovery
-    cache_nodes: &mut PlIndexMap<UniqueId, Box<executors::CacheExec>>,
+    cache_nodes: &mut PlIndexMap<UniqueId, executors::CachePrefill>,
     build_streaming_executor: Option<StreamingExecutorBuilder>,
 ) -> PolarsResult<Box<dyn Executor>> {
     use IR::*;
@@ -227,7 +241,16 @@ fn create_physical_plan_impl(
         };
     }
 
-    let logical_plan = if state.has_cache_parent || matches!(lp_arena.get(root), IR::Scan { .. }) {
+    let logical_plan = if state.has_cache_parent
+        || matches!(
+            lp_arena.get(root),
+            IR::Scan { .. } // Needed for the streaming impl
+                | IR::Cache { .. } // Needed for plans branching from the same cache node
+                | IR::Sink { // Needed for the streaming impl
+                    payload: SinkTypeIR::Partition(_),
+                    ..
+                }
+        ) {
         lp_arena.get(root).clone()
     } else {
         lp_arena.take(root)
@@ -259,22 +282,10 @@ fn create_physical_plan_impl(
                     sink_options,
                     cloud_options,
                 }) => {
-                    let name: &'static str = match &file_type {
-                        #[cfg(feature = "parquet")]
-                        FileType::Parquet(_) => "parquet",
-                        #[cfg(feature = "ipc")]
-                        FileType::Ipc(_) => "ipc",
-                        #[cfg(feature = "csv")]
-                        FileType::Csv(_) => "csv",
-                        #[cfg(feature = "json")]
-                        FileType::Json(_) => "json",
-                        #[allow(unreachable_patterns)]
-                        _ => panic!("enable filetype feature"),
-                    };
-
+                    let name = sink_name(&file_type).to_owned();
                     Ok(Box::new(SinkExecutor {
                         input,
-                        name: name.to_string(),
+                        name,
                         f: Box::new(move |mut df, _state| {
                             let mut file = target
                                 .open_into_writeable(&sink_options, cloud_options.as_ref())?;
@@ -330,6 +341,7 @@ fn create_physical_plan_impl(
                                         .with_float_precision(
                                             options.serialize_options.float_precision,
                                         )
+                                        .with_decimal_comma(options.serialize_options.decimal_comma)
                                         .with_null_value(options.serialize_options.null.clone())
                                         .with_quote_style(options.serialize_options.quote_style)
                                         .finish(&mut df)?;
@@ -354,11 +366,24 @@ fn create_physical_plan_impl(
                         }),
                     }))
                 },
+                SinkTypeIR::Partition(_) => {
+                    let builder = build_streaming_executor
+                        .expect("invalid build. Missing feature new-streaming");
 
-                SinkTypeIR::Partition { .. } => {
-                    polars_bail!(InvalidOperation:
-                        "partition sinks not yet supported in standard engine."
-                    )
+                    let executor = Box::new(PartitionedSinkExecutor::new(
+                        input, builder, root, lp_arena, expr_arena,
+                    ));
+
+                    // Use cache so that this runs during the cache pre-filling stage and not on the
+                    // thread pool, it could deadlock since the streaming engine uses the thread
+                    // pool internally.
+                    let mut prefill = executors::CachePrefill::new_sink(executor);
+                    let exec = prefill.make_exec();
+                    let existing = cache_nodes.insert(prefill.id(), prefill);
+
+                    assert!(existing.is_none());
+
+                    Ok(Box::new(exec))
                 },
             }
         },
@@ -394,26 +419,8 @@ fn create_physical_plan_impl(
             Ok(Box::new(executors::SliceExec { input, offset, len }))
         },
         Filter { input, predicate } => {
-            let mut streamable =
-                is_elementwise_rec_no_cat_cast(expr_arena.get(predicate.node()), expr_arena);
+            let streamable = is_elementwise_rec(predicate.node(), expr_arena);
             let input_schema = lp_arena.get(input).schema(lp_arena).into_owned();
-            if streamable {
-                // This can cause problems with string caches
-                streamable = !input_schema
-                    .iter_values()
-                    .any(|dt| dt.contains_categoricals())
-                    || {
-                        #[cfg(feature = "dtype-categorical")]
-                        {
-                            polars_core::using_string_cache()
-                        }
-
-                        #[cfg(not(feature = "dtype-categorical"))]
-                        {
-                            false
-                        }
-                    }
-            }
             let input = recurse!(input, state)?;
             let mut state = ExpressionConversionState::new(true);
             let predicate = create_physical_expr(
@@ -438,17 +445,8 @@ fn create_physical_plan_impl(
             output_schema,
             scan_type,
             predicate,
-            mut unified_scan_args,
-            id: scan_mem_id,
+            unified_scan_args,
         } => {
-            unified_scan_args.pre_slice = if let Some(mut slice) = unified_scan_args.pre_slice {
-                *slice.len_mut() = _set_n_rows_for_scan(Some(slice.len())).unwrap();
-                Some(slice)
-            } else {
-                _set_n_rows_for_scan(None)
-                    .map(|len| polars_utils::slice_enum::Slice::Positive { offset: 0, len })
-            };
-
             let mut expr_conversion_state = ExpressionConversionState::new(true);
 
             let mut create_skip_batch_predicate = false;
@@ -456,7 +454,7 @@ fn create_physical_plan_impl(
             {
                 create_skip_batch_predicate |= matches!(
                     &*scan_type,
-                    FileScan::Parquet {
+                    FileScanIR::Parquet {
                         options: polars_io::prelude::ParquetOptions {
                             use_statistics: true,
                             ..
@@ -481,7 +479,7 @@ fn create_physical_plan_impl(
                 .transpose()?;
 
             match *scan_type {
-                FileScan::Anonymous { function, .. } => {
+                FileScanIR::Anonymous { function, .. } => {
                     Ok(Box::new(executors::AnonymousScanExec {
                         function,
                         predicate,
@@ -503,37 +501,19 @@ fn create_physical_plan_impl(
                     state.has_cache_parent = true;
                     state.has_cache_child = true;
 
-                    if !cache_nodes.contains_key(&scan_mem_id) {
-                        let build_func = build_streaming_executor
-                            .expect("invalid build. Missing feature new-streaming");
+                    let build_func = build_streaming_executor
+                        .expect("invalid build. Missing feature new-streaming");
 
-                        let executor = build_func(root, lp_arena, expr_arena)?;
+                    let executor = build_func(root, lp_arena, expr_arena)?;
 
-                        cache_nodes.insert(
-                            scan_mem_id.clone(),
-                            Box::new(executors::CacheExec {
-                                input: Some(executor),
-                                id: scan_mem_id.clone(),
-                                // This is (n_hits - 1), because the drop logic is `fetch_sub(1) == 0`.
-                                count: 0,
-                                is_new_streaming_scan: true,
-                            }),
-                        );
-                    } else {
-                        // Already exists - this scan IR is under a CSE (subplan). We need to
-                        // increment the cache hit count here.
-                        let cache_exec = cache_nodes.get_mut(&scan_mem_id).unwrap();
-                        cache_exec.count = cache_exec.count.saturating_add(1);
-                    }
+                    let mut prefill = executors::CachePrefill::new_scan(executor);
+                    let exec = prefill.make_exec();
 
-                    Ok(Box::new(executors::CacheExec {
-                        id: scan_mem_id,
-                        // Rest of the fields don't matter - the actual node was inserted into
-                        // `cache_nodes`.
-                        input: None,
-                        count: Default::default(),
-                        is_new_streaming_scan: true,
-                    }))
+                    let existing = cache_nodes.insert(prefill.id(), prefill);
+
+                    assert!(existing.is_none());
+
+                    Ok(Box::new(exec))
                 },
                 #[allow(unreachable_patterns)]
                 _ => unreachable!(),
@@ -558,7 +538,7 @@ fn create_physical_plan_impl(
                 &mut state,
             )?;
 
-            let allow_vertical_parallelism = options.should_broadcast && expr.iter().all(|e| is_elementwise_rec_no_cat_cast(expr_arena.get(e.node()), expr_arena))
+            let allow_vertical_parallelism = options.should_broadcast && expr.iter().all(|e| is_elementwise_rec(e.node(), expr_arena))
                 // If all columns are literal we would get a 1 row per thread.
                 && !phys_expr.iter().all(|p| {
                     p.is_literal()
@@ -587,6 +567,7 @@ fn create_physical_plan_impl(
             slice,
             sort_options,
         } => {
+            debug_assert!(!by_column.is_empty());
             let input_schema = lp_arena.get(input).schema(lp_arena);
             let by_column = create_physical_expressions_from_irs(
                 &by_column,
@@ -603,33 +584,22 @@ fn create_physical_plan_impl(
                 sort_options,
             }))
         },
-        Cache {
-            input,
-            id,
-            cache_hits,
-        } => {
+        Cache { input, id } => {
             state.has_cache_parent = true;
             state.has_cache_child = true;
 
-            if !cache_nodes.contains_key(&id) {
+            if let Some(cache) = cache_nodes.get_mut(&id) {
+                Ok(Box::new(cache.make_exec()))
+            } else {
                 let input = recurse!(input, state)?;
 
-                let cache = Box::new(executors::CacheExec {
-                    id: id.clone(),
-                    input: Some(input),
-                    count: cache_hits,
-                    is_new_streaming_scan: false,
-                });
+                let mut prefill = executors::CachePrefill::new_cache(input, id);
+                let exec = prefill.make_exec();
 
-                cache_nodes.insert(id.clone(), cache);
+                cache_nodes.insert(id, prefill);
+
+                Ok(Box::new(exec))
             }
-
-            Ok(Box::new(executors::CacheExec {
-                id,
-                input: None,
-                count: cache_hits,
-                is_new_streaming_scan: false,
-            }))
         },
         Distinct { input, options } => {
             let input = recurse!(input, state)?;
@@ -693,7 +663,7 @@ fn create_physical_plan_impl(
             // We first check if we can partition the group_by on the latest moment.
             let partitionable = partitionable_gb(&keys, &aggs, &input_schema, expr_arena, &apply);
             if partitionable {
-                let from_partitioned_ds = (&*lp_arena).iter(input).any(|(_, lp)| {
+                let from_partitioned_ds = lp_arena.iter(input).any(|(_, lp)| {
                     if let Union { options, .. } = lp {
                         options.from_partitioned_ds
                     } else {
@@ -826,7 +796,7 @@ fn create_physical_plan_impl(
             let allow_vertical_parallelism = options.should_broadcast
                 && exprs
                     .iter()
-                    .all(|e| is_elementwise_rec_no_cat_cast(expr_arena.get(e.node()), expr_arena));
+                    .all(|e| is_elementwise_rec(e.node(), expr_arena));
 
             let mut state =
                 ExpressionConversionState::new(POOL.current_num_threads() > exprs.len());
@@ -1078,3 +1048,45 @@ pub fn create_scan_predicate(
         hive_predicate_is_full_predicate,
     })
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_create_multiple_physical_plans_reused_cache() {
+        // Check that reusing the same cache node doesn't panic.
+        // CSE creates duplicate cache nodes with the same ID, but cloud reuses them.
+
+        let mut ir = Arena::new();
+
+        let schema = Schema::from_iter([(PlSmallStr::from_static("x"), DataType::Float32)]);
+        let scan = ir.add(IR::DataFrameScan {
+            df: Arc::new(DataFrame::empty_with_schema(&schema)),
+            schema: Arc::new(schema),
+            output_schema: None,
+        });
+
+        let cache = ir.add(IR::Cache {
+            input: scan,
+            id: UniqueId::new(),
+        });
+
+        let left_sink = ir.add(IR::Sink {
+            input: cache,
+            payload: SinkTypeIR::Memory,
+        });
+        let right_sink = ir.add(IR::Sink {
+            input: cache,
+            payload: SinkTypeIR::Memory,
+        });
+
+        let _multiplan = create_multiple_physical_plans(
+            &[left_sink, right_sink],
+            &mut ir,
+            &mut Arena::new(),
+            None,
+        )
+        .unwrap();
+    }
+}
