@@ -9,11 +9,12 @@ use crate::parquet::encoding::{Encoding, byte_stream_split, hybrid_rle};
 use crate::parquet::error::ParquetResult;
 use crate::parquet::page::{DataPage, DictPage, split_buffer};
 use crate::parquet::types::{NativeType as ParquetNativeType, decode};
+use crate::read::Filter;
 use crate::read::deserialize::dictionary_encoded;
 use crate::read::deserialize::utils::{
     dict_indices_decoder, freeze_validity, unspecialized_decode,
 };
-use crate::read::{Filter, PredicateFilter};
+use crate::read::expr::{ParquetScalar, SpecializedParquetColumnExpr};
 
 #[allow(clippy::large_enum_variant)]
 #[derive(Debug)]
@@ -26,7 +27,7 @@ pub(crate) enum StateTranslation<'a> {
 impl<'a, P, T, D> utils::StateTranslation<'a, FloatDecoder<P, T, D>> for StateTranslation<'a>
 where
     T: NativeType,
-    P: ParquetNativeType,
+    P: ParquetNativeType + for<'b> TryFrom<&'b ParquetScalar>,
     D: DecoderFunction<P, T>,
 {
     type PlainDecoder = &'a [u8];
@@ -110,10 +111,15 @@ impl<T: NativeType> utils::Decoded for (Vec<T>, BitmapBuilder) {
     fn len(&self) -> usize {
         self.0.len()
     }
+
     fn extend_nulls(&mut self, n: usize) {
         self.0.resize(self.0.len() + n, T::default());
         self.1.extend_constant(n, false);
     }
+
+    fn remaining_capacity(&self) -> usize {
+        (self.0.capacity() - self.0.len()).min(self.1.capacity() - self.1.len())
+    }
 }
 
 impl<P, T, D> utils::Decoder for FloatDecoder<P, T, D>
@@ -146,7 +152,6 @@ where
             &mut BitmapBuilder::new(),
             &mut self.0.intermediate,
             &mut target,
-            &mut BitmapBuilder::new(),
             self.0.decoder,
         )?;
         Ok(PrimitiveArray::new(
@@ -156,22 +161,29 @@ where
         ))
     }
 
-    fn has_predicate_specialization(
-        &self,
+    fn evaluate_predicate(
+        &mut self,
         state: &utils::State<'_, Self>,
-        predicate: &PredicateFilter,
+        _predicate: Option<&SpecializedParquetColumnExpr>,
+        pred_true_mask: &mut BitmapBuilder,
+        dict_mask: Option<&Bitmap>,
     ) -> ParquetResult<bool> {
-        let mut has_predicate_specialization = false;
+        if state.page_validity.is_some() {
+            // @Performance: implement validity aware
+            return Ok(false);
+        }
 
-        has_predicate_specialization |=
-            matches!(state.translation, StateTranslation::Dictionary(_));
-        has_predicate_specialization |= matches!(state.translation, StateTranslation::Plain(_))
-            && predicate.predicate.to_equals_scalar().is_some();
+        if let StateTranslation::Dictionary(values) = &state.translation {
+            let dict_mask = dict_mask.unwrap();
+            super::super::dictionary_encoded::predicate::decode(
+                values.clone(),
+                dict_mask,
+                pred_true_mask,
+            )?;
+            return Ok(true);
+        }
 
-        // @TODO: This should be implemented
-        has_predicate_specialization &= state.page_validity.is_none();
-
-        Ok(has_predicate_specialization)
+        Ok(false)
     }
 
     fn extend_decoded(
@@ -198,8 +210,8 @@ where
         &mut self,
         mut state: utils::State<'_, Self>,
         decoded: &mut Self::DecodedState,
-        pred_true_mask: &mut BitmapBuilder,
         filter: Option<Filter>,
+        _chunks: &mut Vec<Self::Output>,
     ) -> ParquetResult<()> {
         match state.translation {
             StateTranslation::Plain(ref mut values) => super::plain::decode(
@@ -210,19 +222,16 @@ where
                 &mut decoded.1,
                 &mut self.0.intermediate,
                 &mut decoded.0,
-                pred_true_mask,
                 self.0.decoder,
             ),
             StateTranslation::Dictionary(ref mut indexes) => dictionary_encoded::decode_dict(
                 indexes.clone(),
                 state.dict.unwrap().values().as_slice(),
-                state.dict_mask,
                 state.is_optional,
                 state.page_validity.as_ref(),
                 filter,
                 &mut decoded.1,
                 &mut decoded.0,
-                pred_true_mask,
             ),
             StateTranslation::ByteStreamSplit(mut decoder) => {
                 let num_rows = decoder.len();
@@ -241,6 +250,15 @@ where
         }
     }
 
+    fn extend_constant(
+        &mut self,
+        decoded: &mut Self::DecodedState,
+        length: usize,
+        value: &ParquetScalar,
+    ) -> ParquetResult<()> {
+        self.0.extend_constant(decoded, length, value)
+    }
+
     fn finalize(
         &self,
         dtype: ArrowDataType,
