@@ -1,11 +1,13 @@
-use arrow::array::{Array, FixedSizeBinaryArray, PrimitiveArray};
+use arrow::array::{Array, FixedSizeBinaryArray, PrimitiveArray, StructArray};
 use arrow::bitmap::Bitmap;
 use arrow::datatypes::{
-    ArrowDataType, DTYPE_CATEGORICAL, DTYPE_ENUM_VALUES, Field, IntegerType, IntervalUnit, TimeUnit,
+    ArrowDataType, DTYPE_CATEGORICAL_LEGACY, DTYPE_CATEGORICAL_NEW, DTYPE_ENUM_VALUES_LEGACY,
+    DTYPE_ENUM_VALUES_NEW, Field, IntegerType, IntervalUnit, TimeUnit,
 };
 use arrow::types::{NativeType, days_ms, i256};
 use ethnum::I256;
 use polars_compute::cast::CastOptionsImpl;
+use polars_utils::pl_str::PlSmallStr;
 
 use super::utils::filter::Filter;
 use super::{
@@ -17,9 +19,9 @@ use crate::parquet::schema::types::{
 };
 use crate::parquet::types::int96_to_i64_ns;
 use crate::read::ParquetError;
-use crate::read::deserialize::binview;
 use crate::read::deserialize::categorical::CategoricalDecoder;
 use crate::read::deserialize::utils::PageDecoder;
+use crate::read::deserialize::{binary, binview};
 
 /// An iterator adapter that maps an iterator of Pages a boxed [`Array`] of [`ArrowDataType`]
 /// `dtype` with a maximum of `num_rows` elements.
@@ -29,7 +31,7 @@ pub fn page_iter_to_array(
     field: Field,
     filter: Option<Filter>,
     init_nested: Option<Vec<InitNested>>,
-) -> ParquetResult<(Option<NestedState>, Box<dyn Array>, Bitmap)> {
+) -> ParquetResult<(Option<NestedState>, Vec<Box<dyn Array>>, Bitmap)> {
     use ArrowDataType::*;
 
     let physical_type = &type_.physical_type;
@@ -141,18 +143,23 @@ pub fn page_iter_to_array(
             )?
             .collect(filter)?;
 
-            let values = array
-                .values()
-                .chunks_exact(n)
-                .map(|value: &[u8]| i32::from_le_bytes(value[..4].try_into().unwrap()))
-                .collect::<Vec<_>>();
-            let validity = array.validity().cloned();
+            let array = array
+                .into_iter()
+                .map(|array| {
+                    let values = array
+                        .values()
+                        .chunks_exact(n)
+                        .map(|value: &[u8]| i32::from_le_bytes(value[..4].try_into().unwrap()))
+                        .collect::<Vec<_>>();
+                    let validity = array.validity().cloned();
+                    Ok(
+                        PrimitiveArray::<i32>::try_new(dtype.clone(), values.into(), validity)?
+                            .to_boxed(),
+                    )
+                })
+                .collect::<ParquetResult<Vec<Box<dyn Array>>>>()?;
 
-            (
-                nested,
-                PrimitiveArray::<i32>::try_new(dtype.clone(), values.into(), validity)?.to_boxed(),
-                ptm,
-            )
+            (nested, array, ptm)
         },
         (PhysicalType::FixedLenByteArray(12), Interval(IntervalUnit::DayTime)) => {
             // @TODO: Make a separate decoder for this
@@ -167,19 +174,70 @@ pub fn page_iter_to_array(
             )?
             .collect(filter)?;
 
-            let values = array
-                .values()
-                .chunks_exact(n)
-                .map(super::super::convert_days_ms)
-                .collect::<Vec<_>>();
-            let validity = array.validity().cloned();
+            let array = array
+                .into_iter()
+                .map(|array| {
+                    let values = array
+                        .values()
+                        .chunks_exact(n)
+                        .map(super::super::convert_days_ms)
+                        .collect::<Vec<_>>();
+                    let validity = array.validity().cloned();
+                    Ok(
+                        PrimitiveArray::<days_ms>::try_new(dtype.clone(), values.into(), validity)?
+                            .to_boxed(),
+                    )
+                })
+                .collect::<ParquetResult<Vec<Box<dyn Array>>>>()?;
 
-            (
-                nested,
-                PrimitiveArray::<days_ms>::try_new(dtype.clone(), values.into(), validity)?
-                    .to_boxed(),
-                ptm,
-            )
+            (nested, array, ptm)
+        },
+        (PhysicalType::FixedLenByteArray(12), Interval(IntervalUnit::MonthDayMillis)) => {
+            // @TODO: Make a separate decoder for this
+
+            const N_BYTES: usize = 12;
+            let (nested, array, ptm) = PageDecoder::new(
+                &field.name,
+                pages,
+                ArrowDataType::FixedSizeBinary(N_BYTES),
+                fixed_size_binary::BinaryDecoder { size: N_BYTES },
+                init_nested,
+            )?
+            .collect(filter)?;
+
+            let out = array
+                .into_iter()
+                .map(|arr| convert_interval_bytes_to_month_day_nano_struct(arr).boxed())
+                .collect();
+
+            (nested, out, ptm)
+        },
+        (PhysicalType::FixedLenByteArray(16), Int128) => {
+            let n = 16;
+            let (nested, array, ptm) = PageDecoder::new(
+                &field.name,
+                pages,
+                ArrowDataType::FixedSizeBinary(n),
+                fixed_size_binary::BinaryDecoder { size: n },
+                init_nested,
+            )?
+            .collect(filter)?;
+
+            let array = array
+                .into_iter()
+                .map(|array| {
+                    let (_, values, validity) = array.into_inner();
+                    let values = values.try_transmute().expect(
+                        "this should work since the parquet decoder has alignment constraints",
+                    );
+                    Ok(
+                        PrimitiveArray::<i128>::try_new(dtype.clone(), values, validity)?
+                            .to_boxed(),
+                    )
+                })
+                .collect::<ParquetResult<Vec<Box<dyn Array>>>>()?;
+
+            (nested, array, ptm)
         },
         (PhysicalType::Int32, Decimal(_, _)) => PageDecoder::new(
             &field.name,
@@ -216,18 +274,23 @@ pub fn page_iter_to_array(
             )?
             .collect(filter)?;
 
-            let values = array
-                .values()
-                .chunks_exact(n)
-                .map(|value: &[u8]| super::super::convert_i128(value, n))
-                .collect::<Vec<_>>();
-            let validity = array.validity().cloned();
+            let array = array
+                .into_iter()
+                .map(|array| {
+                    let values = array
+                        .values()
+                        .chunks_exact(n)
+                        .map(|value: &[u8]| super::super::convert_i128(value, n))
+                        .collect::<Vec<_>>();
+                    let validity = array.validity().cloned();
+                    Ok(
+                        PrimitiveArray::<i128>::try_new(dtype.clone(), values.into(), validity)?
+                            .to_boxed(),
+                    )
+                })
+                .collect::<ParquetResult<Vec<Box<dyn Array>>>>()?;
 
-            (
-                nested,
-                PrimitiveArray::<i128>::try_new(dtype.clone(), values.into(), validity)?.to_boxed(),
-                ptm,
-            )
+            (nested, array, ptm)
         },
         (PhysicalType::Int32, Decimal256(_, _)) => PageDecoder::new(
             &field.name,
@@ -259,18 +322,23 @@ pub fn page_iter_to_array(
             )?
             .collect(filter)?;
 
-            let values = array
-                .values()
-                .chunks_exact(n)
-                .map(|value: &[u8]| i256(I256::new(super::super::convert_i128(value, n))))
-                .collect::<Vec<_>>();
-            let validity = array.validity().cloned();
+            let array = array
+                .into_iter()
+                .map(|array| {
+                    let values = array
+                        .values()
+                        .chunks_exact(n)
+                        .map(|value: &[u8]| i256(I256::new(super::super::convert_i128(value, n))))
+                        .collect::<Vec<_>>();
+                    let validity = array.validity().cloned();
+                    Ok(
+                        PrimitiveArray::<i256>::try_new(dtype.clone(), values.into(), validity)?
+                            .to_boxed(),
+                    )
+                })
+                .collect::<ParquetResult<Vec<Box<dyn Array>>>>()?;
 
-            (
-                nested,
-                PrimitiveArray::<i256>::try_new(dtype.clone(), values.into(), validity)?.to_boxed(),
-                ptm,
-            )
+            (nested, array, ptm)
         },
         (PhysicalType::FixedLenByteArray(n), Decimal256(_, _)) if *n <= 32 => {
             // @TODO: Make a separate decoder for this
@@ -286,18 +354,23 @@ pub fn page_iter_to_array(
             )?
             .collect(filter)?;
 
-            let values = array
-                .values()
-                .chunks_exact(n)
-                .map(super::super::convert_i256)
-                .collect::<Vec<_>>();
-            let validity = array.validity().cloned();
+            let array = array
+                .into_iter()
+                .map(|array| {
+                    let values = array
+                        .values()
+                        .chunks_exact(n)
+                        .map(super::super::convert_i256)
+                        .collect::<Vec<_>>();
+                    let validity = array.validity().cloned();
+                    Ok(
+                        PrimitiveArray::<i256>::try_new(dtype.clone(), values.into(), validity)?
+                            .to_boxed(),
+                    )
+                })
+                .collect::<ParquetResult<Vec<Box<dyn Array>>>>()?;
 
-            (
-                nested,
-                PrimitiveArray::<i256>::try_new(dtype.clone(), values.into(), validity)?.to_boxed(),
-                ptm,
-            )
+            (nested, array, ptm)
         },
         (PhysicalType::FixedLenByteArray(n), Decimal256(_, _)) if *n > 32 => {
             return Err(ParquetError::not_supported(format!(
@@ -342,7 +415,7 @@ pub fn page_iter_to_array(
         (PhysicalType::FixedLenByteArray(2), Float32) => {
             // @NOTE: To reduce code bloat, we just use the FixedSizeBinary decoder.
 
-            let (nested, mut fsb_array, ptm) = PageDecoder::new(
+            let (nested, array, ptm) = PageDecoder::new(
                 &field.name,
                 pages,
                 ArrowDataType::FixedSizeBinary(2),
@@ -351,24 +424,26 @@ pub fn page_iter_to_array(
             )?
             .collect(filter)?;
 
-            let validity = fsb_array.take_validity();
-            let values = fsb_array.values().as_slice();
-            assert_eq!(values.len() % 2, 0);
-            let values = values.chunks_exact(2);
-            let values = values
-                .map(|v| {
-                    // SAFETY: We know that `v` is always of size two.
-                    let le_bytes: [u8; 2] = unsafe { v.try_into().unwrap_unchecked() };
-                    let v = arrow::types::f16::from_le_bytes(le_bytes);
-                    v.to_f32()
+            let array = array
+                .into_iter()
+                .map(|mut fsb_array| {
+                    let validity = fsb_array.take_validity();
+                    let values = fsb_array.values().as_slice();
+                    assert_eq!(values.len() % 2, 0);
+                    let values = values.chunks_exact(2);
+                    let values = values
+                        .map(|v| {
+                            // SAFETY: We know that `v` is always of size two.
+                            let le_bytes: [u8; 2] = unsafe { v.try_into().unwrap_unchecked() };
+                            let v = arrow::types::f16::from_le_bytes(le_bytes);
+                            v.to_f32()
+                        })
+                        .collect();
+                    Ok(PrimitiveArray::<f32>::new(dtype.clone(), values, validity).to_boxed())
                 })
-                .collect();
+                .collect::<ParquetResult<Vec<Box<dyn Array>>>>()?;
 
-            (
-                nested,
-                PrimitiveArray::<f32>::new(dtype, values, validity).to_boxed(),
-                ptm,
-            )
+            (nested, array, ptm)
         },
 
         (PhysicalType::Float, Float32) => PageDecoder::new(
@@ -387,14 +462,23 @@ pub fn page_iter_to_array(
             init_nested,
         )?
         .collect_boxed(filter)?,
+        // Decoder for BinaryOffset
+        (PhysicalType::ByteArray, LargeBinary) => PageDecoder::new(
+            &field.name,
+            pages,
+            dtype,
+            binary::BinaryDecoder,
+            init_nested,
+        )?
+        .collect_boxed(filter)?,
         // Don't compile this code with `i32` as we don't use this in polars
-        (PhysicalType::ByteArray, LargeBinary | LargeUtf8) => {
+        (PhysicalType::ByteArray, LargeUtf8) => {
             let is_string = matches!(dtype, LargeUtf8);
             PageDecoder::new(
                 &field.name,
                 pages,
                 dtype,
-                binview::BinViewDecoder { is_string },
+                binview::BinViewDecoder::new(is_string),
                 init_nested,
             )?
             .collect(filter)?
@@ -406,7 +490,7 @@ pub fn page_iter_to_array(
                 &field.name,
                 pages,
                 dtype,
-                binview::BinViewDecoder { is_string },
+                binview::BinViewDecoder::new(is_string),
                 init_nested,
             )?
             .collect(filter)?
@@ -418,17 +502,41 @@ pub fn page_iter_to_array(
             assert_eq!(value_type.as_ref(), &ArrowDataType::Utf8View);
 
             if field.metadata.is_some_and(|md| {
-                md.contains_key(DTYPE_ENUM_VALUES) || md.contains_key(DTYPE_CATEGORICAL)
-            }) && matches!(key_type, IntegerType::UInt32)
-            {
-                PageDecoder::new(
-                    &field.name,
-                    pages,
-                    dtype,
-                    CategoricalDecoder::new(),
-                    init_nested,
-                )?
-                .collect_boxed(filter)?
+                md.contains_key(DTYPE_ENUM_VALUES_LEGACY)
+                    || md.contains_key(DTYPE_ENUM_VALUES_NEW)
+                    || md.contains_key(DTYPE_CATEGORICAL_NEW)
+                    || md.contains_key(DTYPE_CATEGORICAL_LEGACY)
+            }) && matches!(
+                key_type,
+                IntegerType::UInt8 | IntegerType::UInt16 | IntegerType::UInt32
+            ) {
+                match key_type {
+                    IntegerType::UInt8 => PageDecoder::new(
+                        &field.name,
+                        pages,
+                        dtype,
+                        CategoricalDecoder::<u8>::new(),
+                        init_nested,
+                    )?
+                    .collect_boxed(filter)?,
+                    IntegerType::UInt16 => PageDecoder::new(
+                        &field.name,
+                        pages,
+                        dtype,
+                        CategoricalDecoder::<u16>::new(),
+                        init_nested,
+                    )?
+                    .collect_boxed(filter)?,
+                    IntegerType::UInt32 => PageDecoder::new(
+                        &field.name,
+                        pages,
+                        dtype,
+                        CategoricalDecoder::<u32>::new(),
+                        init_nested,
+                    )?
+                    .collect_boxed(filter)?,
+                    _ => unreachable!(),
+                }
             } else {
                 let (nested, array, ptm) = PageDecoder::new(
                     &field.name,
@@ -439,12 +547,19 @@ pub fn page_iter_to_array(
                 )?
                 .collect(filter)?;
 
-                (
-                    nested,
-                    polars_compute::cast::cast(array.as_ref(), &dtype, CastOptionsImpl::default())
-                        .unwrap(),
-                    ptm,
-                )
+                let array = array
+                    .into_iter()
+                    .map(|array| {
+                        polars_compute::cast::cast(
+                            array.as_ref(),
+                            &dtype,
+                            CastOptionsImpl::default(),
+                        )
+                        .unwrap()
+                    })
+                    .collect();
+
+                (nested, array, ptm)
             }
         },
         (from, to) => {
@@ -534,7 +649,7 @@ fn timestamp(
     filter: Option<Filter>,
     time_unit: TimeUnit,
     nested: Option<Vec<InitNested>>,
-) -> ParquetResult<(Option<NestedState>, Box<dyn Array>, Bitmap)> {
+) -> ParquetResult<(Option<NestedState>, Vec<Box<dyn Array>>, Bitmap)> {
     if physical_type == &PhysicalType::Int96 {
         return match time_unit {
             TimeUnit::Nanosecond => PageDecoder::new(
@@ -606,3 +721,66 @@ fn timestamp(
         .collect_boxed(filter),
     }
 }
+
+/// Converts directly from Parquet INTERVAL to Struct.
+fn convert_interval_bytes_to_month_day_nano_struct(
+    month_day_millis_bytes: FixedSizeBinaryArray,
+) -> StructArray {
+    const ROW_WIDTH: usize = 12;
+
+    let bytes: &[u8] = month_day_millis_bytes.values();
+    let output_length = bytes.len() / ROW_WIDTH;
+
+    assert_eq!(bytes.len(), output_length * ROW_WIDTH);
+
+    let (months_out, days_out, nanoseconds_out): (Vec<i32>, Vec<i32>, Vec<i64>) = (0
+        ..output_length)
+        .map(|i| {
+            let bytes: [u8; ROW_WIDTH] =
+                unsafe { bytes.get_unchecked(i * ROW_WIDTH..(i + 1) * ROW_WIDTH) }
+                    .try_into()
+                    .unwrap();
+
+            let months: i32 = i32::from_le_bytes(bytes[..4].try_into().unwrap());
+            let days: i32 = i32::from_le_bytes(bytes[4..8].try_into().unwrap());
+            let nanoseconds: i64 = (i32::from_le_bytes(bytes[8..12].try_into().unwrap()) as i64)
+                .checked_mul(1_000_000i64) // Convert milliseconds to nanoseconds.
+                .unwrap();
+
+            (months, days, nanoseconds)
+        })
+        .collect();
+
+    let struct_fields = vec![
+        Field::new(
+            PlSmallStr::from_static("months"),
+            ArrowDataType::Int32,
+            true,
+        ),
+        Field::new(PlSmallStr::from_static("days"), ArrowDataType::Int32, true),
+        Field::new(
+            PlSmallStr::from_static("nanoseconds"),
+            ArrowDataType::Duration(TimeUnit::Nanosecond),
+            true,
+        ),
+    ];
+
+    let struct_value_arrays = vec![
+        PrimitiveArray::<i32>::from_vec(months_out).boxed(),
+        PrimitiveArray::<i32>::from_vec(days_out).boxed(),
+        PrimitiveArray::<i64>::try_new(
+            ArrowDataType::Duration(TimeUnit::Nanosecond),
+            nanoseconds_out.into(),
+            None,
+        )
+        .unwrap()
+        .boxed(),
+    ];
+
+    StructArray::new(
+        ArrowDataType::Struct(struct_fields),
+        output_length,
+        struct_value_arrays,
+        month_day_millis_bytes.validity().cloned(),
+    )
+}
