@@ -187,9 +187,9 @@ impl StringNameSpace {
     /// Strings with length equal to or greater than the given length are
     /// returned as-is.
     #[cfg(feature = "string_pad")]
-    pub fn pad_start(self, length: usize, fill_char: char) -> Expr {
+    pub fn pad_start(self, length: Expr, fill_char: char) -> Expr {
         self.0
-            .map_unary(StringFunction::PadStart { length, fill_char })
+            .map_binary(StringFunction::PadStart { fill_char }, length)
     }
 
     /// Pad the end of the string until it reaches the given length.
@@ -198,9 +198,9 @@ impl StringNameSpace {
     /// Strings with length equal to or greater than the given length are
     /// returned as-is.
     #[cfg(feature = "string_pad")]
-    pub fn pad_end(self, length: usize, fill_char: char) -> Expr {
+    pub fn pad_end(self, length: Expr, fill_char: char) -> Expr {
         self.0
-            .map_unary(StringFunction::PadEnd { length, fill_char })
+            .map_binary(StringFunction::PadEnd { fill_char }, length)
     }
 
     /// Pad the start of the string with zeros until it reaches the given length.
@@ -303,8 +303,8 @@ impl StringNameSpace {
 
     /// Convert a String column into a Decimal column.
     #[cfg(feature = "dtype-decimal")]
-    pub fn to_decimal(self, infer_length: usize) -> Expr {
-        self.0.map_unary(StringFunction::ToDecimal(infer_length))
+    pub fn to_decimal(self, scale: usize) -> Expr {
+        self.0.map_unary(StringFunction::ToDecimal { scale })
     }
 
     /// Concat the values into a string array.
@@ -432,8 +432,10 @@ impl StringNameSpace {
 
     #[cfg(feature = "string_to_integer")]
     /// Parse string in base radix into decimal.
-    pub fn to_integer(self, base: Expr, strict: bool) -> Expr {
-        self.0.map_binary(StringFunction::ToInteger(strict), base)
+    /// The resulting dtype is `dtype`
+    pub fn to_integer(self, base: Expr, dtype: Option<DataType>, strict: bool) -> Expr {
+        self.0
+            .map_binary(StringFunction::ToInteger { dtype, strict }, base)
     }
 
     /// Return the length of each string as the number of bytes.
@@ -475,15 +477,8 @@ impl StringNameSpace {
     }
 
     #[cfg(feature = "extract_jsonpath")]
-    pub fn json_decode(
-        self,
-        dtype: Option<impl Into<DataTypeExpr>>,
-        infer_schema_len: Option<usize>,
-    ) -> Expr {
-        self.0.map_unary(StringFunction::JsonDecode {
-            dtype: dtype.map(Into::into),
-            infer_schema_len,
-        })
+    pub fn json_decode(self, dtype: impl Into<DataTypeExpr>) -> Expr {
+        self.0.map_unary(StringFunction::JsonDecode(dtype.into()))
     }
 
     #[cfg(feature = "extract_jsonpath")]
