@@ -1,7 +1,6 @@
-use std::path::PathBuf;
-
 use polars_io::cloud::CloudOptions;
 use polars_utils::mmap::MemSlice;
+use polars_utils::plpath::PlPath;
 
 use super::*;
 
@@ -13,9 +12,9 @@ impl CountStar {
     }
 }
 
-impl OptimizationRule for CountStar {
+impl CountStar {
     // Replace select count(*) from datasource with specialized map function.
-    fn optimize_plan(
+    pub(super) fn optimize_plan(
         &mut self,
         lp_arena: &mut Arena<IR>,
         expr_arena: &mut Arena<AExpr>,
@@ -74,7 +73,7 @@ struct CountStarExpr {
     sources: ScanSources,
     cloud_options: Option<CloudOptions>,
     // File Type
-    scan_type: Box<FileScan>,
+    scan_type: Box<FileScanIR>,
     // Column Alias
     alias: Option<PlSmallStr>,
 }
@@ -91,11 +90,11 @@ fn visit_logical_plan_for_scan_paths(
     match lp_arena.get(node) {
         IR::Union { inputs, .. } => {
             enum MutableSources {
-                Paths(Vec<PathBuf>),
+                Addresses(Vec<PlPath>),
                 Buffers(Vec<MemSlice>),
             }
 
-            let mut scan_type: Option<Box<FileScan>> = None;
+            let mut scan_type: Option<Box<FileScanIR>> = None;
             let mut cloud_options = None;
             let mut sources = None;
 
@@ -110,11 +109,11 @@ fn visit_logical_plan_for_scan_paths(
                     Some(expr) => {
                         match (expr.sources, &mut sources) {
                             (
-                                ScanSources::Paths(paths),
-                                Some(MutableSources::Paths(mutable_paths)),
-                            ) => mutable_paths.extend_from_slice(&paths[..]),
-                            (ScanSources::Paths(paths), None) => {
-                                sources = Some(MutableSources::Paths(paths.to_vec()))
+                                ScanSources::Paths(addrs),
+                                Some(MutableSources::Addresses(mutable_addrs)),
+                            ) => mutable_addrs.extend_from_slice(&addrs[..]),
+                            (ScanSources::Paths(addrs), None) => {
+                                sources = Some(MutableSources::Addresses(addrs.to_vec()))
                             },
                             (
                                 ScanSources::Buffers(buffers),
@@ -147,7 +146,7 @@ fn visit_logical_plan_for_scan_paths(
             }
             Some(CountStarExpr {
                 sources: match sources {
-                    Some(MutableSources::Paths(paths)) => ScanSources::Paths(paths.into()),
+                    Some(MutableSources::Addresses(addrs)) => ScanSources::Paths(addrs.into()),
                     Some(MutableSources::Buffers(buffers)) => ScanSources::Buffers(buffers.into()),
                     None => ScanSources::default(),
                 },
@@ -168,7 +167,7 @@ fn visit_logical_plan_for_scan_paths(
 
             let use_fast_file_count = use_fast_file_count.unwrap_or(match scan_type.as_ref() {
                 #[cfg(feature = "csv")]
-                FileScan::Csv { .. } => true,
+                FileScanIR::Csv { .. } => true,
                 _ => false,
             });
 
