@@ -36,7 +36,7 @@ mod inner {
 
 pub(super) use inner::SlicePushDown;
 
-#[derive(Copy, Clone)]
+#[derive(Copy, Clone, Debug)]
 struct State {
     offset: i64,
     len: IdxSize,
@@ -148,11 +148,10 @@ impl SlicePushDown {
         expr_arena: &mut Arena<AExpr>,
     ) -> PolarsResult<IR> {
         let inputs = lp.get_inputs();
-        let exprs = lp.get_exprs();
 
         let new_inputs = inputs
-            .iter()
-            .map(|&node| {
+            .into_iter()
+            .map(|node| {
                 let alp = lp_arena.take(node);
                 // No state, so we do not push down the slice here.
                 let state = None;
@@ -160,8 +159,8 @@ impl SlicePushDown {
                 lp_arena.replace(node, alp);
                 Ok(node)
             })
-            .collect::<PolarsResult<Vec<_>>>()?;
-        let lp = lp.with_exprs_and_input(exprs, new_inputs);
+            .collect::<PolarsResult<UnitVec<_>>>()?;
+        let lp = lp.with_inputs(new_inputs);
 
         self.no_pushdown_finish_opt(lp, state, lp_arena)
     }
@@ -175,18 +174,17 @@ impl SlicePushDown {
         expr_arena: &mut Arena<AExpr>,
     ) -> PolarsResult<IR> {
         let inputs = lp.get_inputs();
-        let exprs = lp.get_exprs();
 
         let new_inputs = inputs
-            .iter()
-            .map(|&node| {
+            .into_iter()
+            .map(|node| {
                 let alp = lp_arena.take(node);
                 let alp = self.pushdown(alp, state, lp_arena, expr_arena)?;
                 lp_arena.replace(node, alp);
                 Ok(node)
             })
-            .collect::<PolarsResult<Vec<_>>>()?;
-        Ok(lp.with_exprs_and_input(exprs, new_inputs))
+            .collect::<PolarsResult<UnitVec<_>>>()?;
+        Ok(lp.with_inputs(new_inputs))
     }
 
     #[recursive]
@@ -222,25 +220,24 @@ impl SlicePushDown {
                 mut unified_scan_args,
                 predicate,
                 scan_type,
-                id: _,
             }, Some(state)) if predicate.is_none() && match &*scan_type {
                 #[cfg(feature = "parquet")]
-                FileScan::Parquet { .. } => true,
+                FileScanIR::Parquet { .. } => true,
 
                 #[cfg(feature = "ipc")]
-                FileScan::Ipc { .. } => true,
+                FileScanIR::Ipc { .. } => true,
 
                 #[cfg(feature = "csv")]
-                FileScan::Csv { .. } => true,
+                FileScanIR::Csv { .. } => true,
 
                 #[cfg(feature = "json")]
-                FileScan::NDJson { .. } => true,
+                FileScanIR::NDJson { .. } => true,
 
                 #[cfg(feature = "python")]
-                FileScan::PythonDataset { .. } => true,
+                FileScanIR::PythonDataset { .. } => true,
 
                 // TODO: This can be `true` after Anonymous scan dispatches to new-streaming.
-                FileScan::Anonymous { .. } => state.offset == 0,
+                FileScanIR::Anonymous { .. } => state.offset == 0,
             }  =>  {
                 unified_scan_args.pre_slice = Some(state.to_slice_enum());
 
@@ -252,7 +249,6 @@ impl SlicePushDown {
                     scan_type,
                     unified_scan_args,
                     predicate,
-                    id: Default::default(),
                 };
 
                 Ok(lp)
@@ -286,7 +282,7 @@ impl SlicePushDown {
                 left_on,
                 right_on,
                 mut options
-            }, Some(state)) if !matches!(options.options, Some(JoinTypeOptionsIR::Cross { .. })) => {
+            }, Some(state)) if !matches!(options.options, Some(JoinTypeOptionsIR::CrossAndFilter { .. })) => {
                 // first restart optimization in both inputs and get the updated LP
                 let lp_left = lp_arena.take(input_left);
                 let lp_left = self.pushdown(lp_left, None, lp_arena, expr_arena)?;
@@ -358,34 +354,69 @@ impl SlicePushDown {
             (Slice {
                 input,
                 offset,
-                len
-            }, Some(previous_state)) => {
+                mut len
+            }, Some(outer_slice)) => {
                 let alp = lp_arena.take(input);
-                let state = Some(if previous_state.offset == offset  {
-                    State {
-                        offset,
-                        len: std::cmp::min(len, previous_state.len)
+
+                // Both are positive, can combine into a single slice.
+                if outer_slice.offset >= 0 && offset >= 0 {
+                    let state = State {
+                        offset: offset.checked_add(outer_slice.offset).unwrap(),
+                        len: if len as i64 > outer_slice.offset {
+                            (len - outer_slice.offset as IdxSize).min(outer_slice.len)
+                        } else {
+                            0
+                        },
+                    };
+                    return self.pushdown(alp, Some(state), lp_arena, expr_arena);
+                }
+
+                // If offset is negative the length can never be greater than it.
+                if offset < 0 {
+                    if len as i64 > -offset {
+                        len = (-offset) as IdxSize;
                     }
-                } else {
-                    State {
-                        offset,
-                        len
-                    }
-                });
-                let lp = self.pushdown(alp, state, lp_arena, expr_arena)?;
+                }
+
+                // Both are negative, can also combine (but not so simply).
+                if outer_slice.offset < 0 && offset < 0 {
+                    let inner_start_rel_end = offset;
+                    let inner_stop_rel_end = inner_start_rel_end + len as i64;
+                    let naive_outer_start_rel_end = inner_stop_rel_end + outer_slice.offset;
+                    let naive_outer_stop_rel_end = naive_outer_start_rel_end + outer_slice.len as i64;
+                    let clamped_outer_start_rel_end = naive_outer_start_rel_end.max(inner_start_rel_end);
+                    let clamped_outer_stop_rel_end = naive_outer_stop_rel_end.max(clamped_outer_start_rel_end);
+
+                    let state = State {
+                        offset: clamped_outer_start_rel_end,
+                        len: (clamped_outer_stop_rel_end - clamped_outer_start_rel_end) as IdxSize,
+                    };
+                    return self.pushdown(alp, Some(state), lp_arena, expr_arena);
+                }
+
+                let inner_slice = Some(State { offset, len });
+                let lp = self.pushdown(alp, inner_slice, lp_arena, expr_arena)?;
                 let input = lp_arena.add(lp);
                 Ok(Slice {
                     input,
-                    offset: previous_state.offset,
-                    len: previous_state.len
+                    offset: outer_slice.offset,
+                    len: outer_slice.len
                 })
             }
             (Slice {
                 input,
                 offset,
-                len
+                mut len
             }, None) => {
                 let alp = lp_arena.take(input);
+
+                // If offset is negative the length can never be greater than it.
+                if offset < 0 {
+                    if len as i64 > -offset {
+                        len = (-offset) as IdxSize;
+                    }
+                }
+
                 let state = Some(State {
                     offset,
                     len
