@@ -83,12 +83,7 @@ pub(crate) fn is_column_independent_aexpr(expr: Node, arena: &Arena<AExpr>) -> b
         #[cfg(feature = "dtype-struct")]
         AExpr::Function {
             input: _,
-            function:
-                IRFunctionExpr::StructExpr(
-                    IRStructFunction::FieldByIndex(_)
-                    | IRStructFunction::FieldByName(_)
-                    | IRStructFunction::MultipleFields(_),
-                ),
+            function: IRFunctionExpr::StructExpr(IRStructFunction::FieldByName(_)),
             options: _,
         } => true,
         _ => false,
@@ -129,16 +124,41 @@ pub fn expr_output_name(expr: &Expr) -> PolarsResult<PlSmallStr> {
             Expr::Window { function, .. } => return expr_output_name(function),
             Expr::Column(name) => return Ok(name.clone()),
             Expr::Alias(_, name) => return Ok(name.clone()),
-            Expr::KeepName(_) | Expr::Wildcard | Expr::RenameAlias { .. } => polars_bail!(
-                ComputeError:
-                "cannot determine output column without a context for this expression"
-            ),
-            Expr::Columns(_) | Expr::DtypeColumn(_) | Expr::IndexColumn(_) => polars_bail!(
-                ComputeError:
-                "this expression may produce multiple output names"
-            ),
+            Expr::KeepName(_) => polars_bail!(nyi = "`name.keep` is not allowed here"),
+            Expr::RenameAlias { expr, function } => return function.call(&expr_output_name(expr)?),
             Expr::Len => return Ok(get_len_name()),
             Expr::Literal(val) => return Ok(val.output_column_name().clone()),
+
+            #[cfg(feature = "dtype-struct")]
+            Expr::Function {
+                input: _,
+                function: FunctionExpr::StructExpr(StructFunction::FieldByName(name)),
+            } => return Ok(name.clone()),
+
+            // Selector with single by_name is fine.
+            Expr::Selector(Selector::ByName { names, .. }) if names.len() == 1 => {
+                return Ok(names[0].clone());
+            },
+
+            #[cfg(feature = "dtype-struct")]
+            Expr::Function {
+                function:
+                    FunctionExpr::StructExpr(StructFunction::SelectFields(Selector::ByName {
+                        names,
+                        ..
+                    })),
+                ..
+            } if names.len() == 1 => return Ok(names[0].clone()),
+
+            // Other selectors aren't possible right now.
+            Expr::Selector(_) => break,
+
+            #[cfg(feature = "dtype-struct")]
+            Expr::Function {
+                function: FunctionExpr::StructExpr(StructFunction::SelectFields(_)),
+                ..
+            } => break,
+
             _ => {},
         }
     }
@@ -148,25 +168,6 @@ pub fn expr_output_name(expr: &Expr) -> PolarsResult<PlSmallStr> {
     );
 }
 
-/// This function should be used to find the name of the start of an expression
-/// Normal iteration would just return the first root column it found
-pub(crate) fn get_single_leaf(expr: &Expr) -> PolarsResult<PlSmallStr> {
-    for e in expr {
-        match e {
-            Expr::Filter { input, .. } => return get_single_leaf(input),
-            Expr::Gather { expr, .. } => return get_single_leaf(expr),
-            Expr::SortBy { expr, .. } => return get_single_leaf(expr),
-            Expr::Window { function, .. } => return get_single_leaf(function),
-            Expr::Column(name) => return Ok(name.clone()),
-            Expr::Len => return Ok(get_len_name()),
-            _ => {},
-        }
-    }
-    polars_bail!(
-        ComputeError: "unable to find a single leaf column in expr {:?}", expr
-    );
-}
-
 #[allow(clippy::type_complexity)]
 pub fn expr_to_leaf_column_names_iter(expr: &Expr) -> impl Iterator<Item = PlSmallStr> + '_ {
     expr_to_leaf_column_exprs_iter(expr).flat_map(|e| expr_to_leaf_column_name(e).ok())
@@ -183,8 +184,8 @@ pub fn expr_to_leaf_column_name(expr: &Expr) -> PolarsResult<PlSmallStr> {
     polars_ensure!(leaves.len() <= 1, ComputeError: "found more than one root column name");
     match leaves.pop() {
         Some(Expr::Column(name)) => Ok(name.clone()),
-        Some(Expr::Wildcard) => polars_bail!(
-            ComputeError: "wildcard has no root column name",
+        Some(Expr::Selector(_)) => polars_bail!(
+            ComputeError: "selector has no root column name",
         ),
         Some(_) => unreachable!(),
         None => polars_bail!(
@@ -218,21 +219,17 @@ pub fn column_node_to_name(node: ColumnNode, arena: &Arena<AExpr>) -> &PlSmallSt
 /// Get all leaf column expressions in the expression tree.
 pub(crate) fn expr_to_leaf_column_exprs_iter(expr: &Expr) -> impl Iterator<Item = &Expr> {
     expr.into_iter().flat_map(|e| match e {
-        Expr::Column(_) | Expr::Wildcard => Some(e),
+        Expr::Column(_) => Some(e),
         _ => None,
     })
 }
 
 /// Take a list of expressions and a schema and determine the output schema.
-pub fn expressions_to_schema(
-    expr: &[Expr],
-    schema: &Schema,
-    ctxt: Context,
-) -> PolarsResult<Schema> {
+pub fn expressions_to_schema(expr: &[Expr], schema: &Schema) -> PolarsResult<Schema> {
     let mut expr_arena = Arena::with_capacity(4 * expr.len());
     expr.iter()
         .map(|expr| {
-            let mut field = expr.to_field_amortized(schema, ctxt, &mut expr_arena)?;
+            let mut field = expr.to_field_amortized(schema, &mut expr_arena)?;
 
             field.dtype = field.dtype.materialize_unknown(true)?;
             Ok(field)
@@ -282,29 +279,22 @@ pub(crate) fn check_input_column_node(
 pub(crate) fn aexprs_to_schema<I: IntoIterator<Item = K>, K: Into<Node>>(
     expr: I,
     schema: &Schema,
-    ctxt: Context,
     arena: &Arena<AExpr>,
 ) -> Schema {
     expr.into_iter()
-        .map(|node| {
-            arena
-                .get(node.into())
-                .to_field(schema, ctxt, arena)
-                .unwrap()
-        })
+        .map(|node| arena.get(node.into()).to_field(schema, arena).unwrap())
         .collect()
 }
 
 pub(crate) fn expr_irs_to_schema<I: IntoIterator<Item = K>, K: AsRef<ExprIR>>(
     expr: I,
     schema: &Schema,
-    ctxt: Context,
     arena: &Arena<AExpr>,
 ) -> Schema {
     expr.into_iter()
         .map(|e| {
             let e = e.as_ref();
-            let mut field = e.field(schema, ctxt, arena).expect("should be resolved");
+            let mut field = e.field(schema, arena).expect("should be resolved");
 
             // TODO! (can this be removed?)
             if let Some(name) = e.get_alias() {
