@@ -10,8 +10,6 @@ use crate::nodes::compute_node_prelude::*;
 #[derive(Default)]
 pub(super) struct ChunkReader {
     projected_schema: SchemaRef,
-    #[cfg(feature = "dtype-categorical")]
-    _cat_lock: Option<polars_core::StringCacheHolder>,
     ignore_errors: bool,
 }
 
@@ -22,16 +20,8 @@ impl ChunkReader {
     ) -> PolarsResult<Self> {
         let projected_schema = projected_schema.clone();
 
-        #[cfg(feature = "dtype-categorical")]
-        let _cat_lock = projected_schema
-            .iter_values()
-            .any(|x| x.is_categorical())
-            .then(polars_core::StringCacheHolder::hold);
-
         Ok(Self {
             projected_schema,
-            #[cfg(feature = "dtype-categorical")]
-            _cat_lock,
             ignore_errors: options.ignore_errors,
         })
     }
