@@ -1,151 +1,22 @@
-use std::borrow::Cow;
-use std::ops::Range;
 use std::sync::Arc;
 
-use arrow::datatypes::ArrowDataType;
 use polars_core::frame::DataFrame;
-use polars_core::prelude::{Column, DataType, IDX_DTYPE, IntoColumn};
-use polars_core::series::Series;
-use polars_core::utils::arrow::bitmap::Bitmap;
-use polars_core::utils::arrow::datatypes::ArrowSchemaRef;
 use polars_error::{PolarsResult, polars_ensure};
-use polars_io::RowIndex;
-use polars_io::predicates::ScanIOPredicate;
-use polars_io::prelude::_internal::{PrefilterMaskSetting, collect_statistics_with_live_columns};
-use polars_io::prelude::{FileMetadata, ParallelStrategy};
-use polars_utils::{IdxSize, format_pl_smallstr};
+use polars_io::prelude::_internal::PrefilterMaskSetting;
+use polars_io::prelude::ParallelStrategy;
+use polars_utils::IdxSize;
 
 use super::row_group_data_fetch::RowGroupDataFetcher;
 use super::row_group_decode::RowGroupDecoder;
 use super::{AsyncTaskData, ParquetReadImpl};
 use crate::async_executor;
 use crate::morsel::{Morsel, SourceToken, get_ideal_morsel_size};
-use crate::nodes::io_sources::multi_file_reader::reader_interface::output::FileReaderOutputSend;
+use crate::nodes::io_sources::multi_scan::reader_interface::output::FileReaderOutputSend;
+use crate::nodes::io_sources::parquet::projection::ArrowFieldProjection;
+use crate::nodes::io_sources::parquet::statistics::calculate_row_group_pred_pushdown_skip_mask;
 use crate::nodes::{MorselSeq, TaskPriority};
 use crate::utils::task_handles_ext::{self, AbortOnDropHandle};
 
-async fn calculate_row_group_pred_pushdown_skip_mask(
-    row_group_slice: Range<usize>,
-    use_statistics: bool,
-    predicate: Option<&ScanIOPredicate>,
-    metadata: &Arc<FileMetadata>,
-    reader_schema: &ArrowSchemaRef,
-    mut row_index: Option<RowIndex>,
-    verbose: bool,
-) -> PolarsResult<Option<Bitmap>> {
-    if !use_statistics {
-        return Ok(None);
-    }
-
-    let Some(predicate) = predicate else {
-        return Ok(None);
-    };
-    let Some(sbp) = predicate.skip_batch_predicate.as_ref() else {
-        return Ok(None);
-    };
-    let sbp = sbp.clone();
-
-    let num_row_groups = row_group_slice.len();
-    let metadata = metadata.clone();
-    let live_columns = predicate.live_columns.clone();
-    let reader_schema = reader_schema.clone();
-    let skip_row_group_mask = async_executor::spawn(TaskPriority::High, async move {
-        if let Some(ri) = &mut row_index {
-            for md in metadata.row_groups[0..row_group_slice.start].iter() {
-                ri.offset = ri
-                    .offset
-                    .saturating_add(IdxSize::try_from(md.num_rows()).unwrap_or(IdxSize::MAX));
-            }
-        }
-
-        let stats = collect_statistics_with_live_columns(
-            &metadata.row_groups[row_group_slice.clone()],
-            reader_schema.as_ref(),
-            &live_columns,
-            row_index.as_ref().map(|ri| (&ri.name, ri.offset)),
-        )?;
-
-        let mut columns = Vec::with_capacity(1 + live_columns.len() * 3);
-
-        let lengths: Vec<IdxSize> = metadata.row_groups[row_group_slice.clone()]
-            .iter()
-            .map(|rg| rg.num_rows() as IdxSize)
-            .collect();
-        columns.push(Column::new("len".into(), lengths));
-        for (c, stat) in live_columns.iter().zip(stats) {
-            let field = reader_schema.get(c).map(Cow::Borrowed).unwrap_or_else(|| {
-                let row_index = row_index.clone().unwrap();
-                assert_eq!(c, &row_index.name);
-
-                Cow::Owned(arrow::datatypes::Field {
-                    name: row_index.name,
-                    dtype: ArrowDataType::IDX_DTYPE,
-                    is_nullable: false,
-                    metadata: None,
-                })
-            });
-
-            let min_name = format_pl_smallstr!("{c}_min");
-            let max_name = format_pl_smallstr!("{c}_max");
-            let nc_name = format_pl_smallstr!("{c}_nc");
-
-            let (min, max, nc) = match stat {
-                None => {
-                    let dtype = DataType::from_arrow_field(field.as_ref());
-
-                    (
-                        Column::full_null(min_name, num_row_groups, &dtype),
-                        Column::full_null(max_name, num_row_groups, &dtype),
-                        Column::full_null(nc_name, num_row_groups, &IDX_DTYPE),
-                    )
-                },
-                Some(stat) => {
-                    let md = field.metadata.as_deref();
-
-                    (
-                        unsafe {
-                            Series::_try_from_arrow_unchecked_with_md(
-                                min_name,
-                                vec![stat.min_value],
-                                field.dtype(),
-                                md,
-                            )
-                        }?
-                        .into_column(),
-                        unsafe {
-                            Series::_try_from_arrow_unchecked_with_md(
-                                max_name,
-                                vec![stat.max_value],
-                                field.dtype(),
-                                md,
-                            )
-                        }?
-                        .into_column(),
-                        Series::from_arrow(nc_name, stat.null_count.boxed())?.into_column(),
-                    )
-                },
-            };
-
-            columns.extend([min, max, nc]);
-        }
-
-        let statistics_df = DataFrame::new_with_height(num_row_groups, columns)?;
-        sbp.evaluate_with_stat_df(&statistics_df)
-    })
-    .await?;
-
-    if verbose {
-        eprintln!(
-            "[ParquetFileReader]: Predicate pushdown: \
-                                reading {} / {} row groups",
-            skip_row_group_mask.unset_bits(),
-            num_row_groups,
-        );
-    }
-
-    Ok(Some(skip_row_group_mask))
-}
-
 impl ParquetReadImpl {
     /// Constructs the task that distributes morsels across the engine pipelines.
     #[allow(clippy::type_complexity)]
@@ -164,13 +35,10 @@ impl ParquetReadImpl {
             );
         }
 
-        let reader_schema = self.schema.clone();
+        let projected_arrow_fields = self.projected_arrow_fields.clone();
+        let is_full_projection = self.is_full_projection;
 
         let row_group_prefetch_size = self.config.row_group_prefetch_size;
-        // For row group fetching, only set this if we have a projection, as it will cause individual
-        // byte range requests for every column in the row group.
-        let projection = (self.projected_arrow_schema.len() < self.schema.len())
-            .then_some(self.projected_arrow_schema.clone());
         let predicate = self.predicate.clone();
         let memory_prefetch_func = self.memory_prefetch_func;
 
@@ -241,7 +109,7 @@ impl ParquetReadImpl {
                 if verbose {
                     eprintln!(
                         "[ParquetFileReader]: Slice pushdown: \
-                            reading {} / {} row groups",
+                        reading {} / {} row groups",
                         row_group_slice.len(),
                         metadata.row_groups.len()
                     );
@@ -253,14 +121,15 @@ impl ParquetReadImpl {
                 use_statistics,
                 predicate.as_ref(),
                 &metadata,
-                &reader_schema,
+                projected_arrow_fields.clone(),
                 row_index,
                 verbose,
             )
             .await?;
 
             let mut row_group_data_fetcher = RowGroupDataFetcher {
-                projection,
+                projection: projected_arrow_fields.clone(),
+                is_full_projection,
                 predicate,
                 slice_range,
                 memory_prefetch_func,
@@ -363,84 +232,94 @@ impl ParquetReadImpl {
 
     /// Creates a `RowGroupDecoder` that turns `RowGroupData` into DataFrames.
     /// This must be called AFTER the following have been initialized:
-    /// * `self.projected_arrow_schema`
+    /// * `self.projected_arrow_fields`
     /// * `self.physical_predicate`
-    pub(super) fn init_row_group_decoder(&self) -> RowGroupDecoder {
-        let projected_arrow_schema = self.projected_arrow_schema.clone();
+    pub(super) fn init_row_group_decoder(&mut self) -> RowGroupDecoder {
+        let projected_arrow_fields = self.projected_arrow_fields.clone();
         let row_index = self.row_index.clone();
-        let min_values_per_thread = self.config.min_values_per_thread;
+        let target_values_per_thread = self.config.target_values_per_thread;
+        let predicate = self.predicate.clone();
 
         let mut use_prefiltered = matches!(self.options.parallel, ParallelStrategy::Prefiltered);
         use_prefiltered |=
-            self.predicate.is_some() && matches!(self.options.parallel, ParallelStrategy::Auto);
+            predicate.is_some() && matches!(self.options.parallel, ParallelStrategy::Auto);
 
-        let mut predicate_arrow_field_indices = vec![];
-        if use_prefiltered {
-            if let Some(predicate) = self.predicate.as_ref() {
-                predicate_arrow_field_indices = predicate
-                    .live_columns
+        let predicate_field_indices: Arc<[usize]> =
+            if use_prefiltered && let Some(predicate) = predicate.as_ref() {
+                projected_arrow_fields
                     .iter()
-                    // Can be `None` - if the column is e.g. a hive column, or the row index column.
-                    .filter_map(|x| projected_arrow_schema.index_of(x))
-                    .collect::<Vec<_>>();
-                predicate_arrow_field_indices.sort_unstable();
-            }
-        }
+                    .enumerate()
+                    .filter_map(|(i, projected_field)| {
+                        predicate
+                            .live_columns
+                            .contains(projected_field.output_name())
+                            .then_some(i)
+                    })
+                    .collect()
+            } else {
+                Default::default()
+            };
 
         let use_prefiltered = use_prefiltered.then(PrefilterMaskSetting::init_from_env);
 
-        let non_predicate_arrow_field_indices = if use_prefiltered.is_some() {
+        let non_predicate_field_indices: Arc<[usize]> = if use_prefiltered.is_some() {
             filtered_range(
-                predicate_arrow_field_indices.as_slice(),
-                projected_arrow_schema.len(),
+                predicate_field_indices.as_ref(),
+                projected_arrow_fields.len(),
             )
+            .collect()
         } else {
-            vec![]
+            Default::default()
         };
 
         if use_prefiltered.is_some() && self.verbose {
             eprintln!(
                 "[ParquetFileReader]: Pre-filtered decode enabled ({} live, {} non-live)",
-                predicate_arrow_field_indices.len(),
-                non_predicate_arrow_field_indices.len()
+                predicate_field_indices.len(),
+                non_predicate_field_indices.len()
             )
         }
 
-        let predicate_arrow_field_indices = Arc::new(predicate_arrow_field_indices);
-        let non_predicate_arrow_field_indices = Arc::new(non_predicate_arrow_field_indices);
+        let allow_column_predicates = predicate
+            .as_ref()
+            .is_some_and(|x| x.column_predicates.is_sumwise_complete)
+            && row_index.is_none()
+            && !projected_arrow_fields.iter().any(|x| {
+                x.arrow_field().dtype().is_nested()
+                    || matches!(x, ArrowFieldProjection::Mapped { .. })
+            });
 
         RowGroupDecoder {
             num_pipelines: self.config.num_pipelines,
-            projected_arrow_schema,
+            projected_arrow_fields,
             row_index,
-            predicate: self.predicate.clone(),
+            predicate,
+            allow_column_predicates,
             use_prefiltered,
-            predicate_arrow_field_indices,
-            non_predicate_arrow_field_indices,
-            min_values_per_thread,
+            predicate_field_indices,
+            non_predicate_field_indices,
+            target_values_per_thread,
         }
     }
 }
 
 /// Returns 0..len in a Vec, excluding indices in `exclude`.
 /// `exclude` needs to be a sorted list of unique values.
-fn filtered_range(exclude: &[usize], len: usize) -> Vec<usize> {
+fn filtered_range(exclude: &[usize], len: usize) -> impl Iterator<Item = usize> {
     if cfg!(debug_assertions) {
         assert!(exclude.windows(2).all(|x| x[1] > x[0]));
     }
 
     let mut j = 0;
 
-    (0..len)
-        .filter(|&i| {
-            if j == exclude.len() || i != exclude[j] {
-                true
-            } else {
-                j += 1;
-                false
-            }
-        })
-        .collect()
+    (0..len).filter(move |&i| {
+        if j == exclude.len() || i != exclude[j] {
+            true
+        } else {
+            j += 1;
+            false
+        }
+    })
 }
 
 fn split_to_morsels(
@@ -473,7 +352,13 @@ mod tests {
     #[test]
     fn test_filtered_range() {
         use super::filtered_range;
-        assert_eq!(filtered_range(&[1, 3], 7).as_slice(), &[0, 2, 4, 5, 6]);
-        assert_eq!(filtered_range(&[1, 6], 7).as_slice(), &[0, 2, 3, 4, 5]);
+        assert_eq!(
+            filtered_range(&[1, 3], 7).collect::<Vec<_>>().as_slice(),
+            &[0, 2, 4, 5, 6]
+        );
+        assert_eq!(
+            filtered_range(&[1, 6], 7).collect::<Vec<_>>().as_slice(),
+            &[0, 2, 3, 4, 5]
+        );
     }
 }
