@@ -1,7 +1,7 @@
 use std::cmp::Reverse;
 use std::collections::BinaryHeap;
 use std::sync::Arc;
-use std::sync::atomic::{AtomicU64, AtomicUsize, Ordering};
+use std::sync::atomic::{AtomicU64, Ordering};
 
 use arrow::array::builder::ShareStrategy;
 use polars_core::frame::builder::DataFrameBuilder;
@@ -18,6 +18,7 @@ use polars_utils::hashing::HashPartitioner;
 use polars_utils::itertools::Itertools;
 use polars_utils::pl_str::PlSmallStr;
 use polars_utils::priority::Priority;
+use polars_utils::relaxed_cell::RelaxedCell;
 use polars_utils::sparse_init_vec::SparseInitVec;
 use polars_utils::{IdxSize, format_pl_smallstr};
 use rayon::prelude::*;
@@ -74,6 +75,7 @@ fn compute_payload_selector(
     this: &Schema,
     other: &Schema,
     this_key_schema: &Schema,
+    other_key_schema: &Schema,
     is_left: bool,
     args: &JoinArgs,
 ) -> PolarsResult<Vec<Option<PlSmallStr>>> {
@@ -82,31 +84,51 @@ fn compute_payload_selector(
     let mut coalesce_idx = 0;
     this.iter_names()
         .map(|c| {
-            let selector = if should_coalesce && this_key_schema.contains(c) {
-                if is_left != (args.how == JoinType::Right) {
+            #[expect(clippy::never_loop)]
+            loop {
+                let selector = if args.how == JoinType::Right {
+                    if is_left {
+                        if should_coalesce && this_key_schema.contains(c) {
+                            // Coalesced to RHS output key.
+                            None
+                        } else {
+                            Some(c.clone())
+                        }
+                    } else if !other.contains(c) || (should_coalesce && other_key_schema.contains(c)) {
+                        Some(c.clone())
+                    } else {
+                        break;
+                    }
+                } else if should_coalesce && this_key_schema.contains(c) {
+                    if is_left {
+                        Some(c.clone())
+                    } else if args.how == JoinType::Full {
+                        // We must keep the right-hand side keycols around for
+                        // coalescing.
+                        let name = format_pl_smallstr!("__POLARS_COALESCE_KEYCOL{coalesce_idx}");
+                        coalesce_idx += 1;
+                        Some(name)
+                    } else {
+                        None
+                    }
+                } else if !other.contains(c) || is_left {
                     Some(c.clone())
-                } else if args.how == JoinType::Full {
-                    // We must keep the right-hand side keycols around for
-                    // coalescing.
-                    let name = format_pl_smallstr!("__POLARS_COALESCE_KEYCOL{coalesce_idx}");
-                    coalesce_idx += 1;
-                    Some(name)
                 } else {
-                    None
-                }
-            } else if !other.contains(c) || is_left {
-                Some(c.clone())
-            } else {
-                let suffixed = format_pl_smallstr!("{}{}", c, args.suffix());
-                if other.contains(&suffixed) {
-                    polars_bail!(Duplicate: "column with name '{suffixed}' already exists\n\n\
-                    You may want to try:\n\
-                    - renaming the column prior to joining\n\
-                    - using the `suffix` parameter to specify a suffix different to the default one ('_right')")
-                }
-                Some(suffixed)
-            };
-            Ok(selector)
+                    break;
+                };
+
+                return Ok(selector);
+            }
+
+            let suffixed = format_pl_smallstr!("{}{}", c, args.suffix());
+            if other.contains(&suffixed) {
+                polars_bail!(Duplicate: "column with name '{suffixed}' already exists\n\n\
+                You may want to try:\n\
+                - renaming the column prior to joining\n\
+                - using the `suffix` parameter to specify a suffix different to the default one ('_right')")
+            }
+
+            Ok(Some(suffixed))
         })
         .collect()
 }
@@ -243,15 +265,15 @@ impl SampleState {
         mut recv: Receiver<Morsel>,
         morsels: &mut Vec<Morsel>,
         len: &mut usize,
-        this_final_len: Arc<AtomicUsize>,
-        other_final_len: Arc<AtomicUsize>,
+        this_final_len: Arc<RelaxedCell<usize>>,
+        other_final_len: Arc<RelaxedCell<usize>>,
     ) -> PolarsResult<()> {
         while let Ok(mut morsel) = recv.recv().await {
             *len += morsel.df().height();
             if *len >= *JOIN_SAMPLE_LIMIT
                 || *len
                     >= other_final_len
-                        .load(Ordering::Relaxed)
+                        .load()
                         .saturating_mul(LOPSIDED_SAMPLE_FACTOR)
             {
                 morsel.source_token().stop();
@@ -260,7 +282,7 @@ impl SampleState {
             drop(morsel.take_consume_token());
             morsels.push(morsel);
         }
-        this_final_len.store(*len, Ordering::Relaxed);
+        this_final_len.store(*len);
         Ok(())
     }
 
@@ -811,6 +833,7 @@ impl ProbeState {
                     // consecutively on the same partition.
                     probe_partitions.clear();
                     hash_keys.gen_partitions(&partitioner, &mut probe_partitions, emit_unmatched);
+
                     let mut probe_group_start = 0;
                     while probe_group_start < probe_partitions.len() {
                         let p_idx = probe_partitions[probe_group_start];
@@ -950,17 +973,17 @@ impl ProbeState {
                             }
                         }
                     }
+                }
 
-                    if !probe_match.is_empty() {
-                        if !payload_rechunked {
-                            payload.rechunk_mut();
-                        }
-                        probe_out.gather_extend(&payload, &probe_match, ShareStrategy::Always);
-                        probe_match.clear();
-                        let out_morsel = new_morsel(&mut build_out, &mut probe_out);
-                        if send.send(out_morsel).await.is_err() {
-                            return Ok(max_seq);
-                        }
+                if !probe_match.is_empty() {
+                    if !payload_rechunked {
+                        payload.rechunk_mut();
+                    }
+                    probe_out.gather_extend(&payload, &probe_match, ShareStrategy::Always);
+                    probe_match.clear();
+                    let out_morsel = new_morsel(&mut build_out, &mut probe_out);
+                    if send.send(out_morsel).await.is_err() {
+                        return Ok(max_seq);
                     }
                 }
             }
@@ -1181,6 +1204,7 @@ impl EquiJoinNode {
             &left_input_schema,
             &right_input_schema,
             &left_key_schema,
+            &right_key_schema,
             true,
             &args,
         )?;
@@ -1188,6 +1212,7 @@ impl EquiJoinNode {
             &right_input_schema,
             &left_input_schema,
             &right_key_schema,
+            &left_key_schema,
             false,
             &args,
         )?;
@@ -1397,12 +1422,12 @@ impl ComputeNode for EquiJoinNode {
         match &mut self.state {
             EquiJoinState::Sample(sample_state) => {
                 assert!(send_ports[0].is_none());
-                let left_final_len = Arc::new(AtomicUsize::new(if recv_ports[0].is_none() {
+                let left_final_len = Arc::new(RelaxedCell::from(if recv_ports[0].is_none() {
                     sample_state.left_len
                 } else {
                     usize::MAX
                 }));
-                let right_final_len = Arc::new(AtomicUsize::new(if recv_ports[1].is_none() {
+                let right_final_len = Arc::new(RelaxedCell::from(if recv_ports[1].is_none() {
                     sample_state.right_len
                 } else {
                     usize::MAX
