@@ -1,14 +1,15 @@
 use std::sync::Arc;
 
 use parking_lot::Mutex;
-use polars_core::prelude::{InitHashMaps, PlIndexMap};
+use polars_core::frame::DataFrame;
+use polars_core::prelude::{InitHashMaps, PlIndexMap, SortMultipleOptions};
 use polars_core::schema::Schema;
 use polars_error::{PolarsResult, polars_err};
 use polars_expr::state::ExecutionState;
 use polars_mem_engine::create_physical_plan;
 use polars_plan::plans::expr_ir::{ExprIR, OutputName};
-use polars_plan::plans::{AExpr, DataFrameUdf, IR, IRAggExpr, NaiveExprMerger, write_group_by};
-use polars_plan::prelude::GroupbyOptions;
+use polars_plan::plans::{AExpr, IR, IRAggExpr, IRFunctionExpr, NaiveExprMerger, write_group_by};
+use polars_plan::prelude::{GroupbyOptions, *};
 use polars_utils::arena::{Arena, Node};
 use polars_utils::pl_str::PlSmallStr;
 use polars_utils::unique_column_name;
@@ -20,7 +21,7 @@ use crate::physical_plan::lower_expr::{
     build_select_stream, compute_output_schema, is_elementwise_rec_cached,
     is_fake_elementwise_function, is_input_independent,
 };
-use crate::physical_plan::lower_ir::build_slice_stream;
+use crate::physical_plan::lower_ir::{build_row_idx_stream, build_slice_stream};
 use crate::utils::late_materialized_df::LateMaterializedDataFrame;
 
 #[allow(clippy::too_many_arguments)]
@@ -31,7 +32,7 @@ fn build_group_by_fallback(
     output_schema: Arc<Schema>,
     maintain_order: bool,
     options: Arc<GroupbyOptions>,
-    apply: Option<Arc<dyn DataFrameUdf>>,
+    apply: Option<PlanCallback<DataFrame, DataFrame>>,
     expr_arena: &mut Arena<AExpr>,
     phys_sm: &mut SlotMap<PhysNodeKey, PhysNode>,
     format_str: Option<String>,
@@ -39,7 +40,7 @@ fn build_group_by_fallback(
     let input_schema = phys_sm[input.node].output_schema.clone();
     let lmdf = Arc::new(LateMaterializedDataFrame::default());
     let mut lp_arena = Arena::default();
-    let input_lp_node = lp_arena.add(lmdf.clone().as_ir_node(input_schema.clone()));
+    let input_lp_node = lp_arena.add(lmdf.clone().as_ir_node(input_schema));
     let group_by_lp_node = lp_arena.add(IR::GroupBy {
         input: input_lp_node,
         keys: keys.to_vec(),
@@ -167,6 +168,117 @@ fn try_lower_elementwise_scalar_agg_expr(
             }))
         },
 
+        #[cfg(feature = "bitwise")]
+        AExpr::Function {
+            input: inner_exprs,
+            function:
+                IRFunctionExpr::Bitwise(
+                    inner_fn @ (IRBitwiseFunction::And
+                    | IRBitwiseFunction::Or
+                    | IRBitwiseFunction::Xor),
+                ),
+            options,
+        } => {
+            assert!(inner_exprs.len() == 1);
+
+            let input = inner_exprs[0].clone().node();
+            let inner_fn = *inner_fn;
+            let options = *options;
+
+            if is_input_independent(input, expr_arena, expr_cache) {
+                // TODO: we could simply return expr here, but we first need an is_scalar function, because if
+                // it is not a scalar we need to return expr.implode().
+                return None;
+            }
+
+            if !is_elementwise_rec_cached(input, expr_arena, expr_cache) {
+                return None;
+            }
+
+            let agg_id = expr_merger.get_uniq_id(expr).unwrap();
+            let name = uniq_agg_exprs
+                .entry(agg_id)
+                .or_insert_with(|| {
+                    let input_id = expr_merger.get_uniq_id(input).unwrap();
+                    let input_col = uniq_input_exprs
+                        .entry(input_id)
+                        .or_insert_with(unique_column_name)
+                        .clone();
+                    let input_col_node = expr_arena.add(AExpr::Column(input_col));
+                    let trans_agg_node = expr_arena.add(AExpr::Function {
+                        input: vec![ExprIR::from_node(input_col_node, expr_arena)],
+                        function: IRFunctionExpr::Bitwise(inner_fn),
+                        options,
+                    });
+
+                    // Add to aggregation expressions and replace with a reference to its output.
+                    let agg_expr = if let Some(name) = outer_name {
+                        ExprIR::new(trans_agg_node, OutputName::Alias(name))
+                    } else {
+                        ExprIR::new(trans_agg_node, OutputName::Alias(unique_column_name()))
+                    };
+                    agg_exprs.push(agg_expr.clone());
+                    agg_expr.output_name().clone()
+                })
+                .clone();
+            let result_node = expr_arena.add(AExpr::Column(name));
+            Some(result_node)
+        },
+
+        AExpr::Function {
+            input: inner_exprs,
+            function:
+                IRFunctionExpr::Boolean(
+                    inner_fn @ (IRBooleanFunction::Any { .. } | IRBooleanFunction::All { .. }),
+                ),
+            options,
+        } => {
+            assert!(inner_exprs.len() == 1);
+
+            let input = inner_exprs[0].clone().node();
+            let inner_fn = inner_fn.clone();
+            let options = *options;
+
+            if is_input_independent(input, expr_arena, expr_cache) {
+                // TODO: we could simply return expr here, but we first need an is_scalar function, because if
+                // it is not a scalar we need to return expr.implode().
+                return None;
+            }
+
+            if !is_elementwise_rec_cached(input, expr_arena, expr_cache) {
+                return None;
+            }
+
+            let agg_id = expr_merger.get_uniq_id(expr).unwrap();
+            let name = uniq_agg_exprs
+                .entry(agg_id)
+                .or_insert_with(|| {
+                    let input_id = expr_merger.get_uniq_id(input).unwrap();
+                    let input_col = uniq_input_exprs
+                        .entry(input_id)
+                        .or_insert_with(unique_column_name)
+                        .clone();
+                    let input_col_node = expr_arena.add(AExpr::Column(input_col));
+                    let trans_agg_node = expr_arena.add(AExpr::Function {
+                        input: vec![ExprIR::from_node(input_col_node, expr_arena)],
+                        function: IRFunctionExpr::Boolean(inner_fn),
+                        options,
+                    });
+
+                    // Add to aggregation expressions and replace with a reference to its output.
+                    let agg_expr = if let Some(name) = outer_name {
+                        ExprIR::new(trans_agg_node, OutputName::Alias(name))
+                    } else {
+                        ExprIR::new(trans_agg_node, OutputName::Alias(unique_column_name()))
+                    };
+                    agg_exprs.push(agg_expr.clone());
+                    agg_expr.output_name().clone()
+                })
+                .clone();
+            let result_node = expr_arena.add(AExpr::Column(name));
+            Some(result_node)
+        },
+
         node @ AExpr::Function { input, options, .. }
         | node @ AExpr::AnonymousFunction { input, options, .. }
             if options.is_elementwise() && !is_fake_elementwise_function(node) =>
@@ -185,7 +297,7 @@ fn try_lower_elementwise_scalar_agg_expr(
                 })
                 .collect::<Option<Vec<_>>>()?;
 
-            let mut new_node = node.clone();
+            let mut new_node = node;
             match &mut new_node {
                 AExpr::Function { input, .. } | AExpr::AnonymousFunction { input, .. } => {
                     *input = new_input;
@@ -221,7 +333,7 @@ fn try_lower_elementwise_scalar_agg_expr(
                 | IRAggExpr::Sum(input)
                 | IRAggExpr::Var(input, ..)
                 | IRAggExpr::Std(input, ..)
-                | IRAggExpr::Count(input, ..) => {
+                | IRAggExpr::Count { input, .. } => {
                     let agg = agg.clone();
                     let input = *input;
                     if is_input_independent(input, expr_arena, expr_cache) {
@@ -244,7 +356,7 @@ fn try_lower_elementwise_scalar_agg_expr(
                                 .entry(input_id)
                                 .or_insert_with(unique_column_name)
                                 .clone();
-                            let input_col_node = expr_arena.add(AExpr::Column(input_col.clone()));
+                            let input_col_node = expr_arena.add(AExpr::Column(input_col));
                             trans_agg.set_input(input_col_node);
                             let trans_agg_node = expr_arena.add(AExpr::Agg(trans_agg));
 
@@ -284,18 +396,18 @@ fn try_lower_elementwise_scalar_agg_expr(
 
 #[allow(clippy::too_many_arguments)]
 fn try_build_streaming_group_by(
-    input: PhysStream,
+    mut input: PhysStream,
     keys: &[ExprIR],
     aggs: &[ExprIR],
     maintain_order: bool,
     options: Arc<GroupbyOptions>,
-    apply: Option<Arc<dyn DataFrameUdf>>,
+    apply: Option<PlanCallback<DataFrame, DataFrame>>,
     expr_arena: &mut Arena<AExpr>,
     phys_sm: &mut SlotMap<PhysNodeKey, PhysNode>,
     expr_cache: &mut ExprCache,
     ctx: StreamingLowerIRContext,
 ) -> Option<PolarsResult<PhysStream>> {
-    if apply.is_some() || maintain_order {
+    if apply.is_some() {
         return None; // TODO
     }
 
@@ -310,6 +422,20 @@ fn try_build_streaming_group_by(
         ));
     }
 
+    // Augment with row index if maintaining order.
+    let row_idx_name = unique_column_name();
+    let row_idx_node = expr_arena.add(AExpr::Column(row_idx_name.clone()));
+    let mut agg_storage;
+    let aggs = if maintain_order {
+        input = build_row_idx_stream(input, row_idx_name.clone(), None, phys_sm);
+        let first_agg_node = expr_arena.add(AExpr::Agg(IRAggExpr::First(row_idx_node)));
+        agg_storage = aggs.to_vec();
+        agg_storage.push(ExprIR::from_node(first_agg_node, expr_arena));
+        &agg_storage
+    } else {
+        aggs
+    };
+
     let all_independent = keys
         .iter()
         .chain(aggs.iter())
@@ -375,6 +501,19 @@ fn try_build_streaming_group_by(
         input_exprs.push(ExprIR::new(node, OutputName::Alias(name.clone())));
     }
 
+    // If all inputs are input independent add a dummy column so the group sizes are correct. See #23868.
+    if input_exprs
+        .iter()
+        .all(|e| is_input_independent(e.node(), expr_arena, expr_cache))
+    {
+        let dummy_col_name = phys_sm[input.node].output_schema.get_at_index(0).unwrap().0;
+        let dummy_col = expr_arena.add(AExpr::Column(dummy_col_name.clone()));
+        input_exprs.push(ExprIR::new(
+            dummy_col,
+            OutputName::ColumnLhs(dummy_col_name.clone()),
+        ));
+    }
+
     let pre_select =
         build_select_stream(input, &input_exprs, expr_arena, phys_sm, expr_cache, ctx).ok()?;
 
@@ -386,7 +525,7 @@ fn try_build_streaming_group_by(
     )
     .unwrap();
     let agg_node = phys_sm.insert(PhysNode::new(
-        group_by_output_schema,
+        group_by_output_schema.clone(),
         PhysNodeKind::GroupBy {
             input: pre_select,
             key: trans_keys,
@@ -394,14 +533,32 @@ fn try_build_streaming_group_by(
         },
     ));
 
+    // Sort the input based on the first row index if maintaining order.
+    let post_select_input = if maintain_order {
+        let sort_node = phys_sm.insert(PhysNode::new(
+            group_by_output_schema,
+            PhysNodeKind::Sort {
+                input: PhysStream::first(agg_node),
+                by_column: vec![ExprIR::from_node(row_idx_node, expr_arena)],
+                slice: None,
+                sort_options: SortMultipleOptions::new(),
+            },
+        ));
+        trans_output_exprs.pop(); // Remove row idx from post-select.
+        PhysStream::first(sort_node)
+    } else {
+        PhysStream::first(agg_node)
+    };
+
     let post_select = build_select_stream(
-        PhysStream::first(agg_node),
+        post_select_input,
         &trans_output_exprs,
         expr_arena,
         phys_sm,
         expr_cache,
         ctx,
     );
+
     let out = if let Some((offset, len)) = options.slice {
         post_select.map(|s| build_slice_stream(s, offset, len, phys_sm))
     } else {
@@ -418,7 +575,7 @@ pub fn build_group_by_stream(
     output_schema: Arc<Schema>,
     maintain_order: bool,
     options: Arc<GroupbyOptions>,
-    apply: Option<Arc<dyn DataFrameUdf>>,
+    apply: Option<PlanCallback<DataFrame, DataFrame>>,
     expr_arena: &mut Arena<AExpr>,
     phys_sm: &mut SlotMap<PhysNodeKey, PhysNode>,
     expr_cache: &mut ExprCache,
@@ -447,7 +604,7 @@ pub fn build_group_by_stream(
                 expr_arena,
                 keys,
                 aggs,
-                apply.as_deref(),
+                apply.as_ref(),
                 maintain_order,
             )
             .unwrap();
