@@ -1,4 +1,14 @@
-use criterion::{criterion_group, criterion_main, Criterion};
+//! # Benchmarks
+//!
+//! Setup and configure benchmarks to compare the performance of our skiplist
+//! implementation against the standard library's data structures.
+
+#![expect(
+    missing_docs,
+    reason = "benchmarking and criterion creates boilerplate"
+)]
+
+use criterion::{Criterion, criterion_group, criterion_main};
 
 mod btreemap;
 mod hashmap;
@@ -9,7 +19,6 @@ mod skipmap;
 mod vec;
 mod vecdeque;
 
-/// Group Benchmarks
 criterion_group!(
     name = benches;
     config = Criterion::default();
@@ -45,5 +54,4 @@ criterion_group!(
     crate::vecdeque::rand_access,
 );
 
-/// Benchmarks
 criterion_main!(benches);
