@@ -1,16 +1,19 @@
 //! A skiplist implementation which allows faster random access than a standard linked list.
 
+#![allow(warnings)]
+#![allow(clippy)]
+#![allow(unknown_lints)]
+
 use std::{
     cmp, cmp::Ordering, default, fmt, hash, hash::Hash, iter, ops, ops::Bound, ptr::NonNull,
 };
 
+pub use crate::skipnode::{IntoIter, Iter, IterMut};
 use crate::{
-    level_generator::{GeometricalLevelGenerator, LevelGenerator},
+    level_generator::{LevelGenerator, geometric::Geometric},
     skipnode::SkipNode,
 };
 
-pub use crate::skipnode::{IntoIter, Iter, IterMut};
-
 // ////////////////////////////////////////////////////////////////////////////
 // SkipList
 // ////////////////////////////////////////////////////////////////////////////
@@ -20,11 +23,11 @@ pub use crate::skipnode::{IntoIter, Iter, IterMut};
 ///
 /// Unlike a standard linked list, the skiplist can skip ahead when trying to
 /// find a particular index.
-pub struct SkipList<T> {
+pub struct SkipList<T, Gen = Geometric> {
     // Storage, this is not sorted
     head: Box<SkipNode<T>>,
     len: usize,
-    level_generator: GeometricalLevelGenerator,
+    level_generator: Gen,
 }
 
 // ///////////////////////////////////////////////
@@ -43,12 +46,9 @@ impl<T> SkipList<T> {
     /// ```
     #[inline]
     pub fn new() -> Self {
-        let lg = GeometricalLevelGenerator::new(16, 1.0 / 2.0);
-        SkipList {
-            head: Box::new(SkipNode::head(lg.total())),
-            len: 0,
-            level_generator: lg,
-        }
+        // Parameters are fixed and will produce a valid level generator.
+        let lg = Geometric::new(16, 1.0 / 2.0).expect("Failed to create level generator.");
+        Self::new_from_gen(lg)
     }
 
     /// Constructs a new, empty skiplist with the optimal number of levels for
@@ -67,7 +67,28 @@ impl<T> SkipList<T> {
     #[inline]
     pub fn with_capacity(capacity: usize) -> Self {
         let levels = cmp::max(1, (capacity as f64).log2().floor() as usize);
-        let lg = GeometricalLevelGenerator::new(levels, 1.0 / 2.0);
+        // Parameters are safe as levels >= 1 and p is in (0, 1).
+        let lg = Geometric::new(levels, 1.0 / 2.0).expect("Failed to create level generator.");
+        Self::new_from_gen(lg)
+    }
+}
+
+impl<T, G> SkipList<T, G>
+where
+    G: LevelGenerator,
+{
+    /// Constructs a new, empty skip list using a given [LevelGenerator]
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use skiplist::{SkipList, level_generator::GeometricalLevelGenerator};
+    ///
+    /// let lg = GeometricalLevelGenerator::new(10, 0.5);
+    /// let mut skiplist: SkipList<i64> = SkipList::new_from_gen(lg);
+    /// ```
+    #[inline]
+    pub fn new_from_gen(lg: G) -> Self {
         SkipList {
             head: Box::new(SkipNode::head(lg.total())),
             len: 0,
@@ -151,7 +172,7 @@ impl<T> SkipList<T> {
             panic!("Index out of bounds.");
         }
         self.len += 1;
-        let new_node = Box::new(SkipNode::new(value, self.level_generator.random()));
+        let new_node = Box::new(SkipNode::new(value, self.level_generator.level()));
         self.head
             .insert_at(new_node, index)
             .unwrap_or_else(|_| panic!("No insertion position is found!"));
@@ -205,11 +226,7 @@ impl<T> SkipList<T> {
     /// ```
     #[inline]
     pub fn front(&self) -> Option<&T> {
-        if self.is_empty() {
-            None
-        } else {
-            self.get(0)
-        }
+        if self.is_empty() { None } else { self.get(0) }
     }
 
     /// Provides a mutable reference to the front element, or `None` if the
@@ -254,11 +271,7 @@ impl<T> SkipList<T> {
     #[inline]
     pub fn back(&self) -> Option<&T> {
         let len = self.len();
-        if len > 0 {
-            self.get(len - 1)
-        } else {
-            None
-        }
+        if len > 0 { self.get(len - 1) } else { None }
     }
 
     /// Provides a reference to the back element, or `None` if the skiplist is
@@ -279,11 +292,7 @@ impl<T> SkipList<T> {
     #[inline]
     pub fn back_mut(&mut self) -> Option<&mut T> {
         let len = self.len();
-        if len > 0 {
-            self.get_mut(len - 1)
-        } else {
-            None
-        }
+        if len > 0 { self.get_mut(len - 1) } else { None }
     }
 
     /// Provides a reference to the element at the given index, or `None` if the
@@ -597,9 +606,10 @@ impl<T> SkipList<T> {
     }
 }
 
-impl<T> SkipList<T>
+impl<T, G> SkipList<T, G>
 where
     T: PartialEq,
+    G: LevelGenerator,
 {
     /// Returns true if the value is contained in the skiplist.
     ///
@@ -617,6 +627,26 @@ where
         self.iter().any(|val| val.eq(value))
     }
 
+    /// Returns the index of the value if it's contained in the skiplist or None otherwise.
+    ///
+    /// Since this skip list is not ordered, this method has a complexity of O(n)
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use skiplist::SkipList;
+    ///
+    /// let mut skiplist = SkipList::new();
+    /// skiplist.extend(0..10);
+    /// assert_eq!(skiplist.index_of(&4), 4);
+    /// ```
+    #[inline]
+    pub fn index_of(&self, item: &T) -> Option<usize> {
+        self.iter()
+            .enumerate()
+            .find_map(|(idx, it)| if item.eq(it) { Some(idx) } else { None })
+    }
+
     /// Removes all consecutive repeated elements in the skiplist.
     ///
     /// # Examples
@@ -634,7 +664,7 @@ where
     pub fn dedup(&mut self) {
         let removed = self
             .head
-            .retain(|prev, current| prev.map_or(true, |prev| !prev.eq(current)));
+            .retain(|prev, current| prev.is_none_or(|prev| !prev.eq(current)));
         self.len -= removed;
     }
 }
@@ -643,7 +673,10 @@ where
 // Internal methods
 // ///////////////////////////////////////////////
 
-impl<T> SkipList<T> {
+impl<T, G> SkipList<T, G>
+where
+    G: LevelGenerator,
+{
     /// Checks the integrity of the skiplist.
     #[allow(dead_code)]
     fn check(&self) {
@@ -735,10 +768,11 @@ where
                 .collect();
 
             loop {
-                let value = if let Some(ref v) = node.item {
-                    format!("> [{:?}]", v)
-                } else {
-                    "> []".to_string()
+                let value = match node.item {
+                    Some(ref v) => {
+                        format!("> [{:?}]", v)
+                    }
+                    _ => "> []".to_string(),
                 };
 
                 let max_str_len = format!("{} -{}-", value, node.links_len[node.level]).len() + 1;
@@ -761,7 +795,7 @@ where
                         dashes.push('-');
                     }
 
-                    if lvl <= (*node).level {
+                    if lvl <= node.level {
                         rows[lvl].push_str(value_len.as_ref());
                     } else {
                         rows[lvl].push_str(dashes.as_ref());
@@ -946,9 +980,16 @@ impl<T: Hash> Hash for SkipList<T> {
 
 #[cfg(test)]
 mod tests {
-    use super::SkipList;
     use std::ops::Bound::{self, Excluded, Included, Unbounded};
 
+    use anyhow::Result;
+    use rand::{
+        Rng,
+        distr::{Uniform, uniform::UniformUsize},
+    };
+
+    use super::SkipList;
+
     #[test]
     fn push_front() {
         let mut sl = SkipList::new();
@@ -969,18 +1010,17 @@ mod tests {
     }
 
     #[test]
-    fn insert_rand() {
-        use rand::distributions::Uniform;
-        use rand::Rng;
+    fn insert_rand() -> Result<()> {
         let mut rng = rand::thread_rng();
         let mut sl: SkipList<usize> = SkipList::new();
         let mut vec: Vec<usize> = Vec::new();
         for i in 0..100 {
-            let idx = rng.sample(Uniform::new_inclusive(0, i));
+            let idx = rng.sample(Uniform::new_inclusive(0, i)?);
             sl.insert(i, idx);
             vec.insert(idx, i);
         }
         assert_eq!(sl.into_iter().collect::<Vec<_>>(), vec);
+        Ok(())
     }
 
     #[test]
@@ -996,16 +1036,15 @@ mod tests {
     }
 
     #[test]
-    fn remove_rand() {
-        use rand::distributions::Uniform;
-        use rand::Rng;
+    fn remove_rand() -> Result<()> {
         let mut rng = rand::thread_rng();
         let mut v: Vec<i32> = (0..1000).collect();
         let mut sl: SkipList<i32> = (0..1000).collect();
         for i in (0..1000).rev() {
-            let idx = rng.sample(Uniform::new_inclusive(0, i));
+            let idx = rng.sample(Uniform::new_inclusive(0, i)?);
             assert_eq!(sl.remove(idx), v.remove(idx));
         }
+        Ok(())
     }
 
     #[test]
@@ -1065,10 +1104,10 @@ mod tests {
     #[test]
     fn last() {
         let mut sl = SkipList::new();
-        assert_eq!(sl.iter().rev().next(), None);
+        assert_eq!(sl.iter().next_back(), None);
         for i in 0..100 {
             sl.push_back(i);
-            assert_eq!(sl.iter().rev().next(), Some(&i));
+            assert_eq!(sl.iter().next_back(), Some(&i));
         }
     }
 
@@ -1273,6 +1312,17 @@ mod tests {
         }
     }
 
+    #[test]
+    fn index_of() {
+        let size = 200;
+        let sl: SkipList<_> = (0..size).collect();
+        for val in 0..size {
+            let i = sl.index_of(&val).expect("Index can't be None");
+            let item_at_index = sl.get(i).expect("Item can't be None");
+            assert_eq!(&val, item_at_index);
+        }
+    }
+
     #[test]
     fn bound() {
         let sl: SkipList<_> = (0..2).collect();
