@@ -1,13 +1,19 @@
 //! SkipMap stores key-value pairs, with the keys being unique and always
 //! sorted.
 
-use crate::level_generator::{GeometricalLevelGenerator, LevelGenerator};
-use crate::skipnode::{self, insertion_fixup, SkipListAction};
+#![allow(warnings)]
+#![allow(clippy)]
+#![allow(unknown_lints)]
+
 use std::{
     borrow::Borrow, cmp, cmp::Ordering, default, fmt, hash, hash::Hash, iter, mem, ops, ops::Bound,
 };
 
 pub use crate::skipnode::IntoIter;
+use crate::{
+    level_generator::{LevelGenerator, geometric::Geometric},
+    skipnode::{self, SkipListAction, insertion_fixup},
+};
 
 type SkipNode<K, V> = skipnode::SkipNode<(K, V)>;
 
@@ -51,7 +57,7 @@ pub struct SkipMap<K, V> {
     // Storage, this is not sorted
     head: Box<SkipNode<K, V>>,
     len: usize,
-    level_generator: GeometricalLevelGenerator,
+    level_generator: Geometric,
 }
 
 // ///////////////////////////////////////////////
@@ -73,7 +79,8 @@ where
     /// ```
     #[inline]
     pub fn new() -> Self {
-        let lg = GeometricalLevelGenerator::new(16, 1.0 / 2.0);
+        // Parameters are fixed and will produce a valid level generator.
+        let lg = Geometric::new(16, 1.0 / 2.0).expect("Failed to create level generator.");
         SkipMap {
             head: Box::new(SkipNode::head(lg.total())),
             len: 0,
@@ -97,7 +104,8 @@ where
     #[inline]
     pub fn with_capacity(capacity: usize) -> Self {
         let levels = cmp::max(1, (capacity as f64).log2().floor() as usize);
-        let lg = GeometricalLevelGenerator::new(levels, 1.0 / 2.0);
+        // Parameters are safe as levels >= 1 and p is in (0, 1).
+        let lg = Geometric::new(levels, 1.0 / 2.0).expect("Failed to create level generator.");
         SkipMap {
             head: Box::new(SkipNode::head(lg.total())),
             len: 0,
@@ -122,7 +130,7 @@ where
     pub fn insert(&mut self, key: K, value: V) -> Option<V> {
         let level_gen = &mut self.level_generator;
         let inserter = InsertOrReplace::new(key, value, |k, v| {
-            Box::new(SkipNode::new((k, v), level_gen.random()))
+            Box::new(SkipNode::new((k, v), level_gen.level()))
         });
         let insert_res = inserter.act(self.head.as_mut());
         match insert_res {
@@ -289,10 +297,10 @@ impl<K, V> SkipMap<K, V> {
     /// assert!(skipmap.get(&10).is_none());
     /// ```
     #[inline]
-    pub fn get<Q: ?Sized>(&self, key: &Q) -> Option<&V>
+    pub fn get<Q>(&self, key: &Q) -> Option<&V>
     where
         K: Borrow<Q>,
-        Q: Ord,
+        Q: Ord + ?Sized,
     {
         self.find_key(key).and_then(|node| node.value_ref())
     }
@@ -318,10 +326,10 @@ impl<K, V> SkipMap<K, V> {
     /// assert_eq!(skipmap.get(&0), Some(&100));
     /// ```
     #[inline]
-    pub fn get_mut<Q: ?Sized>(&mut self, key: &Q) -> Option<&mut V>
+    pub fn get_mut<Q>(&mut self, key: &Q) -> Option<&mut V>
     where
         K: Borrow<Q>,
-        Q: Ord,
+        Q: Ord + ?Sized,
     {
         self.find_key_mut(key).and_then(|node| node.value_mut())
     }
@@ -389,10 +397,10 @@ impl<K, V> SkipMap<K, V> {
     /// assert!(skipmap.contains_key(&4));
     /// assert!(!skipmap.contains_key(&15));
     /// ```
-    pub fn contains_key<Q: ?Sized>(&self, key: &Q) -> bool
+    pub fn contains_key<Q>(&self, key: &Q) -> bool
     where
         K: Borrow<Q>,
-        Q: Ord,
+        Q: Ord + ?Sized,
     {
         self.find_key(key).is_some()
     }
@@ -410,10 +418,10 @@ impl<K, V> SkipMap<K, V> {
     /// assert_eq!(skipmap.remove(&4), Some(4)); // Removes the last one
     /// assert!(skipmap.remove(&4).is_none());    // No more '4' left
     /// ```
-    pub fn remove<Q: ?Sized>(&mut self, key: &Q) -> Option<V>
+    pub fn remove<Q>(&mut self, key: &Q) -> Option<V>
     where
         K: Borrow<Q>,
-        Q: Ord,
+        Q: Ord + ?Sized,
     {
         let remover = Remover(key);
         match remover.act(self.head.as_mut()) {
@@ -701,10 +709,10 @@ impl<K: Ord, V> SkipMap<K, V> {
 
 impl<K, V> SkipMap<K, V> {
     /// Find the reference to the node equal to the given key.
-    fn find_key<Q: ?Sized>(&self, key: &Q) -> Option<&SkipNode<K, V>>
+    fn find_key<Q>(&self, key: &Q) -> Option<&SkipNode<K, V>>
     where
         K: Borrow<Q>,
-        Q: Ord,
+        Q: Ord + ?Sized,
     {
         let (last_le, _) = self.head.find_last_le_with(
             |(node_key, _), target| Ord::cmp(node_key.borrow(), target),
@@ -719,10 +727,10 @@ impl<K, V> SkipMap<K, V> {
     }
 
     /// Find the mutable reference to the node equal to the given key.
-    fn find_key_mut<Q: ?Sized>(&mut self, key: &Q) -> Option<&mut SkipNode<K, V>>
+    fn find_key_mut<Q>(&mut self, key: &Q) -> Option<&mut SkipNode<K, V>>
     where
         K: Borrow<Q>,
-        Q: Ord,
+        Q: Ord + ?Sized,
     {
         let (last_le, _) = self.head.find_last_le_with_mut(
             |(node_key, _), target| Ord::cmp(node_key.borrow(), target),
@@ -763,10 +771,11 @@ where
                 .collect();
 
             loop {
-                let value = if let (Some(k), Some(v)) = ((*node).key_ref(), (*node).value_ref()) {
-                    format!("> ({:?}, {:?})", k, v)
-                } else {
-                    "> ()".to_string()
+                let value = match ((*node).key_ref(), (*node).value_ref()) {
+                    (Some(k), Some(v)) => {
+                        format!("> ({:?}, {:?})", k, v)
+                    }
+                    _ => "> ()".to_string(),
                 };
 
                 let max_str_len = format!("{} -{}-", value, (*node).links_len[(*node).level]).len();
@@ -796,10 +805,13 @@ where
                     }
                 }
 
-                if let Some(next) = (*node).links[0].and_then(|p| p.as_ptr().as_ref()) {
-                    node = next;
-                } else {
-                    break;
+                match (*node).links[0].and_then(|p| p.as_ptr().as_ref()) {
+                    Some(next) => {
+                        node = next;
+                    }
+                    _ => {
+                        break;
+                    }
                 }
             }
 
@@ -861,18 +873,20 @@ where
     }
 
     unsafe fn act_on_node(self, node: &'a mut SkipNode<K, V>) -> Result<Self::Ok, Self::Err> {
-        let target_key = &self.key;
-        if let Some(target_node) = node.next_mut() {
-            if let Some(node_key) = target_node.key_ref() {
-                if target_key == node_key {
-                    let old_value = mem::replace(target_node.value_mut().unwrap(), self.value);
-                    return Err(old_value);
+        unsafe {
+            let target_key = &self.key;
+            if let Some(target_node) = node.next_mut() {
+                if let Some(node_key) = target_node.key_ref() {
+                    if target_key == node_key {
+                        let old_value = mem::replace(target_node.value_mut().unwrap(), self.value);
+                        return Err(old_value);
+                    }
                 }
             }
+            let new_node = (self.make_node)(self.key, self.value);
+            node.insert_next(new_node);
+            Ok(node.next_mut().unwrap())
         }
-        let new_node = (self.make_node)(self.key, self.value);
-        node.insert_next(new_node);
-        Ok(node.next_mut().unwrap())
     }
     fn fixup(
         level: usize,
@@ -910,16 +924,18 @@ impl<'a, Q: Ord + ?Sized, K: Borrow<Q>, V> SkipListAction<'a, (K, V)> for Remove
         self,
         target_parent: &'a mut SkipNode<K, V>,
     ) -> Result<Self::Ok, Self::Err> {
-        let node_key = target_parent
-            .next_mut()
-            .and_then(|node| node.key_ref())
-            .ok_or(())?
-            .borrow();
-        let target_key = self.0;
-        if node_key == target_key {
-            Ok(target_parent.take_next().unwrap())
-        } else {
-            Err(())
+        unsafe {
+            let node_key = target_parent
+                .next_mut()
+                .and_then(|node| node.key_ref())
+                .ok_or(())?
+                .borrow();
+            let target_key = self.0;
+            if node_key == target_key {
+                Ok(target_parent.take_next().unwrap())
+            } else {
+                Err(())
+            }
         }
     }
     fn fixup(
@@ -1027,7 +1043,7 @@ where
     }
 }
 
-impl<'a, K, V> ops::Index<usize> for SkipMap<K, V> {
+impl<K, V> ops::Index<usize> for SkipMap<K, V> {
     type Output = V;
 
     fn index(&self, index: usize) -> &V {
@@ -1037,7 +1053,7 @@ impl<'a, K, V> ops::Index<usize> for SkipMap<K, V> {
     }
 }
 
-impl<'a, K, V> ops::IndexMut<usize> for SkipMap<K, V> {
+impl<K, V> ops::IndexMut<usize> for SkipMap<K, V> {
     fn index_mut(&mut self, index: usize) -> &mut V {
         self.get_index_mut(index)
             .and_then(|node| node.value_mut())
@@ -1141,7 +1157,7 @@ pub struct Iter<'a, K: 'a, V: 'a>(skipnode::Iter<'a, (K, V)>);
 impl<'a, K: 'a, V: 'a> Iter<'a, K, V> {
     /// SAFETY: There must be `len` nodes after head.
     unsafe fn from_head(head: &'a SkipNode<K, V>, len: usize) -> Self {
-        Self(skipnode::Iter::from_head(head, len))
+        unsafe { Self(skipnode::Iter::from_head(head, len)) }
     }
 }
 
@@ -1167,7 +1183,7 @@ pub struct IterMut<'a, K: 'a, V: 'a>(skipnode::IterMut<'a, (K, V)>);
 impl<'a, K: 'a, V: 'a> IterMut<'a, K, V> {
     /// SAFETY: There must be `len` nodes after head.
     unsafe fn from_head(head: &'a mut SkipNode<K, V>, len: usize) -> Self {
-        Self(skipnode::IterMut::from_head(head, len))
+        unsafe { Self(skipnode::IterMut::from_head(head, len)) }
     }
 }
 
@@ -1231,9 +1247,10 @@ impl<'a, K, V: 'a> DoubleEndedIterator for Values<'a, K, V> {
 
 #[cfg(test)]
 mod tests {
-    use super::SkipMap;
     use std::ops::Bound::{self, Excluded, Included, Unbounded};
 
+    use super::SkipMap;
+
     #[test]
     fn basic_small() {
         let mut sm: SkipMap<i64, i64> = SkipMap::new();
