@@ -8,10 +8,10 @@ use crate::{
     grid::{
         colors::NoColors,
         config::{
-            AlignmentHorizontal, ColorMap, ColoredConfig, CompactConfig, Entity, Formatting,
-            Indent, Sides, SpannedConfig,
+            AlignmentHorizontal, ColorMap, ColoredConfig, CompactConfig, Entity, Indent, Sides,
+            SpannedConfig,
         },
-        dimension::{CompleteDimensionVecRecords, Dimension, Estimate, PeekableDimension},
+        dimension::{CompleteDimension, Dimension, Estimate, PeekableGridDimension},
         records::{
             vec_records::{Text, VecRecords},
             ExactRecords, Records,
@@ -87,7 +87,7 @@ use crate::{
 pub struct Table {
     records: VecRecords<Text<String>>,
     config: ColoredConfig,
-    dimension: CompleteDimensionVecRecords<'static>,
+    dimension: CompleteDimension,
 }
 
 impl Table {
@@ -99,8 +99,7 @@ impl Table {
     /// # Examples
     ///
     /// ```
-    /// use tabled::{Table, Tabled};
-    /// use testing_table::assert_table;
+    /// use tabled::{Table, Tabled, assert::assert_table};
     ///
     /// #[derive(Tabled)]
     /// struct Relationship {
@@ -126,11 +125,50 @@ impl Table {
     /// );
     /// ```
     ///
+    /// ## Don't hesitate to use iterators.
+    ///
+    /// ```
+    /// use tabled::{Table, Tabled, assert::assert_table};
+    ///
+    /// #[derive(Tabled)]
+    /// struct Relationship {
+    ///     person: String,
+    ///     love: bool
+    /// }
+    ///
+    /// let list = vec![
+    ///     Relationship { person: String::from("Clara"), love: true },
+    ///     Relationship { person: String::from("Greg"), love: false },
+    /// ];
+    ///
+    /// // Maybe don't love but don't hate :)
+    /// let iter = list.into_iter()
+    ///     .map(|mut rel| {
+    ///         if !rel.love {
+    ///             rel.love = true;
+    ///         }
+    ///
+    ///         rel
+    ///     });
+    ///
+    /// let table = Table::new(iter);
+    ///
+    /// assert_table!(
+    ///     table,
+    ///     "+--------+------+"
+    ///     "| person | love |"
+    ///     "+--------+------+"
+    ///     "| Clara  | true |"
+    ///     "+--------+------+"
+    ///     "| Greg   | true |"
+    ///     "+--------+------+"
+    /// );
+    /// ```
+    ///
     /// ## Notice that you can pass tuples.
     ///
     /// ```
-    /// use tabled::{Table, Tabled};
-    /// use testing_table::assert_table;
+    /// use tabled::{Table, Tabled, assert::assert_table};
     ///
     /// #[derive(Tabled)]
     /// struct Relationship {
@@ -156,37 +194,11 @@ impl Table {
     /// );
     /// ```
     ///
-    /// ## Notice that you can pass const arrays as well.
-    ///
-    /// ```
-    /// use tabled::Table;
-    /// use testing_table::assert_table;
-    ///
-    /// let list = vec![
-    ///     ["Kate", "+", "+", "+", "-"],
-    ///     ["", "-", "-", "-", "-"],
-    /// ];
-    ///
-    /// let table = Table::new(list);
-    ///
-    /// assert_table!(
-    ///     table,
-    ///     "+------+---+---+---+---+"
-    ///     "| 0    | 1 | 2 | 3 | 4 |"
-    ///     "+------+---+---+---+---+"
-    ///     "| Kate | + | + | + | - |"
-    ///     "+------+---+---+---+---+"
-    ///     "|      | - | - | - | - |"
-    ///     "+------+---+---+---+---+"
-    /// );
-    /// ```
-    ///
     /// ## As a different way to create a [`Table`], you can use [`Table::from_iter`].  
     ///
     /// ```
     /// use std::iter::FromIterator;
-    /// use tabled::Table;
-    /// use testing_table::assert_table;
+    /// use tabled::{Table, assert::assert_table};
     ///
     /// let list = vec![
     ///     vec!["Kate", "+", "+", "+", "-"],
@@ -230,11 +242,286 @@ impl Table {
         }
 
         let records = VecRecords::new(records);
+        let config = ColoredConfig::new(configure_grid());
+        let dimension = CompleteDimension::default();
 
         Self {
             records,
-            config: ColoredConfig::new(configure_grid()),
-            dimension: CompleteDimensionVecRecords::default(),
+            config,
+            dimension,
+        }
+    }
+
+    /// Creates a Table instance, from a list of [`Tabled`] values.
+    ///
+    /// It's an optimized version of [`Table::new`].
+    ///
+    /// ```
+    /// use tabled::{Table, Tabled, assert::assert_table};
+    ///
+    /// #[derive(Tabled)]
+    /// struct Relationship {
+    ///     person: String,
+    ///     love: bool
+    /// }
+    ///
+    /// let list = vec![
+    ///     Relationship { person: String::from("Clara"), love: true },
+    ///     Relationship { person: String::from("Greg"), love: false },
+    /// ];
+    ///
+    /// let table = Table::with_capacity(&list, list.len());
+    ///
+    /// assert_table!(
+    ///     table,
+    ///     "+--------+-------+"
+    ///     "| person | love  |"
+    ///     "+--------+-------+"
+    ///     "| Clara  | true  |"
+    ///     "+--------+-------+"
+    ///     "| Greg   | false |"
+    ///     "+--------+-------+"
+    /// );
+    /// ```
+    pub fn with_capacity<I, T>(iter: I, count_rows: usize) -> Self
+    where
+        I: IntoIterator<Item = T>,
+        T: Tabled,
+    {
+        let mut header = Vec::with_capacity(T::LENGTH);
+        for text in T::headers() {
+            let text = text.into_owned();
+            let cell = Text::new(text);
+            header.push(cell);
+        }
+
+        let mut records = Vec::with_capacity(count_rows + 1);
+        records.push(header);
+
+        for row in iter.into_iter() {
+            let mut list = Vec::with_capacity(T::LENGTH);
+            for text in row.fields().into_iter() {
+                let text = text.into_owned();
+                let cell = Text::new(text);
+
+                list.push(cell);
+            }
+
+            records.push(list);
+        }
+
+        let records = VecRecords::new(records);
+        let config = ColoredConfig::new(configure_grid());
+        let dimension = CompleteDimension::default();
+
+        Self {
+            records,
+            config,
+            dimension,
+        }
+    }
+
+    /// Creates a Table instance, from a list of [`Tabled`] values.
+    ///
+    /// Compared to [`Table::new`] it does not use a "header" (first line).
+    ///
+    /// If you use a reference iterator you'd better use [`FromIterator`] instead.
+    /// As it has a different lifetime constraints and make less copies therefore.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use tabled::{Table, Tabled, assert::assert_table};
+    ///
+    /// #[derive(Tabled)]
+    /// struct Relationship {
+    ///     love: bool
+    /// }
+    ///
+    /// let list = vec![
+    ///     ("Kate", Relationship { love: true }),
+    ///     ("", Relationship { love: false }),
+    /// ];
+    ///
+    /// let table = Table::nohead(list);
+    ///
+    /// assert_table!(
+    ///     table,
+    ///     "+------+-------+"
+    ///     "| Kate | true  |"
+    ///     "+------+-------+"
+    ///     "|      | false |"
+    ///     "+------+-------+"
+    /// );
+    /// ```
+    pub fn nohead<I, T>(iter: I) -> Self
+    where
+        I: IntoIterator<Item = T>,
+        T: Tabled,
+    {
+        let mut records = vec![];
+        for row in iter.into_iter() {
+            let mut list = Vec::with_capacity(T::LENGTH);
+            for text in row.fields().into_iter() {
+                let text = text.into_owned();
+                let cell = Text::new(text);
+
+                list.push(cell);
+            }
+
+            records.push(list);
+        }
+
+        let records = VecRecords::new(records);
+        let config = ColoredConfig::new(configure_grid());
+        let dimension = CompleteDimension::default();
+
+        Self {
+            records,
+            config,
+            dimension,
+        }
+    }
+
+    /// Creates a Key-Value [`Table`] instance, from a list of [`Tabled`] values.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use tabled::{Table, Tabled, assert::assert_table};
+    ///
+    /// #[derive(Tabled)]
+    /// #[tabled(rename_all = "PascalCase")]
+    /// struct Swim {
+    ///     event: String,
+    ///     time: String,
+    ///     #[tabled(rename = "Pool Length")]
+    ///     pool: u8,
+    /// }
+    ///
+    /// const POOL_25: u8 = 25;
+    /// const POOL_50: u8 = 50;
+    ///
+    /// let list = vec![
+    ///     Swim { event: String::from("Men 100 Freestyle"), time: String::from("47.77"), pool: POOL_25 },
+    ///     Swim { event: String::from("Men 400 Freestyle"), time: String::from("03:59.16"), pool: POOL_25 },
+    ///     Swim { event: String::from("Men 800 Freestyle"), time: String::from("08:06.70"), pool: POOL_25 },
+    ///     Swim { event: String::from("Men 4x100 Medley Relay"), time: String::from("03:27.28"), pool: POOL_50 },
+    /// ];
+    ///
+    /// let table = Table::kv(list);
+    ///
+    /// assert_table!(
+    ///     table,
+    ///     "+-------------+------------------------+"
+    ///     "| Event       | Men 100 Freestyle      |"
+    ///     "+-------------+------------------------+"
+    ///     "| Time        | 47.77                  |"
+    ///     "+-------------+------------------------+"
+    ///     "| Pool Length | 25                     |"
+    ///     "+-------------+------------------------+"
+    ///     "| Event       | Men 400 Freestyle      |"
+    ///     "+-------------+------------------------+"
+    ///     "| Time        | 03:59.16               |"
+    ///     "+-------------+------------------------+"
+    ///     "| Pool Length | 25                     |"
+    ///     "+-------------+------------------------+"
+    ///     "| Event       | Men 800 Freestyle      |"
+    ///     "+-------------+------------------------+"
+    ///     "| Time        | 08:06.70               |"
+    ///     "+-------------+------------------------+"
+    ///     "| Pool Length | 25                     |"
+    ///     "+-------------+------------------------+"
+    ///     "| Event       | Men 4x100 Medley Relay |"
+    ///     "+-------------+------------------------+"
+    ///     "| Time        | 03:27.28               |"
+    ///     "+-------------+------------------------+"
+    ///     "| Pool Length | 50                     |"
+    ///     "+-------------+------------------------+"
+    /// );
+    /// ```
+    ///
+    /// Next you'll find a more complex example with a subtle style.
+    ///
+    /// ```
+    /// use tabled::{Table, Tabled, settings::Style};
+    /// use tabled::settings::{style::HorizontalLine, Theme};
+    /// use tabled::assert::assert_table;
+    ///
+    /// #[derive(Tabled)]
+    /// #[tabled(rename_all = "PascalCase")]
+    /// struct Swim {
+    ///     event: String,
+    ///     time: String,
+    ///     #[tabled(rename = "Pool Length")]
+    ///     pool: u8,
+    /// }
+    ///
+    /// const POOL_25: u8 = 25;
+    /// const POOL_50: u8 = 50;
+    ///
+    /// let list = vec![
+    ///     Swim { event: String::from("Men 100 Freestyle"), time: String::from("47.77"), pool: POOL_25 },
+    ///     Swim { event: String::from("Men 400 Freestyle"), time: String::from("03:59.16"), pool: POOL_25 },
+    ///     Swim { event: String::from("Men 800 Freestyle"), time: String::from("08:06.70"), pool: POOL_25 },
+    ///     Swim { event: String::from("Men 4x100 Medley Relay"), time: String::from("03:27.28"), pool: POOL_50 },
+    /// ];
+    ///
+    /// let mut table = Table::kv(list);
+    ///
+    /// let mut style = Theme::from_style(Style::rounded().remove_horizontals());
+    /// for entry in 1 .. table.count_rows() / Swim::LENGTH {
+    ///     style.insert_horizontal_line(entry * Swim::LENGTH, HorizontalLine::inherit(Style::modern()));
+    /// }
+    ///
+    /// table.with(style);
+    ///
+    /// assert_table!(
+    ///     table,
+    ///     "╭─────────────┬────────────────────────╮"
+    ///     "│ Event       │ Men 100 Freestyle      │"
+    ///     "│ Time        │ 47.77                  │"
+    ///     "│ Pool Length │ 25                     │"
+    ///     "├─────────────┼────────────────────────┤"
+    ///     "│ Event       │ Men 400 Freestyle      │"
+    ///     "│ Time        │ 03:59.16               │"
+    ///     "│ Pool Length │ 25                     │"
+    ///     "├─────────────┼────────────────────────┤"
+    ///     "│ Event       │ Men 800 Freestyle      │"
+    ///     "│ Time        │ 08:06.70               │"
+    ///     "│ Pool Length │ 25                     │"
+    ///     "├─────────────┼────────────────────────┤"
+    ///     "│ Event       │ Men 4x100 Medley Relay │"
+    ///     "│ Time        │ 03:27.28               │"
+    ///     "│ Pool Length │ 50                     │"
+    ///     "╰─────────────┴────────────────────────╯"
+    /// );
+    /// ```
+    pub fn kv<I, T>(iter: I) -> Self
+    where
+        I: IntoIterator<Item = T>,
+        T: Tabled,
+    {
+        let headers = T::headers();
+
+        let mut records = Vec::new();
+        for row in iter.into_iter() {
+            for (text, name) in row.fields().into_iter().zip(headers.iter()) {
+                let key = Text::new(name.clone().into_owned());
+                let value = Text::new(text.into_owned());
+
+                records.push(vec![key, value]);
+            }
+        }
+
+        let records = VecRecords::new(records);
+        let config = ColoredConfig::new(configure_grid());
+        let dimension = CompleteDimension::default();
+
+        Self {
+            records,
+            config,
+            dimension,
         }
     }
 
@@ -292,7 +579,7 @@ impl Table {
         T: Tabled,
         I: IntoIterator<Item = T>,
     {
-        let mut builder = Builder::with_capacity(0, T::LENGTH);
+        let mut builder = Builder::with_capacity(1, T::LENGTH);
         builder.push_record(T::headers());
 
         for row in iter {
@@ -305,18 +592,37 @@ impl Table {
     /// It's a generic function which applies options to the [`Table`].
     ///
     /// It applies settings immediately.
+    ///
+    /// ```
+    /// use tabled::{Table, settings::Style};
+    /// use tabled::assert::assert_table;
+    ///
+    /// let data = vec![
+    ///     ("number", "name"),
+    ///     ("285-324-7322", "Rosalia"),
+    ///     ("564.549.6468", "Mary"),
+    /// ];
+    ///
+    /// let mut table = Table::new(data);
+    /// table.with(Style::markdown());
+    ///
+    /// assert_table!(
+    ///     table,
+    ///     "| &str         | &str    |"
+    ///     "|--------------|---------|"
+    ///     "| number       | name    |"
+    ///     "| 285-324-7322 | Rosalia |"
+    ///     "| 564.549.6468 | Mary    |"
+    /// );
+    /// ```
     pub fn with<O>(&mut self, option: O) -> &mut Self
     where
-        for<'a> O:
-            TableOption<VecRecords<Text<String>>, ColoredConfig, CompleteDimensionVecRecords<'a>>,
+        O: TableOption<VecRecords<Text<String>>, ColoredConfig, CompleteDimension>,
     {
         let reastimation_hint = option.hint_change();
-        let mut dims = self.dimension.from_origin();
 
-        option.change(&mut self.records, &mut self.config, &mut dims);
-
-        let (widths, heights) = dims.into_inner();
-        dimension_reastimate(&mut self.dimension, widths, heights, reastimation_hint);
+        option.change(&mut self.records, &mut self.config, &mut self.dimension);
+        self.dimension.reastimate(reastimation_hint);
 
         self
     }
@@ -326,6 +632,34 @@ impl Table {
     ///
     /// It applies settings immediately.
     ///
+    /// ```
+    /// use tabled::{Table, settings::{object::Columns, Alignment}};
+    /// use tabled::assert::assert_table;
+    ///
+    /// let data = vec![
+    ///     ("number", "name"),
+    ///     ("285-324-7322", "Rosalia"),
+    ///     ("564.549.6468", "Mary"),
+    /// ];
+    ///
+    /// let mut table = Table::new(data);
+    /// table.modify(Columns::first(), Alignment::right());
+    /// table.modify(Columns::one(1), Alignment::center());
+    ///
+    /// assert_table!(
+    ///     table,
+    ///     "+--------------+---------+"
+    ///     "|         &str |  &str   |"
+    ///     "+--------------+---------+"
+    ///     "|       number |  name   |"
+    ///     "+--------------+---------+"
+    ///     "| 285-324-7322 | Rosalia |"
+    ///     "+--------------+---------+"
+    ///     "| 564.549.6468 |  Mary   |"
+    ///     "+--------------+---------+"
+    /// );
+    /// ```
+    ///
     /// [`Cell`]: crate::settings::object::Cell
     /// [`Rows`]: crate::settings::object::Rows
     /// [`Location`]: crate::settings::location::Locator
@@ -339,8 +673,8 @@ impl Table {
             opt.change(&mut self.records, &mut self.config, entity);
         }
 
-        let reastimation_hint = option.hint_change();
-        dimension_reastimate_likely(&mut self.dimension, reastimation_hint);
+        let hint = option.hint_change();
+        self.dimension.reastimate(hint);
 
         self
     }
@@ -368,7 +702,7 @@ impl Table {
 
     /// Returns total widths of a table, including margin and horizontal lines.
     pub fn total_height(&self) -> usize {
-        let mut dims = CompleteDimensionVecRecords::from_origin(&self.dimension);
+        let mut dims = self.dimension.clone();
         dims.estimate(&self.records, self.config.as_ref());
 
         let total = (0..self.count_rows())
@@ -383,7 +717,7 @@ impl Table {
 
     /// Returns total widths of a table, including margin and vertical lines.
     pub fn total_width(&self) -> usize {
-        let mut dims = CompleteDimensionVecRecords::from_origin(&self.dimension);
+        let mut dims = self.dimension.clone();
         dims.estimate(&self.records, self.config.as_ref());
 
         let total = (0..self.count_columns())
@@ -415,6 +749,16 @@ impl Table {
     pub fn get_records_mut(&mut self) -> &mut VecRecords<Text<String>> {
         &mut self.records
     }
+
+    /// Returns a dimension.
+    pub fn get_dimension(&self) -> &CompleteDimension {
+        &self.dimension
+    }
+
+    /// Returns a dimension.
+    pub fn get_dimension_mut(&mut self) -> &mut CompleteDimension {
+        &mut self.dimension
+    }
 }
 
 impl Default for Table {
@@ -422,7 +766,7 @@ impl Default for Table {
         Self {
             records: VecRecords::default(),
             config: ColoredConfig::new(configure_grid()),
-            dimension: CompleteDimensionVecRecords::default(),
+            dimension: CompleteDimension::default(),
         }
     }
 }
@@ -442,7 +786,7 @@ impl fmt::Display for Table {
 
             print_grid(f, &self.records, &config, &dims, colors)
         } else {
-            let mut dims = PeekableDimension::default();
+            let mut dims = PeekableGridDimension::default();
             dims.estimate(&self.records, &config);
 
             print_grid(f, &self.records, &config, &dims, colors)
@@ -464,11 +808,13 @@ impl From<Builder> for Table {
     fn from(builder: Builder) -> Self {
         let data = builder.into();
         let records = VecRecords::new(data);
+        let config = ColoredConfig::new(configure_grid());
+        let dimension = CompleteDimension::default();
 
         Self {
             records,
-            config: ColoredConfig::new(configure_grid()),
-            dimension: CompleteDimensionVecRecords::default(),
+            config,
+            dimension,
         }
     }
 }
@@ -536,7 +882,9 @@ fn configure_grid() -> SpannedConfig {
         ),
     );
     cfg.set_alignment_horizontal(Entity::Global, AlignmentHorizontal::Left);
-    cfg.set_formatting(Entity::Global, Formatting::new(false, false, false));
+    cfg.set_line_alignment(Entity::Global, false);
+    cfg.set_trim_horizontal(Entity::Global, false);
+    cfg.set_trim_vertical(Entity::Global, false);
     cfg.set_borders(Style::ascii().get_borders());
 
     cfg
@@ -613,82 +961,3 @@ fn print_grid<F: fmt::Write, D: Dimension>(
         PeekableGrid::new(records, cfg, &dims, NoColors).build(f)
     }
 }
-
-fn dimension_reastimate(
-    dims: &mut CompleteDimensionVecRecords<'_>,
-    widths: Option<Vec<usize>>,
-    heights: Option<Vec<usize>>,
-    hint: Option<Entity>,
-) {
-    let hint = match hint {
-        Some(hint) => hint,
-        None => return,
-    };
-
-    match hint {
-        Entity::Global | Entity::Cell(_, _) => {
-            dims_set_widths(dims, widths);
-            dims_set_heights(dims, heights);
-        }
-        Entity::Column(_) => {
-            dims_set_widths(dims, widths);
-        }
-        Entity::Row(_) => {
-            dims_set_heights(dims, heights);
-        }
-    }
-}
-
-fn dims_set_widths(dims: &mut CompleteDimensionVecRecords<'_>, list: Option<Vec<usize>>) {
-    match list {
-        Some(list) => match dims.get_widths() {
-            Some(widths) => {
-                if widths == list {
-                    dims.clear_width();
-                } else {
-                    dims.set_widths(list);
-                }
-            }
-            None => dims.set_widths(list),
-        },
-        None => {
-            dims.clear_width();
-        }
-    }
-}
-
-fn dims_set_heights(dims: &mut CompleteDimensionVecRecords<'_>, list: Option<Vec<usize>>) {
-    match list {
-        Some(list) => match dims.get_heights() {
-            Some(heights) => {
-                if heights == list {
-                    dims.clear_height();
-                } else {
-                    dims.set_heights(list);
-                }
-            }
-            None => dims.set_heights(list),
-        },
-        None => {
-            dims.clear_height();
-        }
-    }
-}
-
-fn dimension_reastimate_likely(dims: &mut CompleteDimensionVecRecords<'_>, hint: Option<Entity>) {
-    let hint = match hint {
-        Some(hint) => hint,
-        None => return,
-    };
-
-    match hint {
-        Entity::Global | Entity::Cell(_, _) => {
-            dims.clear_width();
-            dims.clear_height()
-        }
-        Entity::Column(_) => {
-            dims.clear_width();
-        }
-        Entity::Row(_) => dims.clear_height(),
-    }
-}
