@@ -70,7 +70,7 @@ fn words_regex() -> &'static Regex {
         Regex::new(
             r"(?x)
              (_*)
-             ([\w'’.:/@\[\]/()&]+)
+             ([\w'’.:/@\[\]/()&\-]+)
              (_*)",
         )
         .expect("unable to compile words regex")
@@ -160,8 +160,16 @@ fn process_word(word: &str) -> Cow<'_, str> {
                 .join("/"),
         )
     } else if is_acronym(word) {
-        // Preserve caps like (BBC) or (DVD)
+        // Preserve caps like (BBC) or (HELLO-WORLD)
         Cow::from(word)
+    } else if has_internal_hyphens(word) {
+        Cow::from(
+            word.split('-')
+                .map(|word| titlecase_internal(word, true))
+                // TODO: Awaiting rust iter.intersperse('-');
+                .collect::<Vec<String>>()
+                .join("-"),
+        )
     } else if starts_with_bracket(word) {
         let rest = titlecase(&word[1..]);
         Cow::from(format!("({}", rest))
@@ -207,14 +215,14 @@ fn is_acronym_regex() -> &'static Regex {
         Regex::new(
             r"(?x)
             \A
-            \(+[A-Z0-9]+\)+
+            \(+[A-Z0-9]+(?:-[A-Z0-9]+)*\)+
             \z",
         )
-        .expect("")
+        .expect("unable to compile acronym regex")
     })
 }
 
-// E.g. (BBC) or (DVD)
+// E.g. (BBC) or (HELLO-WORLD)
 fn is_acronym(word: &str) -> bool {
     // Check if the number of open and closed braces is equal
     word.chars().fold(0, |acc, char| match char {
@@ -231,13 +239,22 @@ fn has_internal_caps(word: &str) -> bool {
 }
 
 fn has_internal_slashes(word: &str) -> bool {
-    !word.is_empty() && word.chars().skip(1).any(|chr| chr == '/')
+    second_letter_onwards(word).contains('/')
+}
+
+fn has_internal_hyphens(word: &str) -> bool {
+    second_letter_onwards(word).contains('-')
 }
 
 fn starts_with_bracket(word: &str) -> bool {
     word.starts_with('(')
 }
 
+fn second_letter_onwards(word: &str) -> &str {
+    let offset = ceil_char_boundary(word, 1);
+    &word[offset..]
+}
+
 #[inline]
 fn fix_small_word_at_start_regex() -> &'static Regex {
     static RE: OnceLock<Regex> = OnceLock::new();
@@ -287,9 +304,31 @@ fn fix_small_word_at_end(text: &str) -> Cow<'_, str> {
     })
 }
 
+// TODO: use str implementation when stable
+// https://github.com/rust-lang/rust/issues/93743
+#[inline]
+pub fn ceil_char_boundary(s: &str, index: usize) -> usize {
+    if index > s.len() {
+        s.len()
+    } else {
+        let upper_bound = Ord::min(index + 4, s.len());
+
+        s.as_bytes()[index..upper_bound]
+            .iter()
+            .position(|b| is_utf8_char_boundary(*b))
+            .map_or(upper_bound, |pos| pos + index)
+    }
+}
+
+#[inline]
+pub(crate) const fn is_utf8_char_boundary(b: u8) -> bool {
+    // This is bit magic equivalent to: b < 128 || b >= 192
+    (b as i8) >= -0x40
+}
+
 #[cfg(test)]
 mod tests {
-    use super::{titlecase, Titlecase};
+    use super::{second_letter_onwards, titlecase, Titlecase};
 
     macro_rules! testcase {
         ($name:ident, $input:expr, $expected:expr) => {
@@ -304,7 +343,7 @@ mod tests {
     testcase!(
         email,
         "For step-by-step directions email someone@gmail.com",
-        "For Step-by-Step Directions Email someone@gmail.com"
+        "For Step-By-Step Directions Email someone@gmail.com"
     );
 
     testcase!(
@@ -355,6 +394,12 @@ mod tests {
         "(ABC) ((ABC)) (Abc ABC) ((Abc) (Abc)) (Abc) (Abc) (aBC) (aBc) (ABC)/(ABC) (ABC)/Abc ABC"
     );
 
+    testcase!(
+        acronym_hyphen,
+        "(HELLO-WORLD) (HELLO-WORLD-DASH) (HELLO-WORLD-01) (HELLO-world) (HELLO-WORLD-)",
+        "(HELLO-WORLD) (HELLO-WORLD-DASH) (HELLO-WORLD-01) (Hello-World) (Hello-WORLD-)"
+    );
+
     testcase!(iphone, "BlackBerry vs. iPhone", "BlackBerry vs. iPhone");
 
     testcase!(
@@ -475,23 +520,22 @@ mod tests {
         "Never Touch Paths Like /var/run Before/After /boot"
     );
 
-    // TODO: Implement these
-    // testcase!(
-    //     in_flight,
-    //     "The in-flight entertainment was excellent",
-    //     "The In-Flight Entertainment Was Excellent"
-    // );
+    testcase!(
+        in_flight,
+        "The in-flight entertainment was excellent",
+        "The In-Flight Entertainment Was Excellent"
+    );
 
-    // testcase!(
-    //     stand_in,
-    //     "The Stand-in teacher gave us homework",
-    //     "The Stand-In Teacher Gave Us Homework"
-    // );
+    testcase!(
+        stand_in,
+        "The Stand-in teacher gave us homework",
+        "The Stand-In Teacher Gave Us Homework"
+    );
 
     testcase!(
         man_in_the_middle,
         "They executed a man-in-the-middle attack",
-        "They Executed a Man-in-the-Middle Attack"
+        "They Executed a Man-In-The-Middle Attack"
     );
 
     testcase!(
@@ -513,4 +557,18 @@ mod tests {
         "発売時の名称は「apple macintosh」であったが、後に拡張版のMacintosh 512Kが発売された段階で「macintosh 128K」と再命名された。test",
         "発売時の名称は「Apple Macintosh」であったが、後に拡張版のMacintosh 512Kが発売された段階で「Macintosh 128K」と再命名された。Test"
     );
+
+    testcase!(multibyte_hyphen, "👽-green-alien", "👽-Green-Alien");
+
+    #[test]
+    fn test_second_letter_onwards() {
+        assert_eq!(second_letter_onwards(""), "");
+        assert_eq!(second_letter_onwards("a"), "");
+        assert_eq!(second_letter_onwards("ab"), "b");
+        assert_eq!(
+            second_letter_onwards("「Apple Macintosh」"),
+            "Apple Macintosh」"
+        );
+        assert_eq!(second_letter_onwards("⌚"), "");
+    }
 }
